;$Id: kawcrmirrorprint.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;     KAWCRMIRRORPRINT
;
; PURPOSE:
;     Create an ascii file containing times of radar mirror position changes
;     for a given flight
;
; CATEGORY:
;     wcrtools
;
; CALLING SEQUENCE:
;     KAWCRMIRRORPRINT, Ncf
;
; INPUTS:
;     Ncf:  KA processed data netcdf file name
;
; KEYWORD PARAMETERS:
;     HELP: show this text
;
; OUTPUTS:
;
; SIDE EFFECTS:
;     Create file <KAyyyymmdd.hhmmss_hhmmss.mirror> 
;
;-
; MODIFICATION HISTORY:
;     Written by:  Samuel Haimov, August 1999
;     S.H., May 2000 -- added leap seconds print
;     S.H., Apr 2003 -- changed kamirror to kawcrmirror and kamirrorprint
;                       to kawcrmirrorprint
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro kawcrmirrorprint,ncf,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'kawcrmirrorprint'
  return
endif

if n_params() eq 0 then begin
   message,'Usage: kawcrmirrorprint,ncf',/info
   return
endif

; Get mirror positions, mirror time switch and flight date

  ncid=ncf
  mir=kawcrmirror(ncid,/time,date=fdate,leaps=leaps)
  n=n_elements(mir[*,0])

; Create output file string array

  mirstr=strarr(4,n)
  mirstr[0,*]=num2str(indgen(n)+1)
  mirstr[1,*]=time2timestr(mir[*,0])
  mirstr[2,*]=time2timestr(mir[*,1])
  ind1=where(mir[*,2] eq 1.)
  ind2=where(mir[*,2] eq 0.)
  if ind1[0] ne -1 then mirstr[3,ind1]='up-looking'
  if ind2[0] ne -1 then mirstr[3,ind2]='side-looking'

; Create the output file and print the string array

  get_lun,lun1
  openw,lun1,'KA'+fdate+'.'+num2str(mir[0,0],fr=0)+'_'+ $
              num2str(mir[n-1,1],fr=0)+'.mirror'
  printf,lun1,'Proc. Date/Time: ',systime()
  printf,lun1,'Flight Date: ',fdate
  printf,lun1,''
  printf,lun1,'Times are UTC; GPS leap seconds='+num2str(leaps,fr=0)
  printf,lun1,'',format='(/A)
  printf,lun1,'WCR Mirror Position for KA data file: '+ncf
  printf,lun1,''
  printf,lun1,format='(A3,2X,A8,3X,A8,3X,A12)',mirstr

; Close the input and output files

  free_lun,lun1
  ncdf_close,ncid

end
