;$Id: plotwcrcocr.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRCOCR
;
; PURPOSE:
;       This program uses calibration data already in the common block to plot
;       hh/vv, vh/hh, hv/vv. This program is normally used by running 
;       plotwcrtestm, plotwcrcalm, plotwcrcalm1 or plotwcrcrm.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRCOCR
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       SMOOTH:   number of smoothing points (0, default)
;       ZOOM:     input array [ readmode, str, stp ]
;                 readmode: 0  - str/stp, start/stop record numbers
;                           1  - str/stp, start/stop profile numbers
;                           2  - str/stp, start/stop UTC in hhmmss
;       INDJUMP:  input string
;                 'all'  - uses even and odd profiles (default)
;                 'even' - uses even profiles only
;                 'odd'  - uses odd profiles only
;       RANGE:    input array [begin_rg, end_rg], (default [3,rangegates-1])
;       PPMAG:    when set works for ppmag files only
;       PLOTFLAG: output status, 0 - ok
;                                1 - no plots produced 
;       HELP:     show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       three plots
;       This routine works with PPMAG and SPPMAG.  In plotwcrnoise.pro vh/hh or
;       hv/hh is plotted for sppmag.  Thus to avoid doublication when both,
;       plotwcrnoise and plotwcrcocr are used, set the ppmag keyword
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1997
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Jan 00 -- modified number of plot points for the smoothed curves
;                       added help keyword
;       S.H., Feb 01 -- changed routine name from plotcocr to plotwcrcocr
;       S.H., Jun 02 -- fixed cross/co ratio plot in rare event of negative data
;                       points only (noise higher than the range gates)
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrcocr,SMOOTH=smo, ZOOM=zm, INDJUMP=ijump, RANGE=rgzm, PPMAG=ppmag, $
                PLOTFLAG=pflag,help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrcocr'
  return
endif

pflag    =1

if KEYWORD_SET(ppmag) and alg ne pp4alg and alg ne pp6alg then return

if n_elements(ijump) eq 0 then ijump='all'
if (KEYWORD_SET(zm) eq 0) then zm=[0, 0, n_elements(prof)-1]
if (n_elements(smo) eq 0) then sm=-1 else sm=smo
if sm gt 0 and sm lt 3 then sm=0

if n_elements(rgzm) eq 0 then begin
  ind2   =3                 ; don't use first gates because of leakage
  ind3   =rangegates-1      ; last range gate to use
endif else if n_elements(rgzm) eq 1 then begin
  ind2   =rgzm
  ind3   =rangegates-1
endif else begin
  ind2   =rgzm(0)
  ind3   =rgzm(1)
endelse

;minsig   =1.58489e-6        ; -58 dBm minimum signal for cross ch. monitor
minsig   =3.1623e-5         ; -45 dBm minimum signal for hv ch. monitor
;gi      =10.^(0.3)         ; approx. minimum gain inbalance, vv/hh= 3 dB
gi       =10.^(-0.3)        ; approx. minimum gain inbalance, vv/hh=-3 dB
maxsm    =101               ; maximum smoothing
minsm    =3                 ; minimum smoothing
maxp     =5000              ; max points for scatter plots

pcharsize=1.7
!p.charsize=pcharsize

wcrzoomprof, zm,ind,btime,etime

if strlowcase(ijump) eq 'even' then $
   ind=ind(lindgen(n_elements(ind)/2)*2) $
else if strlowcase(ijump) eq 'odd' then $
   ind=ind(lindgen((n_elements(ind)-1)/2)*2+1) $
else ijump='all'

nprof=n_elements(prof(ind))-1
if (nprof le 2) then sm=0
if (nprof lt 2*sm+2) then begin
   print, wcrfile,': ',strtrim(string(nprof+1),2), $
         ' profiles to process !  Skipping the file. '
   return
endif

; Average all noise pulses

if size(hhnoise, /n_dim) gt 1 then hhn=mean(hhnoise,1) else hhn=hhnoise
if size(hhnoise, /n_dim) eq 3 then hhn=mean(hhn,1)

if size(vvnoise, /n_dim) gt 1 then vvn=mean(vvnoise,1) else vvn=vvnoise
if size(vvnoise, /n_dim) eq 3 then vvn=mean(vvn,1)

mnhhn    = mean(hhn(ind))
mnvvn    = mean(vvn(ind))

; Average all same pol pulses 

if (alg eq pp4alg or alg eq pp6alg) then begin
   if size(hh,/n_dim) eq 3 then hht=mean(hh,2) else hht=hh
   if size(vv,/n_dim) eq 3 then vvt=mean(vv,2) else vvt=vv
   if size(hv,/n_dim) eq 3 then hhl=mean(hv,2) else hhl=hv
   if size(vh,/n_dim) eq 3 then vvl=mean(vh,2) else vvl=vh
endif else if alg eq sppalg then $ 
   if txsequence(0) eq polH then begin
      if size(hh,/n_dim) eq 3 then hht=mean(hh,2) else hht=hh
      if size(vh,/n_dim) eq 3 then vvl=mean(vh,2) else vvl=vh
   endif else begin
      if size(vv,/n_dim) eq 3 then vvt=mean(vv,2) else vvt=vv
      if size(hv,/n_dim) eq 3 then hhl=mean(hv,2) else hhl=hv
   endelse

; Plot hh/vv

if (alg eq pp4alg or alg eq pp6alg) then begin
  ind1=where(hht(ind2:ind3,ind) gt minsig)
  nind1=n_elements(ind1)
  if nind1 lt 3 then nextplot else begin
     temp=(hht[ind2:ind3,ind])[ind1] 
     zdr =(temp-mnhhn)/((vvt[ind2:ind3,ind])[ind1]-mnvvn)
     if sm eq -1 then sm=(0.05*nind1<maxsm)>minsm 
     if sm gt nind1 then sm=0
     if nind1 gt 2*maxp then begin
        step=nind1/maxp
        ind1=lindgen(nind1/step)*step
     endif else ind1=lindgen(nind1)

     plot,10.*alog10(temp(ind1)),10.*alog10(zdr(ind1)),/nodata,$
        xtitle='HH power [dBm]',ytitle='HH/VV [dBm]',$
        title=ijump+' points ('+num2str(nind1)+'/'+num2str(n_elements(ind1))+$
        ' total/plotted), smooth='+num2str(sm,fr=0)+spc(25),psym=3,$
        ticklen=1,xgridstyle=1,ygridstyle=1,xmargin=[8,8],ymargin=[4,4]
     oplot,10.*alog10(temp(ind1)),10.*alog10(zdr(ind1)),psym=3,color=128    

     if (sm gt 0 and nind1 gt 2*sm) then begin
        isort=sort(temp) & temp=temp(isort)
        zdr=smooth(zdr(isort),sm)
        ind1=lindgen(n_elements(zdr)-2*sm)
        nind1=n_elements(ind1)
        if nind1 gt maxp then begin
           step=nind1/maxp
           ind1=lindgen(nind1/step)*step+sm
        endif else ind1=lindgen(nind1)+sm
        oplot,10.*alog10(temp(ind1)),10.*alog10(zdr(ind1)),thick=3,psym=3
     endif
     pflag=0
  endelse
endif

; Plot vh/hh

if (txsequence(0) eq polH and alg eq sppAlg) or alg eq pp4alg or alg eq pp6alg $
then begin
   ind1=where(vvl(ind2:ind3,ind) gt minsig*gi)
   nind1=n_elements(ind1)
   if nind1 lt 3 then nextplot else begin
      temp=(hht[ind2:ind3,ind])[ind1] 
      ldr =((vvl[ind2:ind3,ind])[ind1]-mnvvn)/(temp-mnhhn)
      if sm eq -1 then sm=(0.05*nind1<maxsm)>minsm 
      if sm gt nind1 then sm=0
      if nind1 gt 2*maxp then begin
         step=nind1/maxp
         ind1=lindgen(nind1/step)*step
      endif else ind1=lindgen(nind1)
      if n_elements(where(ldr[ind1] gt 0.)) lt 3 then nextplot $
      else begin
        plot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),/nodata,$
          xmargin=[8,8],ymargin=[4,4],xtitle='HH power [dBm]', $
          title=ijump+' points ('+num2str(nind1)+'/'+num2str(n_elements(ind1))+$
          ' total/plotted), smooth='+num2str(sm,fr=0)+spc(25),$
          xgridstyle=1,ygridstyle=1,ticklen=1,ytitle='VH/HH [dBm]'
        oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),psym=3,color=128    

        if (sm gt 0 and nind1 gt 3*sm) then begin
          isort=sort(temp) & temp=temp(isort)
          ldr=smooth(ldr(isort),sm)
          ind1=lindgen(n_elements(ldr)-3*sm)
          nind1=n_elements(ind1)
          if nind1 gt maxp then begin
            step=nind1/maxp
            ind1=lindgen(nind1/step)*step+2*sm
          endif else ind1=lindgen(nind1)+2*sm
          oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),thick=3,psym=3
        endif
        pflag=0
     endelse
   endelse
endif  

; Plot hv/vv

if (txsequence(0) eq polV and alg eq sppAlg) or alg eq pp4alg or alg eq pp6alg $
then begin
   ind1=where(hhl(ind2:rangegates-1,ind) gt minsig)
   nind1=n_elements(ind1)
   if nind1 lt 3 then nextplot else begin
      temp=(vvt[ind2:ind3,ind])[ind1] 
      ldr =((hhl[ind2:ind3,ind])[ind1]-mean(hhn))/(temp-mean(vvn))
      if sm eq -1 then sm=(0.05*nind1<maxsm)>minsm 
      if sm gt nind1 then sm=0
      if nind1 gt 2*maxp then begin
         step=nind1/maxp
         ind1=lindgen(nind1/step)*step
      endif else ind1=lindgen(nind1)

      plot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),/nodata,$
         xmargin=[8,8],ymargin=[4,4],xtitle='VV power [dBm]', $
         title=ijump+' points ('+num2str(nind1)+'/'+num2str(n_elements(ind1))+$
         ' total/plotted), smooth='+num2str(sm,fr=0)+spc(25),$
         xgridstyle=1,ygridstyle=1,ticklen=1,ytitle='HV/VV [dBm]'
      oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),psym=3,color=128    
           
      if (sm gt 0 and nind1 gt 3*sm) then begin
         isort=sort(temp) & temp=temp(isort)
         ldr=smooth(ldr(isort),sm)
         ind1=lindgen(n_elements(ldr)-3*sm)
         nind1=n_elements(ind1)
         if nind1 gt maxp then begin
            step=nind1/maxp
            ind1=lindgen(nind1/step)*step+2*sm
         endif else ind1=lindgen(nind1)+2*sm
         oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),thick=3,psym=3
      endif
      pflag=0
   endelse 
endif

end
