;$Id: plotwcrfftm.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRFFTM
;
; PURPOSE:
;       Plots fft data for test purposes of a fft wcr file (the file is 
;       processed w/o segmentation; use plotwcrtestm for large files)
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRFFTM
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;      NOSAVEPS: when set does not save the ps files(ignored if plotps not set) 
;        PLOTPS: when set creates a ps file,(default is x device)
;       PRINTER: string for the printer, (default, 'lj5local')
;                printer='' will supress printing
;       READVXI: when set reads data from common block otherwise from a file
;                (default, 0)
;       RGATE:   input integer, when given plots corresponding range gate
;                default is 10 (0 is the first range gate) 
;                (default [120,350])
;       SETXWIN: set ploting window (default, 0; ignored for ps device)
;       SMOOTH:  number of smoothing points (0, default)
;       TIMECOR: if defined gives UTC time correction of the wcr time
;                After June 1998, IRIG card is used to set the time.
;                Normal use of the card is to get the time from a master
;                IRIG card with GPS time.  The default value is set to
;                the leap seconds in openwcrf. 
;        TLABEL: string for graphics title (default, 'Experiment Data')
;    BIG_ENDIAN: when set, assumes that the input file data are in big endian
;                order.  The default is LITTLE_ENDIAN.
;        HELP:   show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       two plots
;
; EXAMPLES:
;       Corner Reflector calibration plot
;
;       .run openwcrm
;       readwcrdata
;       tlabel  = "Coastal II '99 Data"
;       plotwcrfftm,rgate=15,tlabel=tlabel,/readwcr,setx=1
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Jan 1996
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., 11/99  -- partially restored int.cal. vars in wcrdata common 
;                       block
;       S.H., Jan 00 -- added help keyword
;       S.H., Nov 00 -- modified to accommodate the new plots in plotfftcal
;       S.H., Jan 01 -- added little_endian keyword
;       S.H., Feb 01 -- changed routine name from plotfftcalm to plotwcrfftm
;       S.H., Apr 02 -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrfftm, smooth=sm,plotps=ps,setxwin=xwin,timecor=timecor,printer=pr, $
                 readwcr=readwcr,tlabel=tlabel,rgate=rgate,nosaveps=saveps,$
                 big_endian=bgnd,help=help

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrfftm'
  return
endif

if (KEYWORD_SET(sm) eq 0) then sm = 0        ; smooth
if n_elements(rgate) eq 0 then rgate=10

if KEYWORD_SET(ps) then ps = 1 else ps = 0
if (KEYWORD_SET(xwin) eq 0) then xwin=0
if keyword_set(saveps) then saveps=0 else saveps=1

; readwcr : 0 - read wcr file, 1 - use common block data    
if (KEYWORD_SET(readwcr) eq 0) then readwcr=0 

if (n_elements(pr) eq 0) then pr='lj5local'
if (KEYWORD_SET(tlabel) eq 0) then tlabel='Experiment Data'

if (readwcr eq 0) then fn=dialog_pickfile(/read)

if ps then winn=99 $ 
else winn=xwin

if (readwcr eq 0) then begin
   openwcrf,fn,timecor=timecor,big_endian=bgnd
   str = 0
   stp = tnumrec-1
   if (alg eq fftalg) then readwcrdata,str,stp,big_endian=bgnd $
   else begin
      print, 'Skip '+wcrfile+'. Not a FFT file.'
      return
   endelse
endif

plotwcrfft,rgate=rgate,tlabel=tlabel,plotps=winn
   
if ps then begin
   resetps2x
   if (pr ne '') then spawn,'lpr -h -P'+pr+' idl.ps'
   if saveps then $
      spawn,'mv idl.ps '+wcrfile+'.tstfft'+ $
      num2str(sec2time(timestr2sec(begtime)),fr=0)+'.ps' $
   else spawn,'rm idl.ps'
endif

end
