;$Id: plotwcrnoise.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PLOTWCRNOISE
;
; PURPOSE:
;       This program uses calibration data already in the common block to plot
;       noise and Tx leakage channels and calculate correlation coeffs.
;       This program is normally used by running plotwcrcalm,  
;       plotwcrcalm1 or plotwcrcrm.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PLOTWCRNOISE
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;       SMOOTH:  number of smoothing points (0, default)
;       ZOOM:    input array [ readmode, str, stp ]
;                readmode: 0  - str/stp, start/stop record numbers
;                          1  - str/stp, start/stop profile numbers
;                          2  - str/stp, start/stop UTC in hhmmss
;       INDJUMP: input string
;                'all'  - uses even and odd profiles (default)
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;      HELP:  Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams, wcrdata 
;
; SIDE EFFECTS:
;       create two plots using the current device (ususally x or ps)
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Sep 1995
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Dec 99 -- complied with the new pp arrays
;       S.H., Jan 00 -- added help keyword
;       S.H., Feb 01 -- chenged routine name from plotcal to plotwcrnoise
;       S.H., Oct 01 - replaced atan(complex) with atan(imaginary(complex),
;                                                       float(complex))
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro plotwcrnoise,smooth=smo, zoom=zm, indjump=ijump, help=help

common wcrparams
common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'plotwcrnoise'
  return
endif

if n_elements(ijump) eq 0 then ijump='all'
if (KEYWORD_SET(zm) eq 0) then zm=[0, 0, n_elements(prof)-1]
if (KEYWORD_SET(smo) eq 0) then sm=0 else sm=smo
if (sm lt 3) then sm=0

intcal=1                  ; don't plot Tx leakage (1 to plot)
;minsig =1.58489e-6       ; -58 dBm minimum signal for cross ch. monitor
minsig  =3.1623e-5       ; -45 dBm minimum signal for hv ch. monitor

wcrzoomprof, zm,ind,btime,etime

if strlowcase(ijump) eq 'even' then $
   ind=ind(lindgen(n_elements(ind)/2)*2) $
else if strlowcase(ijump) eq 'odd' then $
   ind=ind(lindgen((n_elements(ind)-1)/2)*2+1) $
else ijump='all'

nprof=n_elements(prof(ind))-1
if (nprof le 2) then sm=0
if (nprof lt 2*sm+2) then begin
   print, wcrfile,': ',strtrim(string(nprof+1),2), $
         ' profiles to process !  Skipping the file. '
   return
endif

if (sm gt 0) then begin
   nprof=nprof-sm
   ind=ind(sm:nprof)
   nprof=nprof-sm
;   Correct etime and btime for the shift due to smoothing
;   tcorr=round(sm*timeInterval)
;   btime=time2sec(long(strmid(btime,0,2)+strmid(btime,3,2)+strmid(btime,6,2)))
;   btime=num2str(long(sec2time(btime+tcorr)))
;   btime=strmid(btime,1,2)+':'+strmid(btime,3,2)+':'+strmid(btime,5,2)

;   etime=time2sec(long(strmid(etime,0,2)+strmid(etime,3,2)+strmid(etime,6,2)))
;   etime=num2str(long(sec2time(etime-tcorr)))
;   etime=strmid(etime,1,2)+':'+strmid(etime,3,2)+':'+strmid(etime,5,2)
endif
  
xtoplabel= 'Profile'
sp40 = '                                        '
xbotlabel= bTime+sp40+'Time (sec)'+sp40+eTime
pcharsize=1.7
!p.charsize=pcharsize
print,'**********************************'
print,wcrfile
print,' '

   if (alg eq pp4alg or alg eq pp6alg ) then begin

      if (sm eq 0) then begin
         if intcal then begin
            vt=vvtx
            vl=vvlk
            hl=hhlk
            ht=hhtx
         endif
         hhn=hhnoise
         vvn=vvnoise 
      endif else begin
         if intcal then begin
            vt=smooth(vvtx,sm)
            vl=smooth(vvlk,sm)
            hl=smooth(hhlk,sm)
            ht=smooth(hhtx,sm)
         endif
         hhn=smooth(hhnoise,sm)
         vvn=smooth(vvnoise,sm)
      endelse

   ; Calculate corr. coeff. for non-smoothed profiles

      if intcal then begin
         Chtvl=strmid(strtrim(string(round(100*correlate(ht(ind), $
               vl(ind)))/100.),1),0,4)
         Chtvt=strmid(strtrim(string(round(100*correlate(ht(ind), $
               vt(ind)))/100.),1),0,4)
         Cvthl=strmid(strtrim(string(round(100*correlate(vt(ind), $
               hl(ind)))/100.),1),0,4)
         Chthl=strmid(strtrim(string(round(100*correlate(ht(ind), $
               hl(ind)))/100.),1),0,4)
         Cvtvl=strmid(strtrim(string(round(100*correlate(vt(ind), $
               vl(ind)))/100.),1),0,4)
      endif

      Cvnhn=strmid(strtrim(string(round(100*correlate(vvnoise(ind), $
            hhnoise(ind)))/100.),1),0,4)

   ; Caluclate the average jumps between "ping" and "pong" DSP 

      ind1=lindgen(nprof/2)*2
      ind2=ind1+1

      if intcal then begin
         htdsp=hhtx(ind2)/hhtx(ind1)
         htdsp=num2str(round(1000.*alog10([mean(htdsp),std(htdsp)]))/100.,len=5)
         htdsp=htdsp(0)+','+htdsp(1)
         vtdsp=vvtx(ind2)/vvtx(ind1)
         vtdsp=num2str(round(1000.*alog10([mean(vtdsp),std(vtdsp)]))/100.,len=5)
         vtdsp=vtdsp(0)+','+vtdsp(1)
         mx = max(max(10*alog10(hhtx(ind))))
         mn = min(min(10*alog10(hhtx(ind))))
         yr = [mn-2.*(mx-mn), mx+0.3*(mx-mn)]
      endif

      mxn = max(10*alog10(hhn(ind)))
      mnn = min(10*alog10(hhn(ind)))
      yrn = [mnn-0.05*(mxn-mnn), mxn+2.*(mxn-mnn)]

      ylabelL='hTx: + (dBm)'
      ylabelR='hhn: ... (dBm)'
      text='smooth: '+ num2str(sm,fr=0)
      if intcal then text=text+'; Chthl='+Chthl+', Chtvt='+Chtvt+ $
           ', Chtvl='+Chtvl+', htDSP=['+htdsp+']'

      plot, time(ind),10*alog10(hhn(ind)), psym=3, $
            ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8],yrange=yrn
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1
      axis, yaxis=1,ytitle=ylabelR,ystyle=1

      lastplot
      if intcal then begin
        plot, prof(ind),10*alog10(ht(ind)),/noerase, psym=1,symsize=0.5, $ 
              ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8],yrange=yr
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ytitle=ylabelL,ystyle=1
        yrt=  yr(1)-0.06*(yr(1)-yr(0))
      endif else begin
        plot, prof(ind),ind,/noerase,/nodata, $ 
              ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8],yrange=yrn
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ystyle=1
        yrt = yrn(1)-0.06*(yrn(1)-yrn(0))
      endelse

      xyouts, prof(ind(0))+0.05*(prof(ind(nprof))-prof(ind(0))),yrt,$
              text,charsize=0.7
      print,strmid(text,11,strlen(text)-1)
      nextplot

      if intcal then begin
        mx = max(10*alog10(vt(ind)))
        mn = min(10*alog10(vt(ind)))
        yr = [mn-2.*(mx-mn), mx+0.3*(mx-mn)]
      endif
      mxn = max(10*alog10(vvn(ind)))
      mnn = min(10*alog10(vvn(ind)))
      yrn = [mnn-0.05*(mxn-mnn), mxn+2.*(mxn-mnn)]

      ylabelL='vT: + (dBm)'
      ylabelR='vvn: ... (dBm)'

      text='smooth: '+ num2str(sm,fr=0)+'; Cvnhn='+Cvnhn
      if intcal then text=text+'; Cvtvl='+Cvtvl+', Cvthl='+Cvthl+ $
         '; vtDSP=['+vtdsp+']
           
      plot, time(ind),10*alog10(vvn(ind)), psym=3, $
            yrange=yrn,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1
      axis, yaxis=1,ytitle=ylabelR,ystyle=1

      lastplot

      if intcal then begin
        plot, prof(ind),10*alog10(vt(ind)),/noerase, psym=1,symsize=0.5, $ 
              yrange=yr,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ytitle=ylabelL,ystyle=1
        yrt=  yr(1)-0.06*(yr(1)-yr(0))
      endif else begin
        plot, prof(ind),ind,/noerase,/nodata, $ 
              yrange=yrn,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ystyle=1
        yrt = yrn(1)-0.06*(yrn(1)-yrn(0))
      endelse

      xyouts, prof(ind(0))+0.05*(prof(ind(nprof))-prof(ind(0))),yrt,$
              text,charsize=0.7
      print,strmid(text,11,strlen(text)-1)

   endif else $

   if (alg eq sppalg) then begin

      if intcal then begin
        if (txsequence(0) eq polH) then begin
           vl=vvlk
           ht=hhtx
        endif else begin
           vl=hhlk
           ht=vvtx
        endelse
        Chtvl=strmid(strtrim(string(round(100*correlate(ht(ind), $
              vl(ind)))/100.),1),0,4)
        if (sm gt 0) then begin
           vl=smooth(vl,sm)
           ht=smooth(ht,sm)
        endif
      endif

      Cvnhn=strmid(strtrim(string(round(100*correlate(vvnoise(ind), $
            hhnoise(ind)))/100.),1),0,4)

      if (sm gt 0) then begin
         hhn=smooth(hhnoise,sm)
         vvn=smooth(vvnoise,sm)
      endif else begin
         hhn=hhnoise
         vvn=vvnoise
      endelse

      ind1=lindgen(nprof/2)*2
      ind2=ind1+1
      text= 'smooth: '+ num2str(sm,fr=0)

      if intcal then begin
       if (txsequence(0) eq polh) then begin
         htdsp=hhtx(ind2)/hhtx(ind1)
         htdsp=num2str(round(1000.*alog10([mean(htdsp),std(htdsp)]))/100.,len=5)
         htdsp=htdsp(0)+','+htdsp(1)
         text= text+'; Chtvl='+Chtvl+'; htDSP=['+htdsp+']'
         ylabel='hT: ... (dBm)'
         mx = max(10*alog10(ht(ind))) +1.
         mn = min(10*alog10(ht(ind)>1.e-5)) -1.
       endif else begin
         vtdsp=vvtx(ind2)/vvtx(ind1)
         vtdsp=num2str(round(1000.*alog10([mean(vtdsp),std(vtdsp)]))/100.,len=5)
         vtdsp=vtdsp(0)+','+vtdsp(1)
         text=text+'; Cvthl='+Chtvl+'; vtDSP=['+vtdsp+']'
         ylabel='vT: ... (dBm)' 
         mx = max(10*alog10(ht(ind))) +1.   ; ht=vvt
         mn = min(10*alog10(ht(ind)>1.e-5)) -1.
       endelse
       yr = [mn-1.*(mx-mn), mx+0.05*(mx-mn)]
      endif

      mxn = max([max(10*alog10(vvn(ind))), max(10*alog10(hhn(ind)))])
      mnn = min([min(10*alog10(vvn(ind))), min(10*alog10(hhn(ind)))])
      yrn = [mnn-0.05*(mxn-mnn), mxn+0.75*(mxn-mnn)]
      
      plot, time(ind),10*alog10(hhn(ind)),psym=1,symsize=0.3,color=64, $ 
            yrange=yrn,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
      axis, xaxis=0,xtitle=xbotlabel,xstyle=1
      axis, yaxis=1,ytitle='hhn: + vvn: x (dBm)',ystyle=1
      oplot,time(ind),10.*alog10(vvn(ind)),psym=7,symsize=0.3,color=145

      lastplot
      if intcal then begin
        plot, prof(ind),10*alog10(ht(ind)),/noerase,psym=3, $
              yrange=yr,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ytitle=ylabel,ystyle=1
        yrt = yr(1)-0.06*(yr(1)-yr(0))
      endif else begin 
        plot, prof(ind),ind,/noerase,/nodata, $
              yrange=yrn,ystyle=5,xstyle=5,ymargin=[4,4],xmargin=[8,8]
        axis, xaxis=1,xtitle=xtoplabel+'; Indjump='+ijump+'!C!C',xstyle=1
        axis, yaxis=0,ystyle=1
        yrt = yrn(1)-0.06*(yrn(1)-yrn(0))
      endelse

      text=text+'; Cvnhn='+Cvnhn
      xyouts, prof(ind(0))+0.02*(prof(ind(nprof))-prof(ind(0))),yrt,$
              text,charsize=0.7
      print,strmid(text,11,strlen(text)-1)

;    endif  for SPPMAG 

; Plot pulse pair calibration (pptx) in m/s  for SPPMAG only
; When pulse pair is not plotted hv/vv or vh/hh is monitored
; In fact there is no need for detailed monitoring of ppTx; use statistics only
 
intcal=0 ; always ignore pulse pair plot and do the cross channel plot 

  if !P.MULTI(2) gt 2 then $
     if !P.MULTI(0) eq 0 then !P.MULTI(0)=!P.MULTI(2)-1 $
     else  !P.MULTI(0)=!P.MULTI(0)-1

  if intcal then begin
   maxv=prf(0)/4*lambda
   if (alg eq sppalg) then begin
     if (sm eq 0) then ppt=maxv*atan(imaginary(pptx(ind)),float(pptx(ind)))/!pi $
     else begin 
        ppt=maxv*atan(imaginary(pptx),float(pptx))/!pi 
        ppt=smooth(ppt,sm) & ppt=ppt(ind)
     endelse
     ylabel='ppTx Velocity (m/s)'
   endif else begin
     if txsequence(0) eq polH and txsequence(1) eq polH then begin
         ylabel='pp12Tx Velocity (m/s)'
         maxv=prf(0)/4*lambda
         if (sm eq 0) then ppt=maxv*atan(imaginary(pptx(0,ind)), $
                                         float(pptx(0,ind)))/!pi $
         else begin 
            ppt=maxv*atan(imaginary(pptx[0,*]),float(pptx[0,*]))/!pi 
            ppt=smooth(ppt,sm) & ppt=ppt(ind)
         endelse
      endif else $
     if txsequence(1) eq polH and txsequence(2) eq polH then begin
         ylabel='pp23Tx Velocity (m/s)'
         maxv=prf(1)/4*lambda
         if (sm eq 0) then ppt=maxv*atan(imaginary(pptx(1,ind)),$
                                         float(pptx(1,ind)))/!pi $
         else begin 
            ppt=maxv*atan(imaginary(pptx[1,*]),float(pptx[1,*]))/!pi 
            ppt=smooth(ppt,sm) & ppt=ppt(ind)
         endelse
      endif else $
     if txsequence(2) eq polH and txsequence(3) eq polH then begin
         ylabel='pp34Tx Velocity (m/s)'
         maxv=prf(2)/4*lambda
         if (sm eq 0) then ppt=maxv*atan(imaginary(pptx(2,ind)), $
                                         float(pptx(2,ind)))/!pi $
         else begin 
            ppt=maxv*atan(imaginary(pptx[2,*]),float(pptx[2,*]))/!pi
            ppt=smooth(ppt,sm) & ppt=ppt(ind)
         endelse
     endif else begin
         ylabel='pp12Tx Velocity (m/s)'
         maxv=prf(0)/4*lambda
         if (sm eq 0) then ppt=maxv*atan(imaginary(pp12tx(ind)), $
                                         float(pp12tx(ind)))/!pi $
         else begin 
            ppt=maxv*atan(imaginary(pptx[0,*]),float(pptx[0,*]))/!pi
            ppt=smooth(ppt,sm) & ppt=ppt(ind)
         endelse
     endelse
   endelse

   minpt = min(ppt)
   maxpt = max(ppt)
   yr = [minpt-0.05*(maxpt-minpt),maxpt+0.15*(maxpt-minpt)]
   plot, time(ind),ppt,yrange=yr,ystyle=1,xstyle=1,ticklen=1,symsize=0.3,$
         psym=1,xgridstyle=1,ygridstyle=1,xmargin=[8,8],ymargin=[4,4], $
         ytitle=ylabel,xtitle=xbotlabel,title='smooth='+num2str(sm,fr=0)+spc(70)

  endif else begin  ; monitor hv/vv or vh/hh
      
    ind2 =3                 ; don't use first gates
    ind3 =rangegates-1 
    maxp =5000              ; max points for scatter plots
    maxsm=101               ; maximum smoothing
    minsm=3                 ; minimum smoothing
;   gi   =10.^(0.3)         ; approx. minimum gain inbalance, vv/hh= 3 dB
    gi   =10.^(-0.3)        ; approx. minimum gain inbalance, vv/hh=-3 dB

    mnhhn    = mean(hhnoise(ind))
    mnvvn    = mean(vvnoise(ind))

    if (txsequence(0) eq polH) then begin
      ind1=where(vh(ind2:ind3,ind) gt minsig*gi)
      nind1=n_elements(ind1)
      if nind1 lt 3 then nextplot else begin
        temp=(hh[ind2:ind3,ind])[ind1] 
        ldr =((vh[ind2:ind3,ind])[ind1]-mnvvn)/(temp-mnhhn)
        sm=(0.05*nind1<maxsm)>minsm 
        if sm gt nind1 then sm=0
        if nind1 gt 2*maxp then begin
           step=nind1/maxp
           ind1=lindgen(nind1/step)*step
        endif else ind1=lindgen(nind1)

        plot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),/nodata,$
          xmargin=[8,8],ymargin=[4,4],xtitle='HH power [dBm]', $
          title=ijump+' points ('+num2str(nind1)+'/'+num2str(n_elements(ind1))+$
          ' total/plotted), smooth='+num2str(sm,fr=0)+spc(25),$
          xgridstyle=1,ygridstyle=1,ticklen=1,ytitle='VH/HH [dBm]'
        oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),psym=3,color=128    

        if (sm gt 0 and nind1 gt 3*sm) then begin
           isort=sort(temp) & temp=temp(isort)
           ldr=smooth(ldr(isort),sm)
           ind1=lindgen(n_elements(ldr)-3*sm)+2*sm
           oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),thick=3
        endif
      endelse
    endif else begin     
      ind1=where(hv(ind2:ind3,ind) gt minsig)
      nind1=n_elements(ind1)
      if nind1 lt 3 then nextplot else begin
        temp=(vv[ind2:ind3,ind])[ind1] 
        ldr =((hv[ind2:ind3,ind])[ind1]-mean(hhnoise))/(temp-mean(vvnoise))
        sm=(0.05*nind1<maxsm)>minsm 
        if sm gt nind1 then sm=0
        if nind1 gt 2*maxp then begin
           step=nind1/maxp
           ind1=lindgen(nind1/step)*step
        endif else ind1=lindgen(nind1)

        plot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),/nodata,$
          xmargin=[8,8],ymargin=[4,4],xtitle='VV power [dBm]', $
          title=ijump+' points ('+num2str(nind1)+'/'+num2str(n_elements(ind1))+$
          ' total/plotted), smooth='+num2str(sm,fr=0)+spc(25),$
          xgridstyle=1,ygridstyle=1,ticklen=1,ytitle='HV/VV [dBm]'
        oplot,10.*alog10(temp(ind1)),10.*alog10(ldr(ind1)),psym=3,color=128    
           
        if (sm gt 0 and nind1 gt 3*sm) then begin
           isort=sort(temp) & temp=temp(isort)
           ldr=smooth(ldr(isort),sm)
           ind1=lindgen(n_elements(ldr)-3*sm)+2*sm
           oplot,10.*alog10(temp),10.*alog10(ldr),thick=3
        endif
      endelse 
    endelse
  endelse
endif  ; for SPPMAG

end
