;$Id: printwcrstamp.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       PRINTWCRSTAMP
;
; PURPOSE:
;       Print wcrfile parameters and statistics at the bottom of a page.
;       Print a date stamp at the upper left corner of a page.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       PRINTWCRSTAMP, Tlabel
;
; INPUTS:
;       TLABEL: title text (default is none)
;
; KEYWORD PARAMETERS:
;       MODE:    0 : displays the information 
;                1 : saves the info in ascii file
;                2 : prints WCR params. at a given position (normal units)
;                    using the current device 
;       PAGE:    page number (could be number or string, e.g., 1 or '1-1')
;
;       INDJUMP: 'all'  - uses even and odd profiles (default)               
;                'even' - uses even profiles only
;                'odd'  - uses odd profiles only
;
;       RANGE:   [begin_rg, end_rg], (default [3,rangegates-1])
;                range gate zoom for co/cross channel statistics
;
;       STATTEXT: string array with stat information.  Works only with mode 2.
;       HELP:  Show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;
; SIDE EFFECTS:
;       print/plot radar information 
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, May 1996
;       S.H., May 98 -- added page keyword        
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Dec 99 -- added stattext 
;       S.H., Jan 00 -- added help keyword
;       S.H., Feb 01 -- changed routine name from printvxistamp to printwcrstamp
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro printwcrstamp,tlabel,mode=mode,page=page,indjump=ijump,range=rgzm, $
                  stattext=stattext,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'printwcrstamp'
  return
endif

if n_params() eq 0 then tlabel=''
if n_elements(mode) eq 0 then mode=2

xyouts,0.08,-0.03,tlabel,charsize=0.9,/normal,/noclip 
printwcrprms,mode,[0.08,-0.05,0.75]
printwcrstat,mode,[0.52,-0.05,0.75],indjump=ijump,range=rgzm,stattext=stattext

if n_elements(page) eq 0 then datestamp,/user $
else datestamp,/user,com='      Page '+string(page)

end
