;$Id: readwcrall.pro,v 1.5 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       READWCRALL
;
; PURPOSE:
;       Load into the wcrdata common block all radar data from a WCR file 
;       If strt/stp exist loads data according to readmode.
;       Note:  Run OPENWCRM program before running this routine.
;       This program is obsolete.  Use readwcrdata.
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       READWCRALL, Data, Strt, Stp
;
; INPUTS:
;       Data:  wcr file structure
;       Strt:  start reading point
;              For rmode = 0  -  strt/stp record in the wcr file
;                          1  -  input:  strt/stp profile numbers
;                                output: strt/stp corresponding wcr file records
;                          2  -  input:  strt/stp UTC in hhmmss, long
;                                output: strt/stp corresponding wcr file records
;       Stp:   stop reading point (see above)
;
; KEYWORD PARAMETERS:
;       TIMEMODE: 1 : [sec,microsec]; lonarr(stp-strt+1) 
;                 2 : hhmmss.dd     ; dblarr(stp-strt+1)   
;                 3 : sssss.dd      ; dblarr(stp-strt+1) (default)
;       READMODE: 0 : reads profs given strt/stp record number(default)
;                 1 : reads profiles between strt/stp profile numbers
;                 2 : reads profiles within strt/stp UTC    
;       HCOR:     when set correct all H-channel data. 
;                 Use hcor=-1 to suppress H correction info print
;       PPMNOAV:  when set, same pol pulses in the burst are not averaged for
;                 PPMAG alg.  The consequence of this is that hh, vv, hv and vh 
;                 become 3-D arrays, and hhnoise, vvnoise, 2-D arrays
;       BADPROF:  output, array with profile numbers of potentially bad profiles
;       CLOSEWCR: when set closes the wcr file using defaull lun=31
;                 integer > 1 gives the wcr file lun
;     BIG_ENDIAN: when set, assumes that the input file data are in big endian
;                 order.  The default is LITTLE_ENDIAN.
;       HELP:     show this text
;
; OUTPUTS:
;
; COMMON BLOCKS:
;       wcrparams 
;
; SIDE EFFECTS:
;       HCOR is ignored for fft wcr files
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Mar 1996
;       S.H., Feb 1997   -- added hcor keyword
;       S.H., Jun 1998   -- added ppmnoav keyword
;       S.H., Aug 1999   -- added badprof keyword
;                           modified help info
;       S.H., Jan 2000   -- added help keyword
;       S.H., Jan 2001   -- added little_endian keyword
;       S.H., Feb 2001   -- changed routine name from readvxiall to readwcrall
;       S.H., Apr 2002   -- replaced LITTLE_ENDIAN keyword with BIG_ENDIAN
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro readwcrall, data, strt, stp, timemode=tmode, readmode=rmode, hcor=hcor, $
                ppmnoav=ppmnoav, badprof=badprof, closewcr=closewcr, $
                big_endian=bgnd, help=help

common wcrparams

on_error,2

if keyword_set(help) then begin
  doc_library,'readwcrall'
  return
endif

nr=tnumrec-1

; Check the input

if (keyword_set(tmode) eq 0) then tmode=3
if (keyword_set(rmode) eq 0) then rmode=0

if (n_params() eq 1) then begin
   rmode = 0 
   strt = 0 
   stp = nr 
endif else $
if (n_params() eq 2) then $ 
   stp = nr

readwcrtb, data, strt,stp,timemode=tmode, readmode=rmode,$
           badprof=badprof,big_endian=bgnd
readwcrnoise, data, strt,stp,ppmnoav=ppmnoav,big_endian=bgnd

if (alg eq fftAlg) then $
   readwcrfftspec, data, strt,stp,big_endian=bgnd $
else begin
   readwcrpowerpp, data,strt,stp,ppmnoav=ppmnoav,big_endian=bgnd
   if keyword_set(hcor) then begin
      if hcor eq -1 then silent=1 else silent=0
      wcrhcor,silent=silent
   endif
endelse 

if n_elements(closewcr) ne 0 then $
   if closewcr lt 2 then closewcrf else closewcrf,closewcr

end
