;$Id: showwcrprms.pro,v 1.8 2003/12/23 22:15:43 haimov Exp $
;+ 
; NAME:
;       SHOWWCRPRG
;
; PURPOSE:
;       Show WCR wcr IDL routines
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       showwcrprms
;
; KEYWORD PARAMETERS:
;       PRMS:  when set show WCR param. variables only (wcrparams common block)
;       DATA:  when set show WCR data variables only (wcrdata common block) 
;       HELP:  show this text
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Oct 1996
;       S.H., Nov 99 -- modified help info
;       S.H., Jan 00 -- added help keyword
;       S.H., Nov 00 -- added changes for DPFFT mode
;       S.H., Feb 01 -- changed routine name from showvxiprms to showwcrprms 
;       S.H., Jul 01 -- comment about wcrflag[2] added
;       S.H., Jan 02 -- added comments about which routines are used to 
;                       load/calculate the variables
;       S.H., Feb 02 -- added basetime and noiseflag
;       S.H., Apr 02 -- added filehdrsz,irigflag,acqtime
;       S.H., Dec 02 -- changed timeinterval and dspsync definitions
;       S.H., Mar 03 -- chaged irigflag definition
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro showwcrprms, prms=prms, data=data, help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'showwcrprms'
  return
endif

if (KEYWORD_SET(data) eq 0) then begin
print,'----------------------------------------------------------------------'
print,'WCR file parameters (common wcrparams) loaded by openwcrf routine:'
print,''
print,'acqtime            - double, DSP data acquisition time in ms'
print,'alg                - integer, data mode code: SPPMAG-7,PPMAG-9,FFT-11'
print,'aspectratio        - range-to-profile ratio based on KAspeed'
print,'averages           - integer, number of pulses averaged per profile'
print,'datablocksize      - integer, WCR file data block size in bytes'
print,'datarate           - float, WCR file acquisition data rate in KB/sec'
print,'dftalg             - integer, FFT (co- and cross-spectra) mode code, 13'
print,'fftalg             - integer, FFT (co-spectra) mode code, 11'
print,'fftbins            - integer, number of spectral lines in fft mode'
print,'fftwindow          - integer, fft window code, 0-5: Rec,Hann,FlatTop,'
print,'                       Gaussian Top,Blackman,Gaussian'
print,'filehdrsz          - long, WCR file file header size (after March 2002)'
print,'fileidstr          - string, WCR file name ID, first 3 chars.'
print,'filenamel          - integer, WCR file name length, 20'
print,'fstblocknum        - integer, first profile number'
print,'headersize         - integer, WCR file data header size in bytes'
print,'iffilter           - integer, WCR Rx IF match filter in MHz'
print,'interrupts         - intarr(n,2), start/stop profs for every data on/off'
print,'                       It is setup to ignore interrupts less than 50'
print,'                       profs to avoid rare incidents of prof. glitches'
print,'                       After May 99 interrupts is obsolete; data on/off'
print,'                       creates a new file every time it is toggled'
print,'irigflag           - integer/intarr[3], timestamps flag (after March 2002)'
print,'                     irig_status/[irig_status,tfp_jamsynch,tfp_status]'
print,'                     irig_status:' 
print,'                       0 - use initial time from host computer'
print,'                       1 - use TFP with IRIG B sync'
print,'                      -1 - data before March 2002'
print,'                      -2 - clock failure, time replaced with a counter'
print,'                           (this is added by writewcrraw.pro)' 
print,'                     tfp_jamsync: 0/1, 1 - forced time jamsynch' 
print,'                     tfp_status: as reported by the tfp 0 to 7' 
print,'kaspeed            - integer, King Air average speed in m/s'
print,'lambda             - float, RF wavelength, 0.00316 m'
print,'lstblocknum        - integer, last good profile number'
print,'maxvel             - float/fltarr, max Doppler velocity'
print,'noiseflag          - long/lonarr(2), shows which pulses in the burst'
print,'                     are used to load/average the noise data:'
print,'                       1234 - for SPPMAG'
print,'                       0    - for FFT or not determined'
print,'                       1-6 or combination (e.g. [12,34]) for PPMAG 4/6,'
print,'                       where noiseflag[0]/[1] is for h/v channel'
print,'noisesamples       - integer, number of noise samples recorded
print,'npulse             - integer, number of pulses in a burst'
print,'numrec             - integer, currently read number of profiles'
print,'polh               - integer, H pol code, 7; 104(h) after 5/99'
print,'polv               - integer, V pol code, 15; 118(h) after 5/99'
print,'pp4alg             - integer, PPMAG mode (6 pulses) code,15 after 5/99'
print,'pp6alg             - integer, PPMAG mode code, 7; 14 after 5/99'
print,'ppstrpol           - strarr(npulse-1), pulsepair pol. sequence'
print,'                       received pol. sequence for the pulse pairs (pp)'
print,'                       [0] - pp12, [1] - pp23, etc.'
print,'prf                - fltarr(6),  after 3/02;  PRF in Hz'
print,'pulselength        - integer, transmitted pulse width in ns'
print,'                        pulse sequence pol. codes,7-H, 15-V, 0-not used'
print,'rangegates         - integer, number of range gates'
print,'rangegatedelay     - float, 1st range gate delay in meters'
print,'rangegatespacing   - float, gate sampling resolution in meters'
print,'rxsequence         - bytarr(6,2), pp Rx sequence, after 3/02'
print,'sppalg             - integer, SPPMAG mode code, 9'
print,'starttime          - string, file start time: hh:mm:ss.ss'
print,'stoptime           - string, end of file time: hh:mm:ss.ss '
print,'tcorrection        - float, radar time timehack'
print,'timeinterval       - float, median time interval between profiles'
print,'timelength         - float, approximate data duration in sec'
print,'tnumrec            - integer, total number of profiles'
print,'txsequence         - bytarr(8), Tx sequence, after 5/99;
print,'txstrpol           - strarr(npulse), Tx pulse sequence, e.g., HHVV, etc.'
print,'wcrdate            - string, WCR file date, yyyymmdd, from wcrfile'
print,'wcrfile            - string, WCR file name'
print,'wcrflag            - string, WCR file status flag'
print,'                     0 - ok'
print,"                     wcrflag[0]= 1, file size doesn't match"
print,'                     wcrflag[1]= # of bad profiles at the file end'
print,'                     wcrflag[2]=1, WCR computer date and timestamps date'
print,"                     don't match or IRIG connection/data was lost/corrupted"
print,"----------------------------------------------------------------------"
endif
if (KEYWORD_SET(prms) eq 0) then begin
print,'----------------------------------------------------------------------'
print,'WCR file data (common wcrdata) loaded by readwcrdata; if different'
print,'    the handling routine is shown in parentheses:'
print,''
print,'basetime      - long, loaded data begin time in secs since 000000 1/1/1970' 
print,'begtime       - string, begin time for loaded data in hh:mm:ss'
print,'dspsync       - fltarr(6), mean, max, min of 2 DSPs time diff. in ms'
print,'                           and mean, max, min for timeinterval in ms'
print,'endtime       - string, end time for loaded data in hh:mm:ss'
print,'hcorrection   - intarr(8),h corr. is applied (lookup table error) to'
print,'                [hhnoise,hhtx,hhlk,hh,hv,ppnoise,pptx,pp]'
print,'                0/1 - not_corrected/corrected (wcrvcor and wcrhcor)'
print,'                After 1999 lookup table was corrected and hcorrection'
print,'                is generally not needed'
print,'vcorrection   - intarr(8),v corr. is applied (lookup table error) to'
print,'                [vvnoise,vvtx,vvlk,vv,vh,ppnoise,pptx,pp]'
print,'                0/1 - not_corrected/corrected (wcrvcor and wcrhcor)' 
print,'                There are no major lookup table errors; vcorrection'
print,'                is generally not needed'
print,'prof          - intarr(numRec), recorded profile numbers'
print,'radrng        - fltarr(rangegates), center of range gates in meters (openwcrf)'
print,'time          - dblarr(numRec), profile time in sec'
print,'timemode      - integer, time vector mode'
print,'                     -1 -  variable is cleared'
print,'                      0 -  time not read'
print,'                      1 -  time is lonarr [UNIXsec, microsec]'
print,'                      2 -  time is hhmmss.dd after midnight'
print,'                      3 -  time is sssss.dd starting from 0.dd'
print,'                      4 -  time is sssss.dd after midnight'
print,'velo          - fltarr(fftbins), velocity axis for fft spectra in m/s (openwcrf)'
print,''
print,'fftnoise  - fltarr(fftbins,numRec), H or V noise spectra for FFT' 
print,'            fltarr(fftbins,2, numRec), H and V noise spectra for DPFFT'
print,'                  [*,0,*] - V noise, [*,1,*] - H noise spectra
print,'hhnoise   - fltarr(numRec), H pol averaged noise channel'
print,'          - fltarr(noisesamples,numRec), H pol noise channel'
print,'          - fltarr(noisesamples,n,numRec), H pol noise channel'
print,'            n=4 - PPMAG4, n=6 - PPMAG6
print,'ppnoise   - complexarr(numRec), HH or VV pp noise for SPPMAG'
print,'          - complexarr(npulse-1,numRec), HH/VV pp noise for PPMAG'
print,'vvnoise   - fltarr(numRec), V pol averaged noise channel'
print,'          - fltarr(noisesamples,numRec), V pol noise channel'
print,'          - fltarr(noisesamples,n,numRec), H pol noise channel'
print,'            n=4 - PPMAG4, n=6 - PPMAG6
print,''
print,'ffttx     - fltarr(fftbins,numRec), H or V FFT leakage spectra for FFT'
print,'            fltarr(fftbins,2,numRec), co/cross leak spectra for DPFFT'
print,'hhtx      - fltarr(numRec), H-Tx H-Rx leakage in mW'
print,'          - fltarr(n,numRec), H-Tx H-Rx leakage in mW'
print,'            n- number of H non-averaged received pulses for PPMAG'
print,'hhlk      - fltarr(numRec), V-Tx H-Rx leakage in mW'
print,'          - fltarr(n,numRec), V-Tx H-Rx leakage in mW'
print,'            n- number of H non-averaged received pulses for PPMAG'
print,''
print,'vvlk      - fltarr(numRec), H-Tx V-Rx leakage in mW'
print,'          - fltarr(n,numRec), H-Tx V-Rx leakage in mW'
print,'            n- number of V non-averaged received pulses for PPMAG'
print,'vvtx      - fltarr(numRec), V-Tx V-Rx leakage in mW'
print,'          - fltarr(n,numRec), V-Tx V-Rx leakage in mW'
print,'            n- number of V non-averaged received pulses for PPMAG'
print,''
print,'pptx      - fltarr(numRec), HH or VV Tx leakage pp  for SPPMAG'
print,'          - fltarr(npulse-1, numRec), HH/VV Tx leakage pp  for PPMAG'
print,''
print,'fftps     - fltarr(fftbins,rangegates,numRec), fft spectra'
print,'            fltarr(fftbins,rangegates,2,numRec), fft co- and cross-'
print,'            spectra for DPFFT; [*,0,*] - V-ch, [*,1,*] - H-ch'
print,'hh        - fltarr(rangegates,numRec), HH averaged power in dBm'
print,'          - fltarr(rangegates,n,numRec), HH power in dBm'
print,'            n- number of H non-averaged received pulses for PPMAG'
print,'hv        - fltarr(rangegates,numRec), HV ave.power in dBm (Tx V, Rx H)'
print,'          - fltarr(rangegates,n,numRec), HV power in dBm'
print,'            n- number of H non-averaged received pulses for PPMAG'
print,'pp        - complexarr(rangegates,numRec), HH or VV pp for SPPMAG'
print,'          - complexarr(rangegates,npulse-1,numRec), HH/VV pp for PPMAG'
print,'vh        - fltarr(rangegates,numRec), VH ave.power in dBm (Tx H, Rx V)'
print,'          - fltarr(rangegates,n,numRec), VH power in dBm'
print,'            n- number of V non-averaged received pulses for PPMAG'
print,'vv        - fltarr(rangegates,numRec), VV averaged power in dBm'
print,'          - fltarr(rangegates,n,numRec), VV power in dBm'
print,'            n- number of V non-averaged received pulses for PPMAG'
print,''
print,'----------------------------------------------------------------------'
endif
end
