;$Id: wcrdisp.pro,v 1.15 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;    WCRDISP 
;    (set of routines; initial compilation is NOT required)
;    This program is obsolete.  Use wcrdisplay instead.
;
; PURPOSE:
;
;    WCR "Kestrel" Real-Time Display (GUI is a blocking widget)
;
; CATEGORY:
;    wcrtools
;
; CALLING SEQUENCE:
;
;    wcrdisp, nimage,minmax=minmax,calval=calval,start=strtp,acspeed=acspeed,$
;             dispctrl=dispctrl,colortable=colortable,help=help
;             
; INPUT:
;
;    nimage    -  number of images: 1 or 2 (default)
;
; KEYWORDS:
;
;    ACSPEED:    input,float, average aircraft speed in m/s (default 90 m/s)
;    CALVAL:     input,fltarr,[hhcal,vvcal,iso] (default [50,49,-18])
;    COLORTABLE: input,integer, color table number(default is 45-Wyo dual table;
;                when less than 64 colors are available the default is 41 Wyo
;                table); if less than 46 tables color table 13 - Rainbow is used
;                To leave the current color table enter negative number
;    DISPCTRL:   when set, removes display control panel from the widget
;    HELP:       display this help text
;    MINMAX:     input,fltarr,[min_hh/vv,max_hh/vv,min_zdr,max_zdr,min_ldr
;                ,max_ldr], default: [-45,10,-4,4,-25,-10]
;                if a pair has min>=max, default values are used
;    START:      input,integer,starting wcr record number
;
; WIDGET CONTROLS:
;
;    New:          Open a dialog_pickfile to select a new WCR file.
;
;    Quit:         Terminates the program without destroying the widget.  
;                  This could be used to preserve interesting images for
;                  later review and notes.
;
;    Destroy:      Terminates the program and destroys the widget
;
;    Pause/Cursor: Pressing this button will pause the display.  It also allows
;                  to extract image values by clicking the left button of the
;                  mouse at a desired pixel of the images.  In case of two 
;                  images, clicking on either image will extract the
;                  corresponding value from the other image.  The values and
;                  their time stamp are printed in the status line
;    Continue:     Continue label replaces Pause/Cursor label when Pause is
;                  active.  Pressing Continue will restore updating the display
;                  with new data.
;
;    Catch:        Pressing this button will force the display to position
;                  at the end of the currently available data in the wcr file.
;                  This feature is useful if the display get behind the
;                  incoming data (this could happen for various reasons,
;                  including using Pause/Cursor)
;
;    Color:        Allow change in the color table being used.  The default
;                  color table is a custom dual-table to allow different colors
;                  for, e.g., reflectivity and velocity images.  If the default
;                  color table is replaced the program checks if a narrow black
;                  strip exists in the middle of the new color table. If it does
;                  the color table splits in two otherwise the same color
;                  table is used for both images
;
;    Img:          Pressing this button will toggle between 1 and 2 image widget
;
;    InitTime:     Editable field.  Contains the starting time of wcr data.
;                  Editing the field will cause the display to start again
;                  from time close to the entered time.  This field should
;                  not be used during data acquisition.  It is intended
;                  for browsing of already recorded files.  The new time must
;                  have one of the following formats: hhmm, hh:mm, hhmmss,
;                  hh:mm:ss, 9999 . Incorrect entries are ignored and the start
;                  time is redisplayed. If enter 9999 the field will show the
;                  end time of the file.
;
;    DispStep:     Editable field. Number of profiles to be calibrated and
;                  displayed simultaneously.  Must be greater than 0. Non-valid
;                  entries are ignored and the default value is used.
;
;    AveProfs:     Editable field. Number of raw profiles averaged before
;                  display.  The default is aspect ratio which scales range:time
;                  to 1:1 based on acspeed (default 90 m/s).  Must be greater
;                  than 0.  It is not allowed both DispStep and AveProfs to be
;                  equal to 1.  In this case the second one to be changed to one
;                  will be replaced automatically by 2.
;
;       Sleep:     Editable field. Sleep time[s] between disk access attempts
;                  for 'Empty file' or 'End of file. Waiting for data...' 
;                  conditions. Default is 5 sec, maximum is 60 sec.
;
;    hhcal:        Editable field. Calibration constant (dB) for H-channel
;    vvcal:        Editable field. Calibration constant (dB) for V-channel
;    isol:         Editable field. Isolation (dB) between H- and V-channel
;
;    Radar products:  availability depends on the wcr operational mode
;       hh:   co-reflectivity for H-ch (dBZ)
;       vv:   co-reflectivity for V-ch (dBZ)
;       vh:   cross-reflectivity; H-ch transmitted, V-ch received
;       ldr:  vh/hh or hv/vv in dB
;       zdr:  hh/vv in dB
;       dv_h: uncorrected for the platform motion mean Doppler velocity in m/s
;             for H-channel
;       dv_v: uncorrected for the platform motion mean Doppler velocity in m/s
;             for V-channel
;       sv:   spectral width in m/s (for wcr FFT mode only)
;    Order:   Pressing this button will toggle image display order: bottom-
;             to-top or top-to-bottom (side/up antenna or down antenna)
;
;    Color bar limits: these are editable fields and the default values
;                      can be replaced.  To activate the new entry 'Enter'
;                      key must be hit after typing the new number.  There is
;                      no check for the validity of the new limits.
;                      Non-numerical entries are replaced with 0 and a
;                      non-fatal error message is issued.
;
;    Status:  Non editable infomation message
;    <Hn>:    Non editable information indicator for the averaged H-ch noise.
;    mxHn:    Resetable indicator for the max <Hn> noise.
;    <Vn>:    Non editable information indicator for the averaged V-ch noise.
;    mxVn:    Resetable indicator for the max <Vn> noise.
;    Hleak:   Non editable information indicator for the leaked H-ch Tx power
;             WCR Tx leaked power (in dBm) at 0 range.  This indicator is used
;             to help detect data acquisition bug which causes a shift in the
;             range gates.  In this case the leak power will be close to the
;             noise level (around -65 dBm)
;    Vleak:   Non editable information indicator for the leaked V-ch Tx power.
;             WCR Tx leaked power (in dBm) at 0 range. This indicator can be
;             used together with Hleak to help detect a problem with Tx power
;             for H or V channel.
;    mxdT:    Resetable maximum average time interval (in ms) between profiles;
;             If 9999 is displayed a negative time jump in the time stamps was
;             encountered.
;
; WIDGET CONTROL NOTE: 'Enter' from the keyboard must be used after modifying an
;                       editable field for the new value to take effect
;
; NOTES: 1) There is a positive side effect of using PICKFILE CANCEL button.
;           It can restore an interrupted display of a file if a new file could
;           not be open due to no data.
;        2) Pause/Cursor button pauses the display and gives control to the
;           mouse for image cursor trace. Clicking left mouse button while over
;           image 1 or image2 will show image time and value in the status line.
;        3) The displayed time is Aircraft/GPS time (after May 1999)
;        4) External (custom) routines used by wcrdisp:  readse, mean, wyoctind,
;           sec2timestr, timestr2sec, num2str, stradj from idltools linbrary,
;           and spc from jhuapl library
;-
; MODIFICATION HISTORY:
;
;    Samuel Haimov, Jan 1997  (v.2.0, major upgrade from ver.1.0, Nov 1996)
;    SH, Jan 97 (v.2.1)    --  handles CANCEL from pickfile
;    SH, Feb 97 (v.3.0)    --  added CATCH button
;    SH, Feb 97            --  added VH button
;    SH, Mar 97 (v.5.1)    --  added FFT zero and first moments display
;                              added 2nd range axes in kft
;                              minor widget adjustments
;                              added CALVAL keyword
;    SH, Mar 97 (v.6.0)    --  added calibration and display control
;                              added DISPCTRL keyword
;                              minor performance optimizations
;    SH, Apr 97 (v.6.2)    --  added internal calibration monitor (H ch. only)
;                              minor main loop optimization
;                              error in isolation removal fixed
;    SH, Apr 97 (v.6.3)    --  adjusted colors for LDR/ZDR noise levels
;                              minor widget adjustments
;    SH, Apr 97 (v.7.0)    --  added image cursor trace
;                              changed wcrfile filter to accept any first letter
;    SH, Apr 97 (v.7.1)    --  modified cursor trace to show matching value
;                              in case of 2-image widget
;    SH, Oct 97 (v.7.1.1)  --  added GROUP/MODAL keywords to xloadct (this is a
;                              workaround for non-blocking widgets under IDL 5.0
;                              changed pickfile with dialog_pickfile for IDL 5.0
;                              Added last processed Prof # to Pause status msg
;                              Fixed bug in time disp on status line for files
;                              crossing midnight
;    SH, May 98 (v.7.1.2)  --  removed MODAL keyword from xloadct and replaced
;                              it with the new BLOCK keyword under IDL 5.1
;    SH, Jan 99 (v.7.2)    --  modified cross-channel calibration
;                              disabled internal calibration monitor and changed
;                              cal. constants input
;                              replaced whattime (C routine) with sec2timestr
;    SH, Mar 99 (v.7.2.1)  --  added wcrdisp_common_blocks routine
;                              eliminated the need to run first .com wcrdisp
;    SH, Jun 99 (RCS v1.1) --  modified for the new version of wcr; this release
;                              handles SPPMAG, PPMAG(PPMAG4) and FFT modes only
;    SH, Jun 99 (RCS v1.2) --  renamed common blocks, added wcrdispopenf routine
;                              to make wcrdisp independant on any wcrtools
;                              routines
;    SH, Aug 99 (RCS v1.3) --  disabled H-ch correction, invert dispctrl for
;                              mantis and tigger
;    SH, Aug 99 (RCS v1.4) --  fixed a bug with double imaging while acquiring
;                              data in real time and changing display control
;                              parameters
;    SH, Jan 00 (RCS v3.0) --  restored int.cal (leak) display, work around
;                              cursor marks problem, many minor changes
;                              replace open and read routines with faster ones
;    SH, Feb 00 (RCS v3.1) --  added Doppler velocity width image for FFT mode
;    SH, Mar 00 (RCS v3.2-4)-- remove the use of num2str for wcrfile check
;                              removed bug in refl. averaging (isn't needed
;                              after use of rebin); modified fft averaging
;                              added a few comments
;    SH, Apr 00 (RCS v3.5) --  skip some checkings in wcrdispopenf, changed
;                              some default values
;    SH, May 00 (RCS v3.6) --  added colortable keyword
;    SH, Dec 00 (v.3.6-8)  --  edited colortable initialization depending on
;                              the number of colors available
;    SH, Jan 01 (RCS v3.9) --  fixed a bug in the cursor display when
;                              minimgsz > rangegates
;    SH, Feb 01 (RCS v3.10)--  modified to work on little endian machines;
;                              this is only for browser use.  In real-time
;                              mode it checks for mantis and skips any checks
;                              for the machine type (for efficiency);
;                              fixed minor bug (initilize uvalue)
;    SH, Mar 01 (RCS 1.0)  --  changed routine names by replacing vxi w/ wcr;
;                              many changes in widget building and handling;
;                              modified widget axes and control displays;
;                              corrected velo vector to match unshifted spectra
;                              and radar sign convention (pos.vel toward radar);
;                              now loads colortable even when wyoctind is not
;                              used and other mods on loading color tables
;    SH, Mar 01            --  added help button to GUI; fixed a bug in 
;                              wcrdispread when initilizing recraw; added update
;                              to avestep control display field; fixed a bug in
;                              fft mode when loopstep is changed; added input
;                              check for nimage (invalid input defaults to 2);
;                              modified colorbar min value position; changed
;                              help texts; workaround for a bug in draw_widget
;                              under MS Windows; minor bug fixed
;    SH, Jun 01            --  aspectratio min value set to 1; added acspeed and
;                              imgorder keywords
;    SH, Jul 01            --  fixed cursor readings when imgorder is set
;    SH, Jul 01            --  added skipprof keyword and skipprof widget field
;    SH, Aug 01            --  fixed y-axis for rangegates<minimgsz and imgorder
;                              set to 1  
;    SH, Oct 01            --  replaced atan(pp) with atan(imaginary(pp),
;                              float(pp)) and other minor changes  
;    SH, Nov 01            --  replaced the workaround for the cursor display 
;                              during pause and replaced it with empty 
;                              command (with idl5.5 empty can be removed from
;                              nimage eq 1 but not for nimage eq 2);
;                              added work around for a memory leak problem in 
;                              the 'pause/cursor' section under Linux
;    SH, Apr 02            --  modified to work with the new data format; took
;                              care of averaging multiple noise samples
;                              (noisesamples > 1); added ImgOrd and Img buttons,
;                              added noise monitor, a few other modifications
;    SH, May 02            --  added profile time interval monitor; removed
;                              IMGORD keyword, added Order buttons to image
;                              windows; fixed limits sync between the 2 images;
;                              removed skipprof and added timeout editable field;
;                              increased visible windows to 450 pixels, added
;                              Leak monitor fields for H and V.
;    SH, Jun 02            --  added display for H- and V-Doppler velocities,
;                              removed TIMEOUT keyword; added range value
;                              display for pause/cursor
;    SH, Jul 03            --  fixed a bug in y-axis update when rangegatedelay
;                              changes
;    SH, Aug 03            --  modified mxdT field (timeint) to show 9999 for
;                              any occurance of negative time interval
;
; -------------------------------------------------------------------------
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrdisp_common_blocks

; Common blocks used by wcrdisp routines

; WCR parameters from WCR data header and some derived parameters
; (not all of them are used by the real time display but are included
;  for completeness; does not affect the efficiency)

common wcrdispp,  alg,aspectratio,averages,datablocksize,dftalg,              $
                  fftalg,fftbins,fftwindow,fileidstr,filenamel,               $
                  fstblocknum,headersize,kaspeed,lambda,                      $
                  lstblocknum,maxvel,noisesamples,npulse,numrec,polh,polv,    $
                  pp4alg,pp6alg,ppstrpol,prf,pulselength,rangegates,          $
                  rangegatedelay,rangegatespacing,rxsequence,sppalg,starttime,$
                  stoptime,timelength,timeinterval,tnumrec,tind,txsequence,   $
                  txstrpol,wcrdate,wcrfile,wcrflag,filehdrsz

; WCR raw data arrays and some time/range parameters
; (leakage arrays are not used; an integral parameter for the leakage is
;  calculated in the data reading routine and passed to the display for
;  monitoring purposes)

common wcrdispd,  vvnoise, hhnoise, ppnoise, fftnoise,       $
                  ; vvtx, hhtx, vvlk, hhlk, pptx, ffttx,     $
                  hh, vv, vh, hv, pp, fftps,                 $
                  time, prof, begtime, endtime, radrng, velos, velo

; Calibration parameters and arrays with calibrated data used in the real-time
; display (hhz,vvz,zdr,ldr,dvh,dvv,vhz; vhz is used for vh or hv as well as for
; doppler width under fft mode))

common wcrdispcal,hhcal,hhcald,vvcal,vvcald,h_slope,v_slope,ldrcal,            $
                  ldrcallin,zdrcal,minfsize,minimgsz,maximgsz,nosignal,avestep,$
                  zero,msz1,loopstep,loopstepd,dml,rngcor,hhz,vvz,zdr,ldr,dvh, $
                  dvv,vhz,velfactor,blocksize,isol,isoldb,isoldbd,alphaiso,    $
                  fftcor,Nch,Nmode,Nm2,indh,indv,swapbytes,sleept

; Widget arrays and parameters

common wcrdispwdg,image1,image2,axsize,bxsize,wxsize,ysize,ct2ind,    $
                  zzmin,zzmax,zzmins,zzmaxs,ldmin,ldmax,ldmins,ldmaxs,$
                  zdmin,zdmax,zdmins,zdmaxs,dvmin,dvmax,dvmins,dvmaxs,$
                  czmin,czmax,czmins,czmaxs,svmin,svmax,svmins,svmaxs,$
                  mxhn,mxvn,timeint,timeintm,vleak

; Widget ids

common wcrdispids,baseid,axis1id,scroll1id,bar1id,b1maxid,b1labelid,b1minid,   $
                  axis2id,scroll2id,bar2id,b2maxid,b2labelid,b2minid,          $
                  scrollw1,scrollw2,barw1,barw2,helpid,                        $
                  quitid,openid,pauseid,colorid,destroyid,catchid,             $
                  inittid,hhcalid,vvcalid,isolid,startid,stepid,aveid,nimgid,  $
                  fill_thackid,sleepid,hh1id,vv1id,ldr1id,zdr1id,dvh1id,dvv1id,$
                  vh1id,img1ordid,hh2id,vv2id,ldr2id,zdr2id,dvh2id,dvv2id,     $
                  vh2id,img2ordid,btlabelid,etlabelid,fileid,msgbaseid,        $
                  cancelid,statusmsgid,statmnhnid,statmxhnid,statmnvnid,       $
                  statmxvnid,stathleakid,statvleakid,statdtid

; Real-time widget control parameters

common wcrdispctrl,lun1,wcrdir,wcrfileold,openwf,quitw,destroyw,destroym,  $
                  catche,rgold,rgsold,rgdold,starte,dispc,inittime,        $
                  scrollw1e,scrollw2e,itime,ivalue,nimg,dspctrl,           $
                  img1order,img2order 

end ; wcrdisp_common_blocks

; -------------------------------------------------------------------------
; Real-time widget display color bar

pro wcrdispbar,tbln,tbli,rg,bxsz,vmin=vmin,vmax=vmax
   common wcrdispcal
   y=rg>minimgsz
   r=findgen(y)*(vmax-vmin)/y+vmin
   r=r##(fltarr(bxsz)+1)
   r=bytscl(r,min=vmin,max=vmax,top=tbli-1)+tbli*(tbln-1)
   tv,r
end ; wcrdispbar

; -------------------------------------------------------------------------
; Simple widget message box

pro wcrdispmsg, txt

   common wcrdispids

   msgbaseid=widget_base(title='WCR DISPLAY MESSAGE',/column)
   msgid=widget_label(msgbaseid,value=txt)
   cancelid=widget_button(msgbaseid,value='CANCEL',event_pro='wcrdispmsg_ev')

   widget_control,msgbaseid,/realize

end ; wcrdispmsg

; -------------------------------------------------------------------------

pro wcrdispmsg_ev, ev

   common wcrdispids
   common wcrdispctrl

   if ev.id eq cancelid then destroym=1B

end ; wcrdispmsg_ev

; -------------------------------------------------------------------------
; Select and open a WCR raw file.
; Check the existance of any data; if not wait for data

pro wcrdispopen,fn,data,fsize,firstopen=fo

  common wcrdispp
  common wcrdispd
  common wcrdispcal
  common wcrdispwdg
  common wcrdispids
  common wcrdispctrl

; Initilize

  if keyword_set(fo) then fo=1B else fo=0B
  ; header=lonarr(23)   ; I don't know why I added this.

; Select a file.

  OPENF: fsize=0L
         close,lun1
         fn=dialog_pickfile(/read,filter='?????-??-*', $
                   title='Select WCR File',path=wcrdir,get_path=dir)

; Take care if no file is selected

  if fn eq '' then begin
     if n_elements(wcrdir) ne 0 and n_elements(wcrfileold) ne 0 then $ 
       fn=wcrdir+wcrfileold
     destroym=2B & openwf=0B
     return
  endif

; Open the file and check if there is any data in it.  Allow to wait for data.

  openr,lun1,fn
  fstatus = fstat(lun1)
  fsize   = fstatus.size
  close,lun1
  if fsize le minfsize then begin

  ;  No data, wait sleept secs

     if fo eq 0B then $
        widget_control,statusmsgid,set_value='Empty file. Waiting for data...' $
     else wcrdispmsg,'Empty file. Waiting for data...'
     wait,sleept

  ;  Check for data again

     OPENFST:
         openr,lun1,fn
         fstatus = fstat(lun1)
         fsize   = fstatus.size
         close,lun1

         ;  If there is no data, wait. Allow to terminate waiting using Cancel.
         ;  For first time openning (no main widget yet) use message widget.

         if fsize le minfsize then $
            if fo eq 0B then begin
               widget_control,destroyid,set_value='Cancel'
               widget_control,openid,sensitive=0
               widget_control,quitid,sensitive=0
               widget_control,pauseid,sensitive=0
               widget_control,catchid,sensitive=0
               junkid=widget_event(baseid,/nowait)

               ;  If wait cancel is requested give control to main widget

               if destroyw eq 1B then begin
                  destroyw=0B & destroym=1B & openwf=0B
                  widget_control,openid,sensitive=1
                  widget_control,quitid,sensitive=1
                  widget_control,destroyid,set_value='Destroy'
                  widget_control,statusmsgid,set_value='Waiting for an event...'
                  return

               ;  otherwise wait for data

               endif else begin wait,sleept & goto,OPENFST & endelse
            endif else begin

            ;  For first time openning, check if wait termination is requested

               junkid=widget_event(msgbaseid,/nowait)
               if destroym eq 1B then begin
                  widget_control,msgbaseid,/destroy
                  return
               endif else begin wait,sleept & goto,OPENFST & endelse
            endelse $

  ; Data is available. Do some widget work and open the new wcr file

         else if fo eq 0B then begin
                 widget_control,destroyid,set_value='Destroy'
                 widget_control,openid,sensitive=1
                 widget_control,quitid,sensitive=1
              endif else widget_control,msgbaseid,/destroy
  endif

; Open wcr file and save wcr file name and location

  if fo eq 0B then $
     widget_control,statusmsgid,set_value='Opening a new WCR file ...'

  wcrdispopenf,fn,data,lun1
  wcrdir=dir

; Determine some params that will be handy later and initilize image arrays

  fstatus    = fstat(lun1)
  fsize      = fstatus.size                  ; current WCR file size
  avestep    = ceil(aspectratio)>1           ; # profs averaged before display
  velfactor  = maxvel/!pi
  ; starttime= strmid(starttime,0,8)         ; done in wcrdispopenf
  ; stoptime = strmid(stoptime,0,8)          ; done in wcrdispopenf

  hhz        = fltarr(maximgsz,rangegates)-150.
  vvz        = fltarr(maximgsz,rangegates)-150.
  zdr        = fltarr(maximgsz,rangegates)-150.
  dvh        = fltarr(maximgsz,rangegates)-50.
  dvv        = fltarr(maximgsz,rangegates)-50.
  ldr        = fltarr(maximgsz,rangegates)-150.
  vhz        = fltarr(maximgsz,rangegates)-150. ; used for hv or sv=std(fft)

  if alg eq fftalg or alg eq sppalg then begin
    ; For fftalg speed up processing by making a 3-D velocity range array

    if alg eq fftalg then velo=reform(velos#(fltarr(rangegates*loopstep)+1), $
                               fftbins,rangegates,loopstep)
    ; If SPPMAG assign image2 to be Doppler velocity (default is vv)                               
    if txsequence[0] eq polh then image2='dv_h' else image2='dv_v'

  endif

  ; Range correction for distributed targets and range in km

    rngcor=(fltarr(loopstep)+1.)#(20.*alog10(radrng)-60.)

  ; After all the necessary files are open remove the old wcr file name

    wcrfileold=wcrfile

end ; wcrdispopen

; -------------------------------------------------------------------------
; Read in main WCR parameters for a selected file.
; Create file association (assoc) which is later used for
; extracting the time stamps from the headers in real-time.
; (all information is taken from a data header; some of
; the derived parameters are not needed but are left for
; completeness and compatibility with wcrtools)

pro wcrdispopenf, fn,data,lun1

common wcrdispp
common wcrdispd
common wcrdispcal

openr,lun1,fn

; Some WCR file constant parameters
  badf       =  -999.     ; float bad value
  badi       =  -999      ; integer bad value
  fileidstr  =  'W20'     ; it is updated to the first 3 chars of the filename
  filenamel  =   20       ; this is fixed and used to update fileIDstr
  filehdrsz  =   206L     ; bytes; after March 2002
  headersize =   20L      ; bytes; after March 2002
  headervars =   5        ; number of 4 byte fields
  bind       =   0        ; blocksize var position in data header, after 3/02
  tind       = [1,2]      ; timestamp var positions in data header, after 3/02  
  noisesamples=   1       ; # noise samples (always 1 before May 1999)
                          ; it is updated from the header info
  sppalg     =    9
  pp4alg     =   14
  pp6alg     =   15
  fftalg     =   11
  dftalg     =   13
  polh       =   104 
  polv       =   118
  sppvars    =    4
  ppmvars    =   14
  npulse     =    4       ; Number of pulses in a burst it is updated if 6
  lambda     =    0.00316 ; MW length in [m]
  timemode   =0           ; initialize timemode of the time vector
  wcrflag   =[0,0,0]      ; wcrflag=# bad profiles at the end of the file
                          ; wcrflag[1]=1, data shift is suspected
                          ; wcrflag[2]=1, WCR computer date and timestamps date
                          ; don't match or IRIG connection/data was lost/corrupted

; Get WCR file name and WCR date

  fileidstr=strmid(fn,strlen(fn)-filenameL,3)        ; allows renaming

  wcrfile=strmid(fn,strpos(fn,fileidstr,/reverse_search),filenamel)
  if strmid(wcrfile,3,2) lt '90' then wcrdate='20'+strmid(wcrfile,3,2) $
  else wcrdate='19'+strmid(wcrfile,3,2)

  wcrdate=long(wcrdate+strmid(wcrfile,6,2)+strmid(wcrfile,9,2))
  if wcrdate lt 19900101 or wcrdate gt 20200101 then begin
     message,'WCR file naming problem warning: '+wcrfile,/info
  endif

  if wcrdate gt 20020331 then begin
    ; Read file header
    version         =long(1)     & readse,lun1,big_end=bgnd, version,swap=swapbytes
    prf             =lonarr(6)   & readse,lun1,big_end=bgnd, prf
    junk            =lonarr(6)   & readse,lun1,big_end=bgnd, junk
    txsequence      =bytarr(6)   & readse,lun1,big_end=bgnd, txsequence
    rxsequence      =bytarr(6,2) & readse,lun1,big_end=bgnd, rxsequence
    pulselength     =long(1)     & readse,lun1,big_end=bgnd, pulselength
    txsamples       =long(1)     & readse,lun1,big_end=bgnd, txsamples
    noisesamples    =long(1)     & readse,lun1,big_end=bgnd, noisesamples
    rangegates      =long(1)     & readse,lun1,big_end=bgnd, rangegates
    rangegatedelay  =long(1)     & readse,lun1,big_end=bgnd, rangegatedelay
    rangegatespacing=long(1)     & readse,lun1,big_end=bgnd, rangegatespacing
    junk            =lonarr(2)   & readse,lun1,big_end=bgnd, junk
    iffilter        =long(1)     & readse,lun1,big_end=bgnd, iffilter
    averages        =long(1)     & readse,lun1,big_end=bgnd, averages
    alg             =long(1)     & readse,lun1,big_end=bgnd, alg
    fftbins         =long(1)     & readse,lun1,big_end=bgnd, fftbins
    fftwindow       =long(1)     & readse,lun1,big_end=bgnd, fftwindow
    junk            =long(1)     & readse,lun1,big_end=bgnd, junk
    irigflag        =long(1)     & readse,lun1,big_end=bgnd, irigflag
    junk            =lonarr(18)  & readse,lun1,big_end=bgnd, junk
    datablocksize   =long(1)     & readse,lun1,big_end=bgnd, datablocksize

    ; Read first data header

    fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum
    startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt
 
    ; Tide up some of the header variables

    if alg eq pp6alg then npulse=6 $                        ; PPMAG6
    else if alg eq pp4alg or alg eq sppalg then npulse=4 $  ; SPPMAG/PPMAG
    else npulse=1                                           ; FFT, DFFT
    prf             =prf(0:npulse-1)

    irigflag        =(irigflag and 1)  ; 0 - host computer time, 1 - IRIG time
    pulselength     =pulselength*25
    rangegatedelay  =float(rangegatedelay)*3.75
    rangegatespacing=float(rangegatespacing)*3.75
    txsequence      =txsequence[0:npulse-1]
    if alg eq sppalg or alg eq fftalg or alg eq dftalg then begin  
      rxsequence    =txsequence[0]
      prf           =prf[0]
    endif else rxsequence =rxsequence[0:npulse-2>0,*]
    case iffilter of
      1: iffilter=5.01 ; MHz (old 5 MHz Chebychev filter,5.01 for identif.)
      2: iffilter=2.   ; MHz (2 MHz Bessel filter)
      3: iffilter=5.   ; MHz (5 MHz Bessel filter)
      5: iffilter=10.  ; MHz (10 MHz Bessel filter)
      else: iffilter=badf   ; unknown or not allowed
    endcase

    ; Check if time stamps match wcrdate

     caldat, startt[0]/86400d0+2440587.5d0, mm, dd, yyyy
     k=yyyy*10000L+mm*100+dd

    if wcrdate ne k then begin
       wcrflag[2]=1
       if wcrdate lt k-1 then begin
          message,'Using time stamps to define data date: '+num2str(k),/info
          wcrdate=k
       endif else $
          message,'Using file name to define data date: '+num2str(wcrdate),/info
    endif

  endif else begin
  ; --------------- header variables from old data formats --------------------
    bgnd=1
    filehdrsz  =   0
    headervars =   23
    headersize =   92
    polh       =    7
    polv       =   15
    bind       =   1
    tind       = [3,4]

  ; Read header data
    version         =long(1) & readse,lun1,big_end=bgnd,version,swap=swapbytes ; 0; file type
    fstblocknum     =long(1) & readse,lun1,big_end=bgnd, fstblocknum     ; 1; data block #
    datablocksize   =long(1) & readse,lun1,big_end=bgnd, datablocksize   ; 2; in bytes
    startt          =lonarr(2) & readse,lun1,big_end=bgnd, startt        ; 3; UTC [s, us]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 5; radar azimuth
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 6; radar elevation
    spare           =long(1) & readse,lun1,big_end=bgnd, spare         ; 7; proc.alg.flag ?

  ; Check if time stamps match wcrdate

    caldat, floor(startt[0]/86400d0)+2440588L, mm, dd, yyyy
    k=yyyy*10000L+mm*100+dd

    if wcrdate lt k-1 then wcrdate=k  ; use time stamp to extract date

  ; Check if before May 18 1999 (new WCR software) and process accordingly

  if (wcrdate lt 19990518) then begin
    pp4alg=7
    prf             =lonarr(4) & readse,lun1,big_end=bgnd, prf         ; 8; [n,n,n,n] Hz
    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength ; 12; n
    pulselength     =pulselength*25                         ;     n*50 [ns], N/A
    txsequence      =bytarr(4) & readse,lun1,big_end=bgnd, txsequence  ; 13;[n,n,n,n],H/V:7/15
    txsequence      =reverse(txsequence)
    pulsesampledelay=long(1) & readse,lun1,big_end=bgnd, pulsesampledelay; 14; n
    pulsesampledelay=(pulsesampledelay+2)*100               ;     (2+n)*100 [ns]
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates      ; 15;
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangegatedelay  ; 16; n
    rangegatedelay  =float(rangegatedelay)*3.75             ;     3.75*n [m]
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing; 17; n
    rangegatespacing=float(rangegatespacing)*3.75           ;     3.75*n [m]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 18; n (n+1)*2 [us]
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 19; 1:rx atten. on
    spare           =long(1) & readse,lun1,big_end=bgnd, spare           ; 20; rx IF filt., N/A
    averages        =long(1) & readse,lun1,big_end=bgnd, averages        ; 21;
    alg             =long(1) & readse,lun1,big_end=bgnd, alg             ; 22;    9,7,11

    ; Add fftbins and fftwindow for data taken after Oct 22, 1996

    if (wcrdate gt 19961022) then begin

       headersize =   headersize+8
       headervars =   headervars+2

       fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins   ; 23; number of bins (FFT modes)
       fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow ; 24; fft window: 0,1,2,3,4,5
                                                              ;     rect.,hann,taper,blackman,
                                                              ;     gauss,gaussian top
    endif else begin
       fftbins=averages*(alg eq fftalg)
       fftwindow=0
    endelse

    ; Pulse width was set manually; set it to -999
      pulselength=badi

  endif else begin   ; for data/header after May 18 1999

    headersize      =160       ; bytes
    headervars      =40        ; number of 4 byte fields

    prf             =lonarr(12) & readse,lun1,big_end=bgnd, prf          ; 8; [n,n,n,...] Hz

    txsequence   =bytarr(8) & readse,lun1,big_end=bgnd, txsequence ; 20;[s,s,s,s],'h','v'

    ; make it compatible with the previous versions
    for k=0,7 do $
        if txsequence[k] eq '104' then txsequence[k]=7 $
        else if txsequence[k] eq '118' then txsequence[k]=15 $
        else txsequence[k]=0   ; not used

    rxsequence =bytarr(16) & readse,lun1,big_end=bgnd, rxsequence        ; 22;[s,s,s,s],'h','v'

    ; follow the original convention for txsequence
    for k=0,15 do $
        if rxsequence[k] eq '104' then rxsequence[k]=7 $
        else if rxsequence[k] eq '118' then rxsequence[k]=15 $
        else rxsequence[k]=0   ; not used
    rxsequence=transpose(reform(rxsequence,8,2))

    pulselength     =long(1)   & readse,lun1,big_end=bgnd, pulselength  ; 26; n
    pulselength     =pulselength*25                        ;     n*50 [ns], 50
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare        ; 27;
    spare           =lonarr(1) & readse,lun1,big_end=bgnd, spare        ; 28;
    noisesamples    =long(1) & readse,lun1,big_end=bgnd, noisesamples   ; 29; n
    rangegates      =long(1) & readse,lun1,big_end=bgnd, rangegates     ; 30;
    rangegatedelay  =long(1) & readse,lun1,big_end=bgnd, rangeGateDelay ; 31; n
    rangegatedelay  =float(rangeGateDelay)*3.75            ;     3.75*n [m]
    rangegatespacing=long(1) & readse,lun1,big_end=bgnd, rangegatespacing; 32; n
    rangegatespacing=float(rangegatespacing)*3.75          ;     3.75*n [m]
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare        ; 33;
    spare           =fltarr(1) & readse,lun1,big_end=bgnd, spare        ; 34;
    spare           =long(1) & readse,lun1,big_end=bgnd,spare           ; 35; rx filt. code
    averages        =long(1) & readse,lun1,big_end=bgnd, averages       ; 36;
    ;alg.codes: SPPMAG: 9, FFT: 11, DPPMAG: 12, DPFFT: 13, PPMAG4: 14, PPMAG6: 15
    alg             =long(1) & readse,lun1,big_end=bgnd, alg            ; 37; 9,14(7),11,13,15

    fftbins  =long(1) & readse,lun1,big_end=bgnd,fftbins   ; 38; number of bins for FFT files
    fftwindow=long(1) & readse,lun1,big_end=bgnd,fftwindow ; 39; fft window: 0,1
                                                           ;     rectangular,hanning   
    ; need to check the valid prfs and change npulse if needed; 
    if alg eq pp6alg then begin                                 ; PPMAG6 
       txsequence=txsequence[0:5] & rxsequence=transpose(rxsequence[*,0:4])
       prf=prf(0:5) & npulse=6 
    endif else if alg eq pp4alg or alg eq sppalg then begin     ; SPPMAG/PPMAG   
       txsequence=txsequence[0:3] & 
       if alg eq pp4alg then begin
         rxsequence=transpose(rxsequence[*,0:2])
         prf=prf(0:3)
       endif else begin
         rxsequence=txsequence[0]
         prf=prf[0]
       endelse
    endif else begin                                            ; FFT, DFFT
      txsequence=txsequence[0] & rxsequence=txsequence[0]
      prf=prf[0] & npulse=1
    endelse

  endelse
endelse
; ---------------------------  End header data -------------------------------

; Some WCR file derivative parameters

  blocksize   = datablocksize+headersize                 ; record size in Bytes
  spare       = fstat(lun1)
  tnumrec     = long(strtrim(spare.size,2))/blocksize    ; total # of recs
  if spare.size ne tnumrec*blocksize+filehdrsz then wcrflag[0]=1
  numrec      = tnumrec                      ; the number of records to be read

; Create string for the transmitted pulse sequence

  txstrpol = strarr(npulse)
  indv   = where(txsequence eq polv)             ; Tx V-pol pulse indices
  if (indv[0] ne -1) then txstrpol[indv] = 'V'
  indh   = where(txsequence eq polH)             ; Tx H-pol pulse indices
  if (indh[0] ne -1) then txstrpol[indh] = 'H'

; Create string array for the received pulse pair sequence                      

  if alg eq sppalg then begin 
    if rxsequence eq polV then ppstrpol='VV' else ppstrpol='HH' 
    txstrpol=txstrpol[0]
  endif else begin
    if npulse gt 1 and wcrdate gt 19990518 then begin
      ppstrpol = strarr(npulse-1,2)
      ind      = where(rxsequence eq polV)
      if (ind[0] ne -1) then ppstrpol[ind] = 'V'
      ind      = where(rxsequence eq polH)
      if (ind(0) ne -1) then ppstrpol[ind] = 'H'
      ppstrpol=reform(ppstrpol[*,0]+ppstrpol[*,1])
    endif else ppstrpol=replicate(' ',npulse-1>1)
  endelse

; Define data association struture for random access to the header
; Access to the data is done differently in this version of wcrdisp

  Nch        = 2+(alg eq fftalg)*(fftbins-2) ; number of data channels
  Nmode      = 2*(alg eq sppalg) + $
               7*(alg eq pp4alg) + $
               11*(alg eq pp6alg)+ $
               1*(alg eq fftalg)
  Nm2        = Nmode/2

  data=assoc(lun1,{header: lonarr(headervars), $
                   dat:fltarr(Nch,noisesamples+1+rangegates,Nmode)},filehdrsz)

; Calculate start time and time interval (between profiles)
;
; starttime    - string, start recording date/time
; timeinterval - float, sampling interval in [sec]

  t=data[0]
  if swapbytes eq 1 then t=swap_endian(t)

; Prepare the accurate start time (to be used for time interval calc.)
  timeinterval=double(t.header[tind[0]])+double(t.header[tind[1]])/1.e6

  starttime=timeinterval mod 86400
  starttime=sec2timestr(starttime) ; $ ; use just hh:mm:ss
  ;         +'.'+ num2str(round(100*(starttime-long(starttime))),fr=0)

; Get last profile number

k=tnumrec-1
t=data[k]
if swapbytes eq 1 then t=swap_endian(t)

lstblocknum=t.header[bind]

; Calculate stop time, time interval (between profs), time length
;
;  stopTime     - string, stop recording date/time
;  timeLenght   - float, time duration in [sec] of all recorded profiles
;  timeInterval - float, sampling interval in [sec]
;  timeint      - integer, averaged sampling interval in ms for every display
;                 loop

k=tnumrec-1
t=data[k]
if swapbytes eq 1 then t=swap_endian(t)

timeinterval=float(abs(timeinterval-(double(t.header[tind[0]])+$
             double(t.header[tind[1]])/1.e6)))/(tnumrec-1.)
timelength=timeinterval*(tnumrec-1)

timeint=long(timeinterval*1000.)   ; used for dT monitor
timeintm=timeint                   ; save the max dT

stoptime=double(t.header[tind[0]] mod 86400)+double(t.header[tind[1]]/1e6)
stoptime=sec2timestr(stoptime) ; $                ; use just hh:mm:ss
;        +'.'+num2str(round(100*(stoptime-long(stoptime))),fr=0)

; Correct lstBlockNum if profiles went over 65536.

  lstBlockNum=fstBlockNum+tnumRec

; Estimate of raw data images aspect ratio

aspectratio = (rangegatespacing/(timeinterval*kaspeed))>1

; Form range gate vector (geometric center of the gate)

radrng = rangegatedelay+findgen(rangegates)*rangegatespacing

; Maximum Doppler velocity

maxvel=prf/4.*lambda

; Create velo for FFT and do some checking for maxvel

if (alg eq fftalg or alg eq dftalg) then begin

   ; FFT spectra are caclulated with the first bin being DC (0 frequency) then 
   ; +df to +Nyquist Freq.(Fn) and then from -Fn+df to -df (df - freq. step).
   ; In other words the fft hasn't been shifted. 
   ; Positive Doppler velocity (negative freq) is toward the radar (radar
   ; convention).

   ; Form velocity axis for non-shifted spectra.
   maxvel=maxvel[0]
   velo=[-findgen(fftbins/2+1),reverse(findgen(fftbins/2-1)+1)]*2.*maxvel/fftbins
endif

end ; wcrdispopenf

; -------------------------------------------------------------------------
; WCR data reader and calibration procedure
; Data are read in portions as defined by the display step (loopstep) and
; # of profs averaged (avestep, default is aspect ratio).  The process is
; governed by a profile counter k. An integral leakage parameter calc is
; passed to the widget display as a routine parameter rather than in a
; common block.

pro wcrdispread,k,calc,mnhn,mnvn

common wcrdispp
common wcrdispd
common wcrdispcal
common wcrdispwdg
common wcrdispctrl
common recrawtemp, recraw,recraw1  ; serves only this routine

m   =loopstep*avestep              ; total number of profiles to read

; Initilize data arrays if necessary

if (size(recraw,/dim))[0] ne m or calc eq -99. then begin

   recraw1={header:lonarr(headersize/4), $
            data:fltarr(Nch,noisesamples+1+rangegates,Nmode)}	
   recraw=replicate(recraw1,m)

   if alg eq fftalg then begin
     fftps=fltarr(fftbins,rangegates,loopstep)
     tmp1 =fltarr(fftbins,rangegates,m)
     vh   =fltarr(rangegates,loopstep)  ; used as a temporary work array
     velo =reform(velos#(fltarr(rangegates*loopstep)+1), $
                  fftbins,rangegates,loopstep)
   endif else begin
     tmp1 =fltarr(rangegates,m)
     vh   =fltarr(loopstep,rangegates)
     hv   =fltarr(loopstep,rangegates)
     pp   =complexarr(loopstep,rangegates)
   endelse
   tmp2   =fltarr(m)
   vvnoise=fltarr(loopstep)
   hhnoise=fltarr(loopstep)
   hh     =fltarr(loopstep,rangegates)
   vv     =fltarr(loopstep,rangegates)
endif

calc=0. ; initialize leak after using calc=-99. as a flag in wcrdispread
        ; for starting new file

; .........................................................................
; Read data from wcr file

  point_lun,lun1,filehdrsz+k*blocksize
  readu,lun1,recraw 
  
  if swapbytes eq 1 then recraw=swap_endian(recraw)

  ; Calculate average time interval for that data segment (in ms)

  ;timeint=double((recraw.header[tind[0]])[0])+double((recraw.header[tind[1]])[0])/1.e6
  ;timeint=long(1000.*abs(timeint-(double((recraw.header[tind[0]])[m-1])+$
  ;             double((recraw.header[tind[1]])[m-1])/1.e6))/(m-1.))

  junk1=m-1-(m mod 2)
  tmp2=1000.*((double((recraw.header[tind[0]])[1:junk1:2])+$
                          double((recraw.header[tind[1]])[1:junk1:2])/1.d6)-$
                         (double((recraw.header[tind[0]])[0:junk1:2])+$
                          double((recraw.header[tind[1]])[0:junk1:2])/1.d6))
  if min(tmp2) lt 0. then timeint=9999 else timeint=long(mean(tmp2))  

  if alg eq fftalg then begin                                ; FFT

    ; Extract spectra
    tmp1=reform(recraw.data[*,noisesamples+1:*,0])
    fftps=rebin(tmp1,fftbins,rangegates,loopstep)

    ; Extract leakage (use minimum to track a possible data shift)
    calc=10.*alog10(min(total(reform(recraw.data[*,noisesamples,0]),1)))+fftcor

    ; Extract noise and signal power from the spectra
    tmp2=total(reform(recraw.data[*,0:noisesamples-1,0]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    if txsequence[0] eq polh then begin
       hhnoise=rebin(tmp2,loopstep)
       hh=transpose(total(fftps,1))
    endif else begin
       vvnoise=rebin(tmp2,loopstep)
       vv=transpose(total(fftps,1))
    endelse

  endif else if alg eq sppalg then begin                     ; SPPMAG

    tmp2   =reform(recraw.data[0,0:noisesamples-1,Nm2:Nmode-1])
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    vvnoise=rebin(tmp2,loopstep)
    tmp2   =reform(recraw.data[1,0:noisesamples-1,Nm2:Nmode-1])
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    hhnoise=rebin(tmp2,loopstep)

    tmp2=reform(recraw.data[1,noisesamples,Nm2])   ;; LEAK H channel
    ;calc=10.*alog10(total(tmp2)/m)                ;; LEAK replaced w/ next line
    calc=10.*alog10(min(tmp2))      ;; LEAK, take min for shift identification
    vleak=10.*alog10(min(reform(recraw.data[0,noisesamples,Nm2]))) ;; LEAK V ch

    if txsequence[0] eq polv then begin
       tmp1=reform(recraw.data[0,noisesamples+1:*,1])
       vv  =transpose(rebin(tmp1,rangegates,loopstep))
       tmp1=reform(recraw.data[1,noisesamples+1:*,1])
       hv  =transpose(rebin(tmp1,rangegates,loopstep))
    endif else begin
       tmp1=reform(recraw.data[0,noisesamples+1:*,1])
       vh  =transpose(rebin(tmp1,rangegates,loopstep))
       tmp1=reform(recraw.data[1,noisesamples+1:*,1])
       hh  =transpose(rebin(tmp1,rangegates,loopstep))
    endelse

    pp=transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,0] $
       ),rangegates,loopstep),rebin(reform(recraw.data[1,noisesamples+1:*,0]), $
       rangegates,loopstep))))
    velfac=velfactor[0]

  endif else if alg eq pp4alg or alg eq pp6alg then begin    ; PPMAG

    tmp2   =mean(reform(recraw.data[0,0:noisesamples-1,Nm2:Nmode-1]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    vvnoise=rebin(tmp2,loopstep)
    tmp2   =mean(reform(recraw.data[1,0:noisesamples-1,Nm2:Nmode-1]),1)
    if noisesamples gt 1 then tmp2=mean(tmp2,1)
    hhnoise=rebin(tmp2,loopstep)

    tmp2=reform(recraw.data[1,noisesamples,Nm2])   ;; LEAK H channel
    ;calc=10.*alog10(total(tmp2)/m)                ;; LEAK
    calc=10.*alog10(min(tmp2))      ;; LEAK, take min for shift identification
    vleak=10.*alog10(min(reform(recraw.data[0,noisesamples,Nm2]))) ;; LEAK V ch

    tmp1=mean(reform(recraw.data[0,noisesamples+1:*,indv+Nm2]),2)
    vv  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[1,noisesamples+1:*,indv+Nm2]),2)
    hv  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[0,noisesamples+1:*,indh+Nm2]),2)
    vh  =transpose(rebin(tmp1,rangegates,loopstep))
    tmp1=mean(reform(recraw.data[1,noisesamples+1:*,indh+Nm2]),2)
    hh  =transpose(rebin(tmp1,rangegates,loopstep))

    ; get one pulse pair for HH and one for VV
    
    junk1=where(ppstrpol eq 'HH')
    if junk1[0] ne -1 then begin
      junk2=(where(prf[junk1] eq max(prf[junk1])))[0]
      velfac=velfactor[junk1[junk2]]
      tmp1=transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,$
        junk1[junk2]]),rangegates,loopstep),rebin(reform(recraw.data[1, $
        noisesamples+1:*,junk1[junk2]]),rangegates,loopstep))))
      junk1=where(ppstrpol eq 'VV')
      if junk1[0] ne -1 then begin
        junk2=(where(prf[junk1] eq max(prf[junk1])))[0]
        pp=[[[transpose(reform(complex(rebin(reform(recraw.data[0, $
          noisesamples+1:*,junk1[junk2]]),rangegates,loopstep),  $
          rebin(reform(recraw.data[1,noisesamples+1:*,junk1[junk2]]),$
          rangegates,loopstep))))]],[[tmp1]]]
        velfac=[velfactor[junk1[junk2]],velfac]  
      endif else begin 
        pp=tmp1
      endelse
    endif else begin
      junk2=(where(prf eq max(prf)))[0]
      velfac=velfactor[junk2]
      pp=transpose(reform(complex(rebin(reform(recraw.data[0,noisesamples+1:*,junk2] $
        ),rangegates,loopstep),rebin(reform(recraw.data[1,noisesamples+1:*,junk2]),$
        rangegates,loopstep))))    
    endelse

  endif

; Calculate average noise and correct h/v noise (disabled)

  vvmn=total(vvnoise)/loopstep
; vvmn=hhmn^v_slope                          ; V-channel lookup table corr.
  mnvn=10.*alog10(vvmn)
  hhmn=total(hhnoise)/loopstep
; hhmn=hhmn^h_slope                          ; H-channel lookup table corr.
  mnhn=10.*alog10(hhmn)

; .........................................................................
; Calibrate data. H-channel is corrected (no cal. pulse or velocity bias
; correction is applied; no thresholding except for a minimum signal)

  if alg eq fftalg then begin
    ; avfac for fft is fftbin^2: dsp fft does not normilize by fftbins
      avfac=float(fftbins)*float(fftbins)

    if txstrpol[0] eq 'H' then begin
      hh =(hh-hhmn) > nosignal
;     hh =(hh^h_slope-hhmn) > nosignal                        ; H-channel corr.
      hhz=shift(hhz,-loopstep,0)
      hhz[dml:msz1,*]=10.*alog10(hh/avfac)+rngcor+hhcal+fftcor

    ; Calculate mean (dvh) and standard deviation (vhz) Doppler velocities
      dvh=shift(dvh,-loopstep,0)
      vh=total(fftps,1)  ; temp use as a work array
      fftps=velo*fftps   ; fftps is not needed anymore so use it as a work array
      dvh[dml:msz1,*]=transpose(total(fftps,1)/vh)
      vhz=shift(vhz,-loopstep,0)
      vhz[dml:msz1,*]=sqrt(transpose(total(velo*fftps,1)/vh)-dvh[dml:msz1,*]^2)
    endif else begin
      vv =(vv-vvmn) > nosignal
;     vv =(vv^v_slope-vvmn) > nosignal                        ; V-channel corr.
      vvz=shift(vvz,-loopstep,0)
      vvz[dml:msz1,*]=10.*alog10(vv/avfac)+rngcor+vvcal+fftcor

    ; Calculate mean (dvv) and standard deviation (vhz) Doppler velocities
      dvv=shift(dvv,-loopstep,0)
      vh=total(fftps,1)  ; temp use as a work array
      fftps=velo*fftps   ; fftps is not needed anymore so use it as a work array
      dvv[dml:msz1,*]=transpose(total(fftps,1)/vh)
      vhz=shift(vhz,-loopstep,0)
      vhz[dml:msz1,*]=sqrt(transpose(total(velo*fftps,1)/vh)-dvv[dml:msz1,*]^2)
    endelse

  endif else begin
    ldr=shift(ldr,-loopstep,0) ; get rid of the oldest/first loopstep points
    vhz=shift(vhz,-loopstep,0)
    
    if alg eq sppalg then  $
      if txstrpol[0] eq 'H' then begin
        dvh=shift(dvh,-loopstep,0)
        dvh[dml:msz1,*]=velfac[0]*atan(imaginary(pp),float(pp))
        hh =(hh-hhmn) > nosignal
;       hh =(hh^h_slope-hhmn) > nosignal                      ; H-channel corr.
        vh =10.*alog10((vh-vvmn-isol*ldrcallin*hh)>zero)      ; single iso corr.
;       vh =(vh^v_slope-vvmn) > nosignal                      ; H-channel corr.
        hh =10.*alog10(hh)
        hhz=shift(hhz,-loopstep,0)
        hhz[dml:msz1,*]=hh+rngcor+hhcal
        ldr[dml:msz1,*]=vh-hh-ldrcal
        vhz[dml:msz1,*]=vh+rngcor+vvcal
      endif else begin
        dvv=shift(dvv,-loopstep,0)
        dvv[dml:msz1,*]=velfac[0]*atan(imaginary(pp),float(pp))
        vv =(vv-vvmn) > nosignal
;       vv  =(vv^v_slope-vvmn) > nosignal                     ; H-channel corr.
        hv =(hv-hhmn-isol/ldrcallin*vv)>zero
;       hv =(hv^h_slope-hhmn-isol/ldrcallin*vv)>zero          ; single iso corr.
        hv =10.*alog10(hv)                                    ; H-ch. corr.
        vv =10.*alog10(vv)
        vvz=shift(vvz,-loopstep,0)
        vvz[dml:msz1,*]=vv+rngcor+vvcal
        ldr[dml:msz1,*]=hv-vv+ldrcal
        vhz[dml:msz1,*]=hv+rngcor+hhcal
      endelse $
    else begin
      if size(pp,/n_dim) eq 3 then begin
        dvh=shift(dvh,-loopstep,0)
        dvv=shift(dvv,-loopstep,0)
        dvh[dml:msz1,*]=velfac[1]*atan(imaginary(pp[*,*,1]),float(pp[*,*,1]))
        dvv[dml:msz1,*]=velfac[0]*atan(imaginary(pp[*,*,0]),float(pp[*,*,0]))
      endif else begin
        if (where(ppstrpol eq 'HH'))[0] ne -1 then begin
          dvh=shift(dvh,-loopstep,0)
          dvh[dml:msz1,*]=velfac[0]*atan(imaginary(pp),float(pp))
        endif else begin
          dvv=shift(dvv,-loopstep,0)
          dvv[dml:msz1,*]=velfac[0]*atan(imaginary(pp),float(pp))
        endelse
      endelse
      vv  =(vv-vvmn) > zero
;     vv  =(vv^v_slope-vvmn) > nosignal                        ; V-channel corr.
      hh  =(hh-hhmn) > nosignal
;     hh  =(hh^h_slope-hhmn) > nosignal                        ; H-channel corr.
      vh  =10.*alog10((vh-vvmn-alphaiso*isol*ldrcallin*hh $
                           -(1-alphaiso)*isol*vv) > zero)
;     vh  =(vh^v_slope-vvmn) > nosignal                        ; V-channel corr.
      vv  =10.*alog10(vv)
      hh  =10.*alog10(hh)
      hhz=shift(hhz,-loopstep,0)
      vvz=shift(vvz,-loopstep,0)
      zdr=shift(zdr,-loopstep,0)
      hhz[dml:msz1,*]=hh+rngcor+hhcal
      vvz[dml:msz1,*]=vv+rngcor+vvcal
      ldr[dml:msz1,*]=vh-hh-ldrcal
      zdr[dml:msz1,*]=hh-vv+zdrcal
      vhz[dml:msz1,*]=vh+rngcor+vvcal
    endelse
  endelse

end ; wcrdispread

; -------------------------------------------------------------------------
; Read time stamp for the current displayed block of profiles

function wcrdisptime,data,k,btime
   common wcrdispcal
   common wcrdispp
   if n_params() gt 2 then begin
      rec=k-dml*avestep > btime
   endif else rec=k

   t=data[rec]
   if swapbytes eq 1 then t=swap_endian(t)
   timelabel=sec2timestr(t.header[tind[0]] mod 86400)
   return,timelabel[0]
end ; wcrdisptime

; -------------------------------------------------------------------------
; Build real-time display widget

pro wcrdispwidget

common wcrdispp
common wcrdispd
common wcrdispcal
common wcrdispwdg
common wcrdispids
common wcrdispctrl

  ; Assign colorbar labels

  case image1 of
    'hh'   : begin & b1min=zzmins & b1max=zzmaxs & b1label=' hh(dBZ)' & end
    'vv'   : begin & b1min=zzmins & b1max=zzmaxs & b1label=' vv(dBZ)' & end
    'dv_h' : begin & b1min=dvmins & b1max=dvmaxs & b1label='dv_h(m/s)' & end
    'dv_v' : begin & b1min=dvmins & b1max=dvmaxs & b1label='dv_v(m/s)' & end
    'zdr'  : begin & b1min=zdmins & b1max=zdmaxs & b1label='zdr(dB)' & end
    'ldr'  : begin & b1min=ldmins & b1max=ldmaxs & b1label='ldr(dB)' & end
    'vh'   : begin & b1min=czmins & b1max=czmaxs & b1label=' vh(dBZ)' & end
    'sv'   : begin & b1min=svmins & b1max=svmaxs & b1label=' sv(m/s)' & end
  endcase

  case image2 of
     'hh'  : begin & b2min=zzmins & b2max=zzmaxs & b2label=' hh(dBZ)' & end
     'vv'  : begin & b2min=zzmins & b2max=zzmaxs & b2label=' vv(dBZ)' & end
     'dv_h': begin & b2min=dvmins & b2max=dvmaxs & b2label='dv_h(m/s)' & end
     'dv_v': begin & b2min=dvmins & b2max=dvmaxs & b2label='dv_v(m/s)' & end
     'zdr' : begin & b2min=zdmins & b2max=zdmaxs & b2label='zdr(dB)' & end
     'ldr' : begin & b2min=ldmins & b2max=ldmaxs & b2label='ldr(dB)' & end
     'vh'  : begin & b2min=czmins & b2max=czmaxs & b2label=' vh(dBZ)' & end
     'sv'  : begin & b2min=svmins & b2max=svmaxs & b2label=' sv(m/s)' & end
     else  :
  endcase

  ; Build widget structure

  baseid=widget_base(title='Wyoming Cloud Radar "Kestrel" REAL-TIME DISPLAY', $
                     /column)

  ; Main button group

    bg1id       =widget_base(baseid,/row,/frame)
       openid   =widget_button(bg1id,value='New',xsize=50,event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ',xsize=40)
       pauseid  =widget_button(bg1id,value='Pause/Cursor',event_pro='bg1_ev')
       catchid  =widget_button(bg1id,value='Catch',xsize=50,event_pro='bg1_ev')
       colorid  =widget_button(bg1id,value='Color',xsize=50,event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ',xsize=40)
       quitid   =widget_button(bg1id,value='Quit',xsize=50,event_pro='bg1_ev')
       destroyid=widget_button(bg1id,value='Destroy',xsize=70, $
                               event_pro='bg1_ev')
       dummyid  =widget_label(bg1id,value=' ',xsize=150)
       helpid   =widget_button(bg1id,value='Help',xsize=50,event_pro='bg1_ev')

  ; Dsiplay control group

    if dspctrl eq 0 then begin
     dispcid    =widget_base(baseid,column=9,/align_center)
       dispc8id =widget_base(dispcid,/column)
         dummyid=widget_label(dispc8id,value='Img')
         nimgid =widget_button(dispc8id,value=num2str(nimg),xsize=30, $
                               event_pro='dispc_ev')
       dispc1id =widget_base(dispcid,/column,/frame)
         inittid=widget_label(dispc1id,value='  InitTime  ')
         startid=widget_text(dispc1id,/edit,value=starttime,xsize=8, $
                             event_pro='bg1_ev')
       dispc2id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc2id,value='DispStep')
         stepid =widget_text(dispc2id,/edit,value=num2str(loopstep),xsize=8, $
                             event_pro='dispc_ev')
       dispc3id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc3id,value='AveProfs')
         aveid  =widget_text(dispc3id,/edit,value=num2str(avestep),xsize=8, $
                             event_pro='dispc_ev')
       dispc7id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc7id,value='Sleep')
         sleepid=widget_text(dispc7id,/edit,value=num2str(sleept,fr=1),xsize=4,$
                             event_pro='dispc_ev')
       dummyid =widget_base(dispcid,/column)
         dummyid=widget_label(dummyid,value='')
       dispc4id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc4id,value='hhcal')
         hhcalid=widget_text(dispc4id,/edit,value=num2str(hhcal,fr=1),xsize=8, $
                             event_pro='dispc_ev')
       dispc5id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc5id,value='vvcal')
         vvcalid=widget_text(dispc5id,/edit,value=num2str(vvcal,fr=1),xsize=8, $
                             event_pro='dispc_ev')
       dispc6id =widget_base(dispcid,/column,/frame)
         dummyid=widget_label(dispc6id,value='isol')
         isolid =widget_text(dispc6id,/edit,value=num2str(isoldb,fr=1),xsize=8,$
                             event_pro='dispc_ev')
    endif

  ; Top image (image1) widget

    if alg eq fftalg then vhlabel='sv' $
    else vhlabel='vh'
    image1id =widget_base(baseid,/column,/frame)
    bg2id=widget_base(image1id,/row)
      dummyid=widget_label(bg2id,value='     Range    |  Image:   ')
      bg2img1id=widget_base(bg2id,/row,/exclusive)
       hh1id =widget_button(bg2img1id,value='hh',xsize=50,event_pro='bg2_ev')
       vv1id =widget_button(bg2img1id,value='vv',xsize=50,event_pro='bg2_ev')
       vh1id =widget_button(bg2img1id,value=vhlabel,xsize=50,event_pro='bg2_ev')
       ldr1id=widget_button(bg2img1id,value='ldr',xsize=50,event_pro='bg2_ev')
       zdr1id=widget_button(bg2img1id,value='zdr',xsize=50,event_pro='bg2_ev')
       dvh1id=widget_button(bg2img1id,value='dv_h',xsize=55,event_pro='bg2_ev')
       dvv1id=widget_button(bg2img1id,value='dv_v',xsize=55,event_pro='bg2_ev')
      bg3img1id=widget_base(bg2id,/row)
       img1ordid=widget_button(bg3img1id,value='Order',xsize=50,event_pro='bg2_ev')
    base1id       =widget_base(image1id,/row)
      base11id    =widget_base(base1id,/row)
        axisid    =widget_base(base11id,/column)
          axis1id =widget_draw(axisid,xsize=axsize,ysize=ysize)
          dummyid =widget_label(axisid,value='kft    km')
        ; for next line y_scroll=ysize is what is needed; +1 is a MS Windows
        ; bug workaround 
        scroll1id =widget_draw(base11id,xsize=maximgsz,ysize=ysize, $
                               x_scroll=wxsize,y_scroll=ysize+1,retain=2)
        bar1id   =widget_draw(base11id,xsize=bxsize,ysize=ysize,/frame)
      base12id   =widget_base(base1id,/column)
        b1maxid  =widget_text(base12id,/align_left,value=b1max,xsize=5, $
                              /editable,event_pro='minmax_ev')
        dummyid   =widget_label(base12id,value=' ')
        if rangegates gt 100 then dummyid   =widget_label(base12id,value=' ')
        b1labelid =widget_label(base12id,value=b1label)
        dummyid   =widget_label(base12id,value=' ', $
                   scr_ysize=((rangegates>minimgsz)-100)>30)
        b1minid   =widget_text(base12id,/align_left,value=b1min,xsize=5, $
                              /editable,event_pro='minmax_ev')

  ; Time labels and wcr info text

    timeid     =widget_base(baseid,/row,/frame)
      btlabelid=widget_label(timeid,/frame,value=' ',xsize=75)
      fileid   =widget_label(timeid,value=' ',/align_center,xsize=450)
      etlabelid=widget_label(timeid,/frame,value=starttime,xsize=75)

  ;  Bottom image (image2) widget

  if nimg eq 2 then begin
     image2id =widget_base(baseid,/column,/frame)
     bg2id=widget_base(image2id,/row)
      dummyid=widget_label(bg2id,value='     Range    |  Image:   ')
      bg2img2id=widget_base(bg2id,/row,/exclusive)
       hh2id =widget_button(bg2img2id,value='hh',xsize=50,event_pro='bg2_ev')
       vv2id =widget_button(bg2img2id,value='vv',xsize=50,event_pro='bg2_ev')
       vh2id =widget_button(bg2img2id,value=vhlabel,xsize=50,event_pro='bg2_ev')
       ldr2id=widget_button(bg2img2id,value='ldr',xsize=50,event_pro='bg2_ev')
       zdr2id=widget_button(bg2img2id,value='zdr',xsize=50,event_pro='bg2_ev')
       dvh2id=widget_button(bg2img2id,value='dv_h',xsize=55,event_pro='bg2_ev')
       dvv2id=widget_button(bg2img2id,value='dv_v',xsize=55,event_pro='bg2_ev')
      bg3img2id=widget_base(bg2id,/row)
       img2ordid=widget_button(bg3img2id,value='Order',xsize=50,event_pro='bg2_ev')
     base2id       =widget_base(image2id,/row)
       base21id    =widget_base(base2id,/row)
         axisid    =widget_base(base21id,/column)
           axis2id =widget_draw(axisid,xsize=axsize,ysize=ysize)
           dummyid =widget_label(axisid,value='kft    km')
         ; for next line y_scroll=ysize is what is needed; +1 is a MS Windows
         ; bug workaround 
         scroll2id =widget_draw(base21id,xsize=maximgsz,ysize=ysize, $
                                x_scroll=wxsize,y_scroll=ysize+1,retain=2)
         bar2id    =widget_draw(base21id,xsize=bxsize,ysize=ysize,/frame)
       base22id    =widget_base(base2id,/column)
         b2maxid   =widget_text(base22id,/align_left,value=b2max,xsize=5, $
                               /editable,event_pro='minmax_ev')
         dummyid   =widget_label(base22id,value=' ')
         if rangegates gt 100 then dummyid   =widget_label(base22id,value=' ')
         b2labelid =widget_label(base22id,value=b2label) 
         dummyid   =widget_label(base22id,value=' ', $
                    scr_ysize=((rangegates>minimgsz)-100)>30)
         b2minid   =widget_text(base22id,/align_left,value=b2min,xsize=5, $
                               /editable,event_pro='minmax_ev')
  endif

  ; Status message line

  statbaseid=widget_base(baseid,column=8)
    statbase1id =widget_base(statbaseid,/column)
      statlabelid=widget_label(statbase1id,value='Status:',/align_left)
      statusmsgid=widget_text(statbase1id,value=' ',xsize=43)
    statbase2id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase2id,value='<Hn>')
      statmnhnid =widget_text(statbase2id,value='-99.9',xsize=5)
    statbase3id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase3id,value='mxHn')
      statmxhnid =widget_text(statbase3id,value='-99.9',xsize=5, $
                              /editable,event_pro='minmax_ev')
    statbase4id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase4id,value='<Vn>')
      statmnvnid =widget_text(statbase4id,value='-99.9',xsize=5)
    statbase5id =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase5id,value='mxVn')
      statmxvnid =widget_text(statbase5id,value='-99.9',xsize=5, $
                              /editable,event_pro='minmax_ev')
    statbase6id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase6id,value='Hleak')
      stathleakid=widget_text(statbase6id,value='-99.9',xsize=4)
    statbase8id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase8id,value='Vleak')
      statvleakid=widget_text(statbase8id,value='-99.9',xsize=4)
    statbase7id  =widget_base(statbaseid,/column)
      dummyid    =widget_label(statbase7id,value='mxdT')
      statdtid   =widget_text(statbase7id,value='0',xsize=4, $
                             /editable,event_pro='minmax_ev')
end ; wcrdispwidget

; -------------------------------------------------------------------------
; Control widget events from 'color' table button, 'quit', 'new', 'destroy',
; 'pause/cursor', 'init time'

pro bg1_ev,ev

   common wcrdispp
   common wcrdispwdg
   common wcrdispids
   common wcrdispctrl

   case ev.id of
     colorid : begin
        widget_control,statusmsgid,set_value='Waiting for a new color table.'
        xloadct,group=baseid,/block
        ct2ind=wyoctind()
        if ct2ind eq -1 then ct2ind=!d.table_size
        wset,barw1
        if image1 eq 'dv_h' then wcrdispbar,(ct2ind lt !d.table_size)+1,      $
           ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax                     $
        else if image1 eq 'dv_v' then wcrdispbar,(ct2ind lt !d.table_size)+1, $
           ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax                     $
        else if image1 eq 'sv' then                                           $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax        $
        else if image1 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1,  $
           ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax                     $
        else if image1 eq 'hh' or image1 eq 'vv' then                         $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax        $
        else if image1 eq 'ldr' then                                          $
           wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax        $
        else if image1 eq 'vh' then                                           $
             wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax

        if image2 ne ' ' then begin
           wset,barw2
           if image2 eq 'dv_h' then wcrdispbar,(ct2ind lt !d.table_size)+1,    $
              ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax                   $
           else if image2 eq 'dv_v' then wcrdispbar,(ct2ind lt !d.table_size)  $
              +1,ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax                $
           else if image1 eq 'sv' then                                         $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax      $
           else if image2 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1,$
              ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax                   $
           else if image2 eq 'hh' or image2 eq 'vv' then                       $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax      $
           else if image2 eq 'ldr' then                                        $
              wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax      $
           else if image2 eq 'vh' then                                         $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
        endif
               end
     quitid    : quitw=1B
     openid    : openwf=1B
     catchid   : catche=1B
     startid   : starte=1B
     destroyid : destroyw=1B
     pauseid   : begin
                 widget_control,pauseid,get_value=bl
                 if bl eq 'Continue' then begin
                    widget_control,pauseid,set_value='Pause/Cursor'
                    widget_control,pauseid,sensitive=1
                    widget_control,quitid,sensitive=1
                    widget_control,openid,sensitive=1
                    widget_control,destroyid,sensitive=1
                    return
                 endif
                 widget_control,pauseid,set_value='Continue'
                 widget_control,quitid,sensitive=0
                 widget_control,openid,sensitive=0
                 widget_control,destroyid,sensitive=0
               ;  widget_control,img1ordid,sensitive=0
                 end
     helpid    : begin

junk=["                     Description of WCR Display GUI                  ",$ 
"                                                                           ",$
"For more help on the use of wcrdisp, exit the program and run wcrdisp,/help",$
"from the IDL prompt. To exit this help press 'Done with WCR Display Help'  ",$
"                                                                           ",$
"This is a blocking widget and only one copy per idl session is allowed.    ",$
"                                                                           ",$
"'Enter' from the keyboard must be used after modifying an editable field   ",$
" for the new value to take effect                                          ",$
"                                                                           ",$
"Widget user interface:                                                     ",$
"                                                                           ",$
"New:          Open a dialog_pickfile to select a new WCR file.             ",$
"                                                                           ",$
"Quit:         Terminates the program without destroying the widget.        ",$
"              This could be be used to preserve interesting images for     ",$
"              later review and notes.                                      ",$
"                                                                           ",$
"Destroy:      Terminates the program and destroys the widget               ",$
"                                                                           ",$
"Pause/Cursor: Pressing this button will pause the display.  It also allows ",$
"              to extract image values by clicking the left button of the   ",$
"              mouse at a desired pixel of the images.  In case of two      ",$
"              images, clicking on either image will also extract the       ",$
"              corresponding value from the other image.  The values and    ",$
"              their time stamp are printed in the status line              ",$
"Continue:     Continue label replaces Pause/Cursor label when Pause is     ",$
"              active.  Pressing Continue will restore updating the display ",$
"              with new data.                                               ",$
"                                                                           ",$
"Catch:        Pressing this button will force the display to position      ",$
"              at the end of the currently available data in the wcr file.  ",$
"              This feature is useful if the display get behind the         ",$
"              incoming data (this could happen for various reasons,        ",$
"              including using Pause/Cursor)                                ",$
"                                                                           ",$
"Color:        Allow change in the color table being used.  The default     ",$
"              color table is a custom dual-table to allow different colors ",$
"              for, e.g., reflectivity and velocity images.  If the default ",$
"              color table is replaced the program checks if a narrow black ",$
"              strip exists in the middle of the new color table. If it does",$
"              the color table splits in two otherwise the same color       ",$
"              table is used for both images                                ",$
"                                                                           ",$
"Img:          Pressing this button will toggle between 1 and 2 image widget",$
"                                                                           ",$
"InitTime:     Editable field.  Contains the starting time of wcr data.     ",$
"              Editing the field will cause the display to start again      ",$
"              from time close to the entered time.  This field should      ",$
"              not be used during data acquisition.  It is intended         ",$
"              for browsing of already recorded files.  The new time must   ",$
"              have one of the following formats: hhmm, hh:mm, hhmmss,      ",$
"              hh:mm:ss, 9999 . Incorrect entries are ignored and the start ",$
"              time is redisplayed. If enter 9999 the field will show the   ",$
"              end time of the file.                                        ",$
"                                                                           ",$
"DispStep:     Editable field. Number of profiles to be calibrated and      ",$
"              displayed simultaneously.  Must be greater than 0. Non-valid ",$
"              entries are ignored and the default value is used.           ",$
"                                                                           ",$
"AveProfs:     Editable field. Number of raw profiles averaged before       ",$
"              display.  The default is aspect ratio which scales range:time",$
"              to 1:1 based on acspeed (default 90 m/s).  Must be greater   ",$
"              than 0.  It is not allowed both DispStep and AveProfs to be  ",$
"              equal to 1.  In this case the second one to be changed to 1  ",$
"              will be automatically replaced by 2.                         ",$
"                                                                           ",$
"Sleep:        Editable field. Sleep time[s] between disk access attempts   ",$
"              for 'Empty file' or 'End of file. Waiting for data...'       ",$
"              conditions. Default is 5 sec, maximum is 60 sec.             ",$
"                                                                           ",$
"hhcal:        Editable field. Calibration constant (dB) for H-channel      ",$
"vvcal:        Editable field. Calibration constant (dB) for V-channel      ",$
"isol:         Editable field. Isolation (dB) between H- and V-channel      ",$
"                                                                           ",$
"Radar products (image selection):                                          ",$
"               availability depends on the wcr operational mode            ",$
"   hh:   co-reflectivity for H-ch (dBZ)                                    ",$
"   vv:   co-reflectivity for V-ch (dBZ)                                    ",$
"   vh:   cross-reflectivity; H-ch transmitted, V-ch received               ",$
"   ldr:  vh/hh or hv/vv in dB                                              ",$
"   zdr:  hh/vv in dB                                                       ",$
"   dv_h: uncorrected for the platform motion mean Doppler velocity in m/s  ",$
"         for H channel(antenna)                                            ",$
"   dv_v: uncorrected for the platform motion mean Doppler velocity in m/s  ",$
"         for V channel(antenna)                                            ",$
"   sv:   spectral width in m/s (for wcr FFT mode only)                     ",$
"  Order: Pressing this button will toggle image display order: bottom-     ",$
"         to-top or top-to-bottom (side/up antennas or down antennas)       ",$
"                                                                           ",$
"Color bar limits: these are editable fields and the default values         ",$
"                  can be replaced.  To activate the new entry 'Enter'      ",$
"                  key must be hit after typing the new limit.  There is    ",$
"                  no check for the validity of the entries.                ",$
"                  Non-numerical entries are replaced with 0 and a non-fatal",$
"                  error message is issued.                                 ",$
"Status:    Non editable infomation message                                 ",$
"  <Hn>:    Non editable information indicator for the averaged H-ch noise. ",$
"  mxHn:    Resetable indicator for the max <Hn> (click on the field and    ",$
"           press Enter)                                                    ",$ 
"  <Vn>:    Non editable information indicator for the averaged V-ch noise. ",$
"  mxVn:    Resetable indicator for the max <Vn>  (click on the field and   ",$
"           press Enter)                                                    ",$
"  Hleak:   Non editable information indicator for the leaked H-ch Tx power.",$
"           WCR Tx leaked power (in dBm) at 0 range. This indicator is used ",$
"           to help detect data acquisition bug which causes a shift in the ",$
"           range gates.  In this case the leak power will be close to the  ",$
"           noise level (around -65 dBm)                                    ",$
"  Vleak:   Non editable information indicator for the leaked V-ch Tx power.",$
"           WCR Tx leaked power (in dBm) at 0 range. This indicator can be  ",$
"           used together with Hleak to help detect a problem with Tx power ",$
"           for H or V channel.                                             ",$
"  mxdT:    Resetable maximum average time interval(in ms) between profiles;",$
"           If 9999 is displayed a negative time jump in the time stamps was",$
"           encountered."]

                 xdisplayfile,title='WCR Display Help', text=junk, $
                              group=baseid, /modal
                 end
     else      :
   endcase

end ; bg1_ev

; -------------------------------------------------------------------------
; Control widget events from image ranges (velocity, reflectivity data)

pro bg2_ev,ev

   common wcrdispp
   common wcrdispwdg
   common wcrdispids
   common wcrdispd       ; used by imgordid only
   common wcrdispcal     ; used by imgordid only
   common wcrdispctrl    ; used by imgordid only

   case ev.id of
      hh1id : begin
                  image1='hh' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b1maxid,set_value=zzmaxs
                  widget_control,b1minid,set_value=zzmins
                  widget_control,b1labelid,set_value=' hh(dBZ)'
              end
      vv1id : begin
                  image1='vv' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b1maxid,set_value=zzmaxs
                  widget_control,b1minid,set_value=zzmins
                  widget_control,b1labelid,set_value=' vv(dBZ)'
              end
     ldr1id : begin
                  image1='ldr' &  wset,barw1
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
                  widget_control,b1maxid,set_value=ldmaxs
                  widget_control,b1minid,set_value=ldmins
                  widget_control,b1labelid,set_value='ldr(dB)'
              end
     zdr1id : begin
                  image1='zdr' &  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=zdmin,vmax=zdmax
                  widget_control,b1maxid,set_value=zdmaxs
                  widget_control,b1minid,set_value=zdmins
                  widget_control,b1labelid,set_value='zdr(dB)'
              end
     dvh1id : begin
                  image1='dv_h' &  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvmin,vmax=dvmax
                  widget_control,b1maxid,set_value=dvmaxs
                  widget_control,b1minid,set_value=dvmins
                  widget_control,b1labelid,set_value='vel(m/s)'
              end
     dvv1id : begin
                  image1='dv_v' &  wset,barw1
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvmin,vmax=dvmax
                  widget_control,b1maxid,set_value=dvmaxs
                  widget_control,b1minid,set_value=dvmins
                  widget_control,b1labelid,set_value='vel(m/s)'
              end              
      vh1id : begin
                  wset,barw1
                  widget_control,vh1id,get_value=vhlabel
                  if alg eq fftalg then begin
                    image1='sv'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
                    widget_control,b1labelid,set_value=' '+vhlabel+'(m/s)'
                    widget_control,b1maxid,set_value=svmaxs
                    widget_control,b1minid,set_value=svmins
                  endif else begin
                    image1='vh'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
                    widget_control,b1labelid,set_value=' '+vhlabel+'(dBZ)'
                    widget_control,b1maxid,set_value=czmaxs
                    widget_control,b1minid,set_value=czmins
                  endelse
              end
 img1ordid  : begin
                 img1order=img1order eq 0
                 ; Update y-axes for image1

                 widget_control,axis1id,get_value=axisw1
                 wset, axisw1 & erase    ; go to axis top window

                 junkid=((minimgsz-rangegates)*rangegatespacing)>0
                 ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                             junkid*(img1order eq 1))/1000.	
                 plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                      xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 xl_cm=!d.x_size/!d.x_px_cm
                 tkly=-1./xl_cm            ; reasonable tick length

                 axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0, $
		         yrange=!y.crange*(img1order eq 0)+ $
                   reverse(!y.crange)*(img1order eq 1),ystyle=1,chars=1.1,/normal
                 plot,ax_yrange,/nodata,/noerase,/normal, $
                   xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0,ystyle=1, $
		         yrange=!y.crange*(img1order eq 0)+ $
                   reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal	 
              end
      hh2id : begin
                  image2='hh' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b2maxid,set_value=zzmaxs
                  widget_control,b2minid,set_value=zzmins
                  widget_control,b2labelid,set_value=' hh(dBZ)'
              end
      vv2id : begin
                  image2='vv' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax
                  widget_control,b2maxid,set_value=zzmaxs
                  widget_control,b2minid,set_value=zzmins
                  widget_control,b2labelid,set_value=' vv(dBZ)'
              end
     ldr2id : begin
                  image2='ldr' &  wset,barw2
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax
                  widget_control,b2maxid,set_value=ldmaxs
                  widget_control,b2minid,set_value=ldmins
                  widget_control,b2labelid,set_value='ldr(dB)'
              end
     zdr2id : begin
                  image2='zdr' &  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=zdmin,vmax=zdmax
                  widget_control,b2maxid,set_value=zdmaxs
                  widget_control,b2minid,set_value=zdmins
                  widget_control,b2labelid,set_value='zdr(dB)'
              end
     dvh2id : begin
                  image2='dv_h' &  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvmin,vmax=dvmax
                  widget_control,b2maxid,set_value=dvmaxs
                  widget_control,b2minid,set_value=dvmins
                  widget_control,b2labelid,set_value='vel(m/s)'
              end
     dvv2id : begin
                  image2='dv_v' &  wset,barw2
                  wcrdispbar,(ct2ind lt !d.table_size)+1,ct2ind,$
                              rangegates,bxsize,vmin=dvmin,vmax=dvmax
                  widget_control,b2maxid,set_value=dvmaxs
                  widget_control,b2minid,set_value=dvmins
                  widget_control,b2labelid,set_value='vel(m/s)'
              end              
      vh2id : begin
                  wset,barw2
                  widget_control,vh2id,get_value=vhlabel
                  if alg eq fftalg then begin
                    image2='sv'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax
                    widget_control,b2labelid,set_value=' '+vhlabel+'(m/s)'
                    widget_control,b2maxid,set_value=svmaxs
                    widget_control,b2minid,set_value=svmins
                  endif else begin
                    image2='vh'
                    wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
                    widget_control,b2labelid,set_value=' '+vhlabel+'(dBZ)'
                    widget_control,b2maxid,set_value=czmaxs
                    widget_control,b2minid,set_value=czmins
                  endelse
              end
 img2ordid  : begin
                 img2order=img2order eq 0
                 ; Update y-axes for image2 axis

                 widget_control,axis2id,get_value=axisw2
                 wset, axisw2 & erase

                 junkid=((minimgsz-rangegates)*rangegatespacing)>0
                 ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                             junkid*(img2order eq 1))/1000.	
                 plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                      xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 xl_cm=!d.x_size/!d.x_px_cm
                 tkly=-1./xl_cm            ; reasonable tick length

                 axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0, $
		         yrange=!y.crange*(img2order eq 0)+ $
                   reverse(!y.crange)*(img2order eq 1),ystyle=1,chars=1.1,/normal
                 plot,ax_yrange,/nodata,/noerase,/normal, $
                   xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                 axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0,ystyle=1, $
		         yrange=!y.crange*(img2order eq 0)+ $
                   reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal	 
              end
       else :
   endcase

end ; bg2_ev

; -------------------------------------------------------------------------
; Provide widget change activities depending on an event request:
; 'destroy', 'quit', 'open', 'catch', 'inittime'

pro check_ev,fn,data,fsize,strtp

   common wcrdispp
   common wcrdispcal
   common wcrdispids
   common wcrdispctrl

   if destroyw then begin                         ;  Destroy widget
      close, lun1
      widget_control, baseid, /destroy
      return
   endif else if quitw then begin                 ;  Quit widget
      close, lun1
      return
   endif else if openwf then begin                ;  Open new wcr file
      close,lun1
      strtp=0
      wcrdispopen,fn,data,fsize
      return
   endif else if catche then begin                ; Go to the end of current
      hhz[dml:msz1,*]=-150.                       ; data and draw a black strip
      vvz[dml:msz1,*]=-150.
      ldr[dml:msz1,*]=-150.
      zdr[dml:msz1,*]=-150.
      dvh[dml:msz1,*]=-50.
      dvv[dml:msz1,*]=-50.
      vhz[dml:msz1,*]=-150.

   endif else if starte then begin  ; Handle request for restart at new time

   ; If stoptime = starttime ignore the request (do nothing)

     if stoptime eq starttime then begin
        starte=0B
        widget_control,startid,set_value=starttime
     endif else begin

       ; Check if the new start time is correct; if not do nothing
       ; If new clear data arrays and start form the new time (approximately)

       widget_control,startid,get_value=newinittime  ; get the new time
       newinittime=newinittime[0]

       ; Check the format of the input (9999,hhmm,hhmmss,hh:mm,h:mm:ss,hh:mm:ss)

       if newinittime eq '9999' then newinittime=stoptime
       temp=strlen(newinittime)
       if temp gt 8 or temp lt 4 then newinittime='0' $
       else if temp eq 4 then $
         newinittime=strmid(newinittime,0,2)+':'+strmid(newinittime,2,2)+':00' $
       else if temp eq 5 then newinittime=newinittime+':00' $
       else if temp eq 6 then $
         newinittime=strmid(newinittime,0,2)+':'+strmid(newinittime,2,2)+':'+ $
                     strmid(newinittime,4,2)

       ; Check the validity of the new time and find the starting profile

       temp=timestr2sec(newinittime)
       if temp eq -1 or newinittime eq stoptime then begin
         starte=0B
         if temp eq -1 then $
            ; incorrect time string; don't do anything and restore starttime
            inittime=starttime $
         else inittime=stoptime
       endif else begin          ; check the new time and find the start profile
         junk1=timestr2sec(starttime)
         junk2=timestr2sec(stoptime)
         if junk1 gt junk2 then begin                    ; midnight was crossed
           if temp ge junk1 or temp lt junk2 then begin
           ; The new time is ok; find the starting profile
             if temp lt junk1 then temp=temp+86400.
             strtp = long((temp-junk1)/timeinterval)>0
             inittime=newinittime
             hhz[*]=-150.
             vvz[*]=-150.
             ldr[*]=-150.
             zdr[*]=-150.
             dvh[*]= -50.
             dvv[*]= -50.
             vhz[*]=-150.
           endif else begin
           ; New time is beyond the available time range. Ignore it.
             starte = 0B
             inittime=starttime
           endelse
         endif else begin                           ; midnight was not crossed
           if temp ge junk1 and temp lt junk2 then begin
           ; new time is ok
             strtp = long((temp-junk1)/timeinterval)>0
             inittime=newinittime
             hhz[*]=-150.
             vvz[*]=-150.
             ldr[*]=-150.
             zdr[*]=-150.
             dvh[*]= -50.
             dvv[*]= -50.
             vhz[*]=-150.
           endif else begin
           ; New time is beyond the available time range. Ignore it.
             starte = 0B
             inittime=starttime
           endelse
         endelse
       endelse
       widget_control,startid,set_value=inittime
       if inittime eq starttime then $
         widget_control,inittid,set_value='InitTime' $
       else if inittime eq stoptime then $
         widget_control,inittid,set_value='EndTime' $
       else widget_control,inittid,set_value='NewTime'
     endelse
   endif

end ; check_ev

; -------------------------------------------------------------------------
; Provides widget control for real-time display parameters:
; calibration constants,isolation,display step(llopstep), profile
; averaging (avestep)

pro dispc_ev, ev

  common wcrdispp
  common wcrdispd
  common wcrdispwdg
  common wcrdispcal
  common wcrdispids
  common wcrdispctrl

   widget_control, ev.id, get_value=temp

   if temp[0] eq '0' then temp=9999 else begin
      temp=float(temp[0])
      if temp le 0 then temp=9999
   endelse

   dispc=1B
   case ev.id of
      hhcalid : begin
                  if temp eq 9999 then begin
                     hhcal=hhcald
                     widget_control,hhcalid,set_value=num2str(hhcal,fr=1)
                  endif else hhcal=temp
                  ldrcal=hhcal-vvcal
                  zdrcal=ldrcal
                end
      vvcalid : begin
                  if temp eq 9999 then begin
                     vvcal=vvcald
                     widget_control,vvcalid,set_value=num2str(vvcal,fr=1)
                  endif else vvcal=temp
                  ldrcal=hhcal-vvcal
                  zdrcal=ldrcal
                end
       isolid : begin
                  if temp eq 9999 then begin
                     isoldb=isoldbd
                     widget_control,isolid,set_value=num2str(isoldb,fr=1)
                  endif else isoldb = temp
                  isol   = 10.^(0.1*(-isoldb))
                end
       stepid : begin
                  if temp eq 9999 then loopstep=loopstepd $
                  else loopstep=(fix(temp)>(1+(avestep eq 1)))<maximgsz
                  widget_control,stepid,set_value=num2str(loopstep)
                  dml     =maximgsz-loopstep
                  rngcor  =(fltarr(loopstep)+1.)#(20.*alog10(radrng)-60.)
                end
        aveid : begin
                  if temp eq 9999 then $
			      avestep=ceil(aspectratio)>(1+(loopstep eq 1)) $
                  else avestep=(fix(temp)>(1+(loopstep eq 1)))<minimgsz
                  widget_control,aveid,set_value=num2str(avestep)
                end
      sleepid : begin
                  if temp eq 9999 then sleept=5. $
                  else sleept=temp<60.
                  widget_control,sleepid,set_value=num2str(sleept,fr=1)
                end
        nimgid: begin                    
                  nimg=(nimg eq 2)+2*(nimg eq 1)
                  widget_control,baseid,/destroy
                  wcrdispwidget
                  widget_control,baseid,/realize
                  widget_control,hh1id,sensitive=1
                  widget_control,vv1id,sensitive=1
                  widget_control,zdr1id,sensitive=1
                  widget_control,ldr1id,sensitive=1
                  widget_control,dvh1id,sensitive=1
                  widget_control,dvv1id,sensitive=1
                  widget_control,aveid,set_value=num2str(avestep)
                  widget_control,statmnhnid,set_value='-99.9'
                  widget_control,statmxhnid,set_value='-99.9'
                  widget_control,statmnvnid,set_value='-99.9'
                  widget_control,statmxvnid,set_value='-99.9'
                  widget_control,stathleakid,set_value='-99.9'
                  widget_control,statvleakid,set_value='-99.9'
                  widget_control,statdtid,set_value='0'
                  if dspctrl eq 0 then $
                     widget_control,startid,set_value=starttime
                  if nimg eq 2 then begin
                     widget_control,hh2id,sensitive=1
                     widget_control,vv2id,sensitive=1
                     widget_control,zdr2id,sensitive=1
                     widget_control,ldr2id,sensitive=1
                     widget_control,dvh2id,sensitive=1
                     widget_control,dvv2id,sensitive=1
                  endif
                  
                  if alg eq sppalg or alg eq fftalg then begin
                     widget_control,zdr1id,sensitive=0
                     if txstrpol[0] eq 'H' then begin
                       widget_control,vv1id,sensitive=0
                       widget_control,dvv1id,sensitive=0
                       widget_control,vh1id,set_value='vh'
                     endif else begin
                       widget_control,hh1id,sensitive=0
                       widget_control,dvh1id,sensitive=0
                       widget_control,vh1id,set_value='hv'
                     endelse
                     if alg eq fftalg then begin
                       widget_control,ldr1id,sensitive=0
                       widget_control,vh1id,set_value='sv'
                     endif
                     if nimg eq 2 then begin
                       widget_control,zdr2id,sensitive=0
                       if txstrpol[0] eq 'H' then begin
                         widget_control,vv2id,sensitive=0
                         widget_control,vh2id,set_value='vh'
                         widget_control,dvv2id,sensitive=0
                       endif else begin
                         widget_control,hh2id,sensitive=0
                         widget_control,vh2id,set_value='hv'
                         widget_control,dvh2id,sensitive=0
                       endelse
                       if alg eq fftalg then begin
                         widget_control,ldr2id,sensitive=0
                         widget_control,vh2id,set_value='sv'
                       endif
                     endif
                  endif

                  prfs=prf/1000

                  if alg eq fftalg then begin
                     prfs=num2str(prfs[0])
                     widget_control,fileid,set_value=wcrfile+'/FFT: '+txstrpol[0]+$
                     '/'+strmid(prfs,0,2)+'KHz/'+num2str(maxvel[0],fr=1)+'m/s'
                  endif else begin
                     strpols=string(reform(byte(txstrpol)))
                     prfs=string(reform(byte(num2str(prfs,len=3)),3*n_elements(prfs)))
                     widget_control,fileid, set_value=wcrfile+'/'+strpols+'/prf:'+$
                       prfs+'/#av:'+num2str(averages)+'/pulse:'+num2str(pulselength)+'/'
                  endelse   

                  widget_control,axis1id,get_value=axisw1
                  widget_control,scroll1id,get_value=scrollw1
                  widget_control,bar1id,get_value=barw1
                  wset,barw1
                  if image1 eq 'dv_h' then $
                     wcrdispbar,(ct2ind lt !d.table_size)+1,           $
                       ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax  $
                  else if image1 eq 'dv_v' then                        $
                     wcrdispbar,(ct2ind lt !d.table_size)+1,           $
                       ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax  $
                  else if image1 eq 'sv' then                          $
                     wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin, $
                       vmax=svmax                                      $
                  else if image1 eq 'zdr' then                         $
                     wcrdispbar,(ct2ind lt !d.table_size)+1,           $
                       ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax  $
                  else if image1 eq 'hh' or image1 eq 'vv' then        $
                     wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin, $
                       vmax=zzmax                                      $
                  else if image1 eq 'ldr' then                         $
                     wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin, $
                       vmax=ldmax                                      $
                  else if image1 eq 'vh' then                          $
                     wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin, $
                       vmax=czmax
                  widget_control,scroll1id,set_draw_view=[maximgsz,0]

                  if nimg eq 2 then begin
                     widget_control,axis2id,get_value=axisw2
                     widget_control,scroll2id,get_value=scrollw2
                     widget_control,bar2id,get_value=barw2
                     wset,barw2
                     if image2 eq 'dv_h' then $
                       wcrdispbar,(ct2ind lt !d.table_size)+1,          $
                         ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $
                     else if image2 eq 'dv_v' then                      $
                       wcrdispbar,(ct2ind lt !d.table_size)+1,          $
                         ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $                         
                     else if image2 eq 'sv' then                        $
                       wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,$
                         vmax=svmax                                     $
                     else if image2 eq 'zdr' then                       $
                       wcrdispbar,(ct2ind lt !d.table_size)+1,          $
                         ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax $
                     else if image2 eq 'hh' or image2 eq 'vv' then      $
                       wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,$
                         vmax=zzmax                                     $
                     else if image2 eq 'ldr' then                       $
                       wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,$
                         vmax=ldmax                                     $
                     else if image2 eq 'vh' then                        $
                       wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,$
                         vmax=czmax
                     widget_control,scroll2id,set_draw_view=[maximgsz,0]
                  endif

                  wset, axisw1 & erase
                  junkid=((minimgsz-rangegates)*rangegatespacing)>0
                  ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                         junkid*(img1order eq 1))/1000.	
                  plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                    xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                  xl_cm=!d.x_size/!d.x_px_cm
                  tkly=-1./xl_cm
                  axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly, $
                    yaxis=0,ystyle=1, yrange=!y.crange*(img1order eq 0)+ $
                    reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal
                  plot,ax_yrange,/nodata,/noerase,/normal, $
                    xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                  axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0, $
                    ystyle=1,yrange=!y.crange*(img1order eq 0)+$
                    reverse(!y.crange)*(img1order eq 1),chars=1.1,/normal
	 
                  if nimg eq 2 then begin
                    wset, axisw2 & erase
                    ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                      junkid*(img2order eq 1))/1000.	
                    axis,1.0,ax_yrange[0],/nodat,yticklen=tkly,yaxis=0,$
                      ystyle=1,yrange=!y.crange*(img2order eq 0)+ $
                      reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
                    plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                      xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
                    axis,0.45,ax_yrange[0]/0.305,/nodata,yticklen=tkly, $
                      yaxis=0,ystyle=1,yrange=!y.crange*(img2order eq 0)+ $
                      reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
	          endif
                end
         else : dispc=0B

   endcase

end ; dispc_ev

; -------------------------------------------------------------------------
; Provides widget control for real-time display parameters:
; min/max values for image ranges

pro minmax_ev, ev

   common wcrdispp
   common wcrdispwdg
   common wcrdispids

   widget_control, ev.id, get_value=minmaxs
   minmax=(float(minmaxs))[0]

   case ev.id of

      b1minid : begin
                  case image1 of
                    'hh'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'vv'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'dv_h' : begin dvmin=minmax & dvmins=minmaxs & end
                    'dv_v' : begin dvmin=minmax & dvmins=minmaxs & end
                    'zdr'  : begin zdmin=minmax & zdmins=minmaxs & end
                    'ldr'  : begin ldmin=minmax & ldmins=minmaxs & end
                    'vh'   : begin czmin=minmax & czmins=minmaxs & end
                    'sv'   : begin svmin=minmax & svmins=minmaxs & end
                    else   :
                  endcase
                  widget_control,b1minid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'dv_h' and image2 eq 'dv_v') or $
                     (image1 eq 'dv_v' and image2 eq 'dv_h') then $
                     widget_control,b2minid,set_value=num2str(minmax,fr=0)
                end

      b1maxid : begin
                  case image1 of
                    'hh'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'vv'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'dv_h' : begin dvmax=minmax & dvmaxs=minmaxs & end
                    'dv_v' : begin dvmax=minmax & dvmaxs=minmaxs & end
                    'zdr'  : begin zdmax=minmax & zdmaxs=minmaxs & end
                    'ldr'  : begin ldmax=minmax & ldmaxs=minmaxs & end
                    'vh'   : begin czmax=minmax & czmaxs=minmaxs & end
                    'sv'   : begin svmax=minmax & svmaxs=minmaxs & end
                    else   :
                  endcase
                  widget_control,b1maxid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'dv_h' and image2 eq 'dv_v') or $
                     (image1 eq 'dv_v' and image2 eq 'dv_h') then $
                     widget_control,b2maxid,set_value=num2str(minmax,fr=0)
                end

      b2minid : begin
                  case image2 of
                    'hh'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'vv'   : begin zzmin=minmax & zzmins=minmaxs & end
                    'dv_h' : begin dvmin=minmax & dvmins=minmaxs & end
                    'dv_v' : begin dvmin=minmax & dvmins=minmaxs & end
                    'zdr'  : begin zdmin=minmax & zdmins=minmaxs & end
                    'ldr'  : begin ldmin=minmax & ldmins=minmaxs & end
                    'vh'   : begin czmin=minmax & czmins=minmaxs & end
                    'sv'   : begin svmin=minmax & svmins=minmaxs & end
                    else   :
                  endcase
                  widget_control,b2minid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'dv_h' and image2 eq 'dv_v') or $
                     (image1 eq 'dv_v' and image2 eq 'dv_h') then $
                     widget_control,b1minid,set_value=num2str(minmax,fr=0)
                end

      b2maxid : begin
                  case image2 of
                    'hh'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'vv'   : begin zzmax=minmax & zzmaxs=minmaxs & end
                    'dv_h' : begin dvmax=minmax & dvmaxs=minmaxs & end
                    'dv_v' : begin dvmax=minmax & dvmaxs=minmaxs & end
                    'zdr'  : begin zdmax=minmax & zdmaxs=minmaxs & end
                    'ldr'  : begin ldmax=minmax & ldmaxs=minmaxs & end
                    'vh'   : begin czmax=minmax & czmaxs=minmaxs & end
                    'sv'   : begin svmax=minmax & svmaxs=minmaxs & end
                    else   :
                  endcase
                  widget_control,b2maxid,set_value=num2str(minmax,fr=0)
                  if image1 eq image2 or (image1 eq 'hh' and image2 eq 'vv') $
                     or (image1 eq 'vv' and image2 eq 'hh') or $
                     (image1 eq 'dv_h' and image2 eq 'dv_v') or $
                     (image1 eq 'dv_v' and image2 eq 'dv_h') then $
                     widget_control,b1maxid,set_value=num2str(minmax,fr=0)
                end
      statmxhnid: begin
                  mxhn=-99.
                  widget_control,statmxhnid,set_value=num2str(mxhn)
                end
      statmxvnid: begin
                  mxvn=-99.
                  widget_control,statmxvnid,set_value=num2str(mxvn)
                end
      statdtid: begin
                  timeintm=0L
                  widget_control,statdtid,set_value=num2str(timeintm)
                end
      else :
   endcase

end ; minmax_ev

; -------------------------------------------------------------------------
; Widget control for top image scroll bar

pro scrollw1_ev,ev

   common wcrdispids
   common wcrdispctrl

   if ev.press then begin
      scrollw1e=1B
      itime=ev.x
      ivalue=ev.y
   endif

end ; scrollw1_ev

; -------------------------------------------------------------------------
; Widget control for bottom image scroll bar

pro scrollw2_ev,ev

   common wcrdispids
   common wcrdispctrl

   if ev.press then begin
      scrollw2e=1B
      itime=ev.x
      ivalue=ev.y
   endif

end ; scrollw2_ev

; -------------------------------------------------------------------------
; Main real-time display routine.  Real time display is controlled by a main
; loop which monitors the availability of data/new data in a raw WCR file.
; Thus it does not utilize widget high level xmanager function but the widget
; low level function widget_event.

pro wcrdisp, nimage,minmax=minmax,calval=calval,start=strtp,acspeed=acspeed,$
                    dispctrl=dispctrl,help=help,colortable=colortable
                    
  common wcrdispp
  common wcrdispd
  common wcrdispcal
  common wcrdispwdg
  common wcrdispids
  common wcrdispctrl

; .........................................................................

if keyword_set(help) then begin
  doc_library,'wcrdisp'
  return
endif

; Initilize

  on_error,2

  ; Set display control pannel.
  
  if keyword_set(dispctrl) then dspctrl=1 else dspctrl=0

  ; Depending on the input assign default image1 and image2

  if n_params() eq 0 then begin
     nimage=2 & image1='hh' & image2='vv'
  endif else if nimage eq 2 then begin
     image1='hh' & image2='vv'
  endif else if nimage eq 1 then begin
    image1='hh' & image2=' '
  endif else begin
     nimage=2 & image1='hh' & image2='vv'
  endelse

  nimg=nimage

  ; Load color table

  if n_elements(colortable) eq 0 then begin
     loadct,0,get_names=junkid
     if n_elements(junkid) lt 46 then colortable=15 $
     else colortable=45  ; WYO12 dual color table
     loadct,colortable
  endif else if colortable ge 0 then loadct,colortable

  if (colortable eq 45 or colortable eq 46) and !D.N_COLORS lt 64 then $
     colortable=41

  ; If dual color table find starting index for the second subcolor table

  ct2ind=wyoctind()
  if ct2ind eq -1 then ct2ind=!d.table_size

  ; Start from a given profile if requested

  if n_elements(strtp) eq 0 then strtp=0

  ; Check sleep time and loopstep. For WASP set default to 5s, otherwise 1s

  junkid=strmid(getenv('DISPLAY'),0,4)

  if junkid eq 'wasp' then begin
    sleept=5.                   ; empty/end file wait time in secs
    loopstep=25                 ; image display step, can't be byte -> shift !!!
  endif else begin
    sleept=1.
    loopstep=5
  endelse

  ; Calibration coefficients

  if n_elements(calval) eq 0 then begin
     hhcal     = 50.        ; dB
     vvcal     = 49.        ; dB
     iso       =-18.        ; dB
     vvcald=vvcal           ; dB
     hhcald=hhcal           ; dB
  endif else begin
     ncal=n_elements(calval)
     if (ncal lt 2 or ncal gt 3) then $
        message,"Wrong input for keyword CALVAL" $
     else begin
       hhcal   = calval[0]
       vvcal   = calval[1]
     endelse
     if ncal eq 2 then iso  = -18.             $
     else              iso  = -abs(calval[2])
  endelse

  ; Aircraft average air speed in m/s; used for determining approximate aspectratio
  ; of radar range resolution vs. radar profile resolution in meters
  
  if n_elements(acspeed) eq 0 then kaspeed=90. else kaspeed=acspeed
  
  ; Assign some global values

  maximgsz    =600            ; image (scroll window) max x-size
  minimgsz    =100            ; image (scroll window) min y-size
  minfsize    =0L             ; minimum file size to open or read update
  nimagei     =nimage         ; save the initially requested # of images
  maxrg       =250            ; maximum range gates for nimage=2 
  axsize      =75B            ; axis windows x_size
  wxsize      =450            ; scroll windows x_size
  bxsize      =15B            ; color bar windows x_size
  ysize       =minimgsz       ; all windows y_size
  loopstepd   =loopstep
  destroyw    =0B             ; close wcr and destroy widget
  destroym    =0B             ; destroy message widget
  quitw       =0B             ; close wcr file and quit
  openwf      =0B             ; open new wcr file
  catche      =0B             ; go to the end of current data
  scrollw1e   =0B             ; image1 cursor control variable
  scrollw2e   =0B             ; image2 cursor control variable
  img1order   =0              ; default, bottom-top image 1 order
  img2order   =0              ; default, bottom-top image 2 order
  rgold       =0              ; # of range gates
  rgsold      =0              ; range gate spacing in m
  rgdold      =0              ; range gate delay in m
  wcrfileold  =''             ; WCR file name
  dml         =maximgsz-loopstep
  msz1        =maximgsz-1
  startid     =999            ; work around for a bug in HP IDL 4.0
  ivalue      =0              ; initialize cursor range position

  get_lun,lun1                ; get WCR file lun

  fileidstr = ''              ; defined after first file open
  nosignal  = 1.e-15          ; min signal level in mW
  zero      = 1.e-25          ; 0 level for data
  fftcor    = 30.             ; dB; error in fft wcr cal (in W instead of mW)
; h_slope   = 1./0.95         ; error in H-channel lookup table before 1999
  h_slope   = 1.              ; no correction after May 1999
  v_slope   = 1.              ; no correction
  alphaiso  = 0.5             ; split 2-way measured isolation to Tx and Rx iso.

  ldrcal    = hhcal-vvcal
  ldrcallin = 10.^(0.1*ldrcal)
  zdrcal    = hhcal-vvcal
  isoldb    = -iso            ; H/V channels isolation
  isoldbd   = isoldb
  isol      = 10.^(0.1*iso)

  ; Default min/max values for received power quantities

  zzmax =  10.                    ; co-pol,    dBZ
  zzmin = -45.                    ; co-pol,    dBZ
  zdmax =   4.                    ; zdr,       dBZ
  zdmin =  -4.                    ; zdr,       dBZ
  ldmax = -10.                    ; ldr,       dBZ
  ldmin = -25.                    ; ldr,       dBZ
  czmax = -10.                    ; cross-pol, dBZ
  czmin = -45.                    ; cros-pol,  dBZ

  ; Check for minmax input

  if n_elements(minmax) ge 2 then $
     if minmax[0] lt minmax[1] then begin
        zzmin=minmax[0] & zzmax=minmax[1]
     endif
  if n_elements(minmax) ge 4 then $
     if minmax[2] lt minmax[3] then begin
        zdmin=minmax[2] & zdmax=minmax[3]
     endif
  if n_elements(minmax) eq 6 then $
     if minmax(4) lt minmax[5] then begin
        ldmin=minmax[4] & ldmax=minmax[5]
  endif

  ; Convert min/max to strings for widget use

  zzmaxs=num2str(round(zzmax))
  if zzmin lt 0. and zzmax gt 0. then zzmaxs='+'+zzmaxs
  zzmins=num2str(round(zzmin))
  zdmaxs=num2str(round(zdmax))
  if zdmin lt 0. and zdmax gt 0. then zdmaxs='+'+zdmaxs
  zdmins=num2str(round(zdmin))
  ldmaxs=num2str(round(ldmax))
  if ldmin lt 0. and ldmax gt 0. then ldmaxs='+'+ldmaxs
  ldmins=num2str(round(ldmin))
  czmaxs=num2str(round(czmax))
  if czmin lt 0. and czmax gt 0. then czmaxs='+'+czmaxs
  czmins=num2str(round(czmin))

  ; Open WCR file

  wcrdispopen,fn,data,fsize,/firstopen
  if destroym ne 0B then begin
     return
     free_lun,lun1
  endif

  ; Default min/max Doppler velocity and Doppler width

    dvmax = max(maxvel)             ; mean vel, m/s
    dvmin = -dvmax                  ; mean vel, m/s
    svmax = dvmax                   ; vel std,  m/s
    svmin = 0                       ; vel std,  m/s

  ; Convert min/max Doppler to strings for widget use

    dvmaxs=num2str(round(dvmax))
    if dvmin lt 0. and dvmax gt 0. then dvmaxs='+'+dvmaxs
    dvmins=num2str(round(dvmin))

    svmaxs=num2str(round(svmax))
    if dvmin lt 0. and svmax gt 0. then svmaxs='+'+svmaxs
    svmins=num2str(round(svmin))

; .........................................................................

NEWFILE:     ; start new wcr file here (if requested by check_ev)

  ysize=rangegates>minimgsz
  calc =-99. ; current H leak, also flags new file for wcrdispread
  vleak=-99. ; current V leak
  mnhn =-99. ; initilize mean H noise
  mxhn =-99. ; initilize max H noise
  mnvn =-99. ; initilize mean V noise
  mxvn =-99. ; initilize max V noise
  timeint = 0L ; initilize dT (mean time interval for every display loop)
  timeintm= 0L ; initilize max dT

; Set up or update 1/2 scrolling window widget

  if openwf eq 0B then begin             ; first open: build and realize widget
     if rangegates gt maxrg then nimg=1 else nimg=nimagei
     wcrdispwidget
     widget_control,baseid,/realize
  endif else $
     if (rgold ne rangegates) then begin ; change in range gates: rebuild widget
        widget_control,baseid,/destroy
        if rangegates gt maxrg then nimg=1 else nimg=nimagei
        wcrdispwidget
        widget_control,baseid,/realize
     endif else begin                    ; use existing widget: initilize
        widget_control,hh1id,sensitive=1
        widget_control,vv1id,sensitive=1
        widget_control,zdr1id,sensitive=1
        widget_control,ldr1id,sensitive=1
        widget_control,dvh1id,sensitive=1
        widget_control,dvv1id,sensitive=1
        widget_control,aveid,set_value=num2str(avestep)
        widget_control,statmnhnid,set_value='-99.9'
        widget_control,statmxhnid,set_value='-99.9'
        widget_control,statmnvnid,set_value='-99.9'
        widget_control,statmxvnid,set_value='-99.9'
        widget_control,stathleakid,set_value='-99.9'
        widget_control,statvleakid,set_value='-99.9'
        widget_control,statdtid,set_value='0'
        if dspctrl eq 0 then widget_control,startid,set_value=starttime
        if nimg eq 2 then begin
           widget_control,hh2id,sensitive=1
           widget_control,vv2id,sensitive=1
           widget_control,zdr2id,sensitive=1
           widget_control,ldr2id,sensitive=1
           widget_control,dvh2id,sensitive=1
           widget_control,dvv2id,sensitive=1
        endif
    endelse

  ; Now that the widget is created set appropriate buttons

  if alg eq sppalg or alg eq fftalg then begin  ; SPPMAG or FFT
     widget_control,zdr1id,sensitive=0
     if txstrpol[0] eq 'H' then begin           ; HHHH
        widget_control,vv1id,sensitive=0
        widget_control,dvv1id,sensitive=0
        widget_control,vh1id,set_value='vh'
     endif else begin                           ; VVVV
        widget_control,hh1id,sensitive=0
        widget_control,dvh1id,sensitive=0
        widget_control,vh1id,set_value='hv'
     endelse
     if alg eq fftalg then begin            ; for FFT remove ldr, use vh for sv
        widget_control,ldr1id,sensitive=0
        widget_control,vh1id,set_value='sv'
     endif
     if nimg eq 2 then begin                ; repeat above for image2 (bottom)
        widget_control,zdr2id,sensitive=0
        if txstrpol[0] eq 'H' then begin
           widget_control,vv2id,sensitive=0
           widget_control,dvv2id,sensitive=0
           widget_control,vh2id,set_value='vh'
        endif else begin
           widget_control,hh2id,sensitive=0
           widget_control,dvh2id,sensitive=0
           widget_control,vh2id,set_value='hv'
        endelse
        if alg eq fftalg then begin
           widget_control,ldr2id,sensitive=0
           widget_control,vh2id,set_value='sv'
        endif
     endif
  endif

  ; Setup wcr info text

    prfs=prf/1000

    if alg eq fftalg then begin
      prfs=num2str(prfs[0])
      widget_control,fileid,set_value=wcrfile+'/FFT: '+txstrpol[0]+'/'+ $
        strmid(prfs,0,2)+'KHz/'+num2str(maxvel[0],fr=1)+'m/s'
    endif else begin
      strpols=string(reform(byte(txstrpol)))
      prfs=string(reform(byte(num2str(prfs,len=3)),3*n_elements(prfs)))
      widget_control,fileid, set_value=wcrfile+'/'+strpols+'/prf:'+$
        prfs+'/#av:'+num2str(averages)+'/pulse:'+num2str(pulselength)+'/'
    endelse

  ; Get window ids and draw color bars

    if openwf eq 0B or rgold ne rangegates then begin
       widget_control,axis1id,get_value=axisw1
       widget_control,scroll1id,get_value=scrollw1
       widget_control,bar1id,get_value=barw1
       wset,barw1
       if image1 eq 'dv_h' then wcrdispbar,(ct2ind lt !d.table_size)+1,        $
                                ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $
       else if image1 eq 'dv_v' then wcrdispbar,(ct2ind lt !d.table_size)+1,   $
                                ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $
       else if image1 eq 'sv' then                                             $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax    $
       else if image1 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1,    $
                                ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax $
       else if image1 eq 'hh' or image1 eq 'vv' then                           $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax    $
       else if image1 eq 'ldr' then                                            $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax    $
       else if image1 eq 'vh' then                                             $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax

       if nimg eq 2 then begin
          widget_control,axis2id,get_value=axisw2
          widget_control,scroll2id,get_value=scrollw2
          widget_control,bar2id,get_value=barw2
          wset,barw2
          if image2 eq 'dv_h' then wcrdispbar,(ct2ind lt !d.table_size)+1,     $
                                ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $
          else if image2 eq 'dv_v' then wcrdispbar,(ct2ind lt !d.table_size)+1,$
                                ct2ind,rangegates,bxsize,vmin=dvmin,vmax=dvmax $
          else if image2 eq 'sv' then                                          $
                wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=svmin,vmax=svmax    $
          else if image2 eq 'zdr' then wcrdispbar,(ct2ind lt !d.table_size)+1, $
                                ct2ind,rangegates,bxsize,vmin=zdmin,vmax=zdmax $
          else if image2 eq 'hh' or image2 eq 'vv' then                        $
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=zzmin,vmax=zzmax  $
          else if image2 eq 'ldr' then                                         $
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=ldmin,vmax=ldmax  $
          else if image2 eq 'vh' then                                          $
                  wcrdispbar,1,ct2ind,rangegates,bxsize,vmin=czmin,vmax=czmax
       endif
    endif

  ; Set the scroll bars to the right

    widget_control,scroll1id,set_draw_view=[maximgsz,0]
    if nimg eq 2 then widget_control,scroll2id,set_draw_view=[maximgsz,0]

  ; Plot y-axes in image1/image2 axis windows

    if (openwf eq 0B) or (openwf eq 1B and rgold ne rangegates) or $
       (rgsold ne rangegatespacing) or (rgdold ne rangegatedelay) then begin

        wset, axisw1 & erase    ; go to axis top window

        ;   Establish plot parameters; y-range is in kft

        junkid=((minimgsz-rangegates)*rangegatespacing)>0
        ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
                   junkid*(img1order eq 1))/1000.	
        plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
             xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
        xl_cm=!d.x_size/!d.x_px_cm
        tkly=-1./xl_cm            ; reasonable tick length

        ;   Plot axis in kft and in km after establishing new plot parameters

        axis,0.45,ax_yrange[0]/0.305,/nodat,ytickle=tkly,yaxis=0,ystyle=1, $
		 yrange=!y.crange*(img1order eq 0)+reverse(!y.crange)*(img1order eq 1),$
           chars=1.1,/normal
        plot,ax_yrange,/nodata,/noerase,/normal, $
             xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
        axis,1.0,ax_yrange[0],/nodata,yticklen=tkly,yaxis=0,ystyle=1, $
		 yrange=!y.crange*(img1order eq 0)+reverse(!y.crange)*(img1order eq 1),$
           chars=1.1,/normal
	 
        if nimg eq 2 then begin  ; for image 2 repeat the above
           wset, axisw2 & erase
           ax_yrange=([radrng[0],radrng[rangegates-1]+junkid]- $
              junkid*(img2order eq 1))/1000.	
           axis,1.0,ax_yrange[0],/nodat,yticklen=tkly,yaxis=0,ystyle=1, $
		    yrange=!y.crange*(img2order eq 0)+ $
              reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
		 plot,ax_yrange/0.305,/nodata,/noerase,/normal, $
                xstyle=5,ystyle=5,xmargin=[10,0],ymargin=[0,0]
           axis,0.45,ax_yrange[0]/0.305,/nodata,yticklen=tkly,yaxis=0,ystyle=1,$
		    yrange=!y.crange*(img2order eq 0)+ $
              reverse(!y.crange)*(img2order eq 1),chars=1.1,/normal
	   endif
    endif

if openwf eq 0 then print,'wcrdisp in progress ...'
; .........................................................................

; Start the main loop

  ; Initilize

  openwf  =0B
  destroyw=0B
  destroym=0B
  catche  =0B
  dispc   =0B
  rgold   =rangegates
  rgsold  =rangegatespacing
  rgdold  =rangegatedelay
  img1    =bytarr(maximgsz,rangegates)
  img2    =bytarr(maximgsz,rangegates)

  LOOP1: starte=0B                ; restart main loop here if new InitTime
       kstrt   =long(0 + strtp)
       kstep=loopstep*avestep
     ; If start profile exceeds existing profiles zero it.  KSTRT is NOT
     ; initialized in order to allow widget openning and position at the end
     ; of the file waiting for data

       if strtp ge (numrec/kstep-2)*kstep-1 then strtp=0L

  LOOP2: ;restart main loop here if new data or display control action requested

       kstep=loopstep*avestep                                      
       kstop=(numrec/kstep-1)*kstep-1

       ; If kstrt > kstop in idl 'for' loop won't execute the body
       ; If kstrt+kstep > kstop skip the loop and wait for more data

       if (kstop lt kstrt+kstep) then begin     ; avoid insufficient data
          k=kstrt
          goto, CATCH
       endif

       widget_control,pauseid,sensitive=1       ; restore pause button
       widget_control,catchid,sensitive=1       ; restore catch button

  for k=kstrt,kstop,kstep do begin

  ;  ##########################################################################
  ;  Update time and current profiles being processed

     dispitime=wcrdisptime(data,k,strtp)
     widget_control,btlabelid,set_value=dispitime
     widget_control,statusmsgid,set_value='Processing profiles '+ $
             num2str(k)+' to '+num2str(k+kstep)

  ;  ##########################################################################
  ;  Read and calibrate current loopstep*avestep profiles of wcr data

     wcrdispread,k,calc,mnhn,mnvn

  ;  ##########################################################################
  ;  Scale requested image1 and image2

     case image1 of
        'hh'  : img1=bytscl(hhz,max=zzmax,min=zzmin,top=ct2ind-1)
        'vv'  : img1=bytscl(vvz,max=zzmax,min=zzmin,top=ct2ind-1)
        'dv_h': img1=bytscl(dvh,max=dvmax,min=dvmin,top=ct2ind-1)+ $
                     ct2ind*(ct2ind lt !d.table_size)
        'dv_v': img1=bytscl(dvv,max=dvmax,min=dvmin,top=ct2ind-1)+ $
                     ct2ind*(ct2ind lt !d.table_size)
        'sv'  : img1=bytscl(vhz,max=svmax,min=svmin,top=ct2ind-1)
        'zdr' : img1=bytscl(zdr,max=zdmax,min=zdmin,top=ct2ind- $
                     (ct2ind eq !d.table_size))+ct2ind*(ct2ind lt !d.table_size)
        'ldr' : img1=bytscl(ldr,max=ldmax,min=ldmin,top=ct2ind- $
                     (ct2ind eq !d.table_size))
        'vh'  : img1=bytscl(vhz,max=czmax,min=czmin,top=ct2ind-1)
     endcase

     case image2 of
        'hh'  : img2=bytscl(hhz,max=zzmax,min=zzmin,top=ct2ind-1)
        'vv'  : img2=bytscl(vvz,max=zzmax,min=zzmin,top=ct2ind-1)
        'dv_h': img2=bytscl(dvh,max=dvmax,min=dvmin,top=ct2ind-1)+ $
                     ct2ind*(ct2ind lt !d.table_size)
        'dv_v': img2=bytscl(dvv,max=dvmax,min=dvmin,top=ct2ind-1)+ $
                     ct2ind*(ct2ind lt !d.table_size)                     
        'sv'  : img2=bytscl(vhz,max=svmax,min=svmin,top=ct2ind-1)
        'zdr' : img2=bytscl(zdr,max=zdmax,min=zdmin,top=ct2ind- $
                     (ct2ind eq !d.table_size))+ct2ind*(ct2ind lt !d.table_size)
        'ldr' : img2=bytscl(ldr,max=ldmax,min=ldmin,top=ct2ind- $
                     (ct2ind eq !d.table_size))
        'vh'  : img2=bytscl(vhz,max=czmax,min=czmin,top=ct2ind-1)
        else  :
     endcase

  ;  ##########################################################################
  ;  Plot image1 and image2

     wset, scrollw1
     tv, img1,order=img1order
     if nimg eq 2 then begin
        wset, scrollw2
        tv, img2,order=img2order
     endif

  ;  ##########################################################################
  ;  Update end-time and NOISE/LEAK display
  ;  Internal calibration was disabled (ver. 7.2) and then restored(RCS v.3.0)
  ;  Currently status display is modified to display noise,leak and time 
  ;  interval. calc, mnhn,mxhn, mnvn,mxvn, timeint,timeintm are initialized with
  ;  a new file.

     dispetime=wcrdisptime(data,(k+kstep)<(kstop-1))
     widget_control,etlabelid,set_value=dispetime

     widget_control,stathleakid,set_value=strcompress(calc,/remove_all)
     widget_control,statvleakid,set_value=strcompress(vleak,/remove_all)
     widget_control,statmnhnid,set_value=strcompress(mnhn,/remove_all)
     widget_control,statmnvnid,set_value=strcompress(mnvn,/remove_all)
     if mnhn gt mxhn then begin
       mxhn=mnhn
       widget_control,statmxhnid,set_value=strcompress(mxhn,/remove_all)
     endif
     if mnvn gt mxvn then begin
       mxvn=mnvn
       widget_control,statmxvnid,set_value=strcompress(mxvn,/remove_all)
     endif
     if timeint gt timeintm then begin
       timeintm=timeint
       widget_control,statdtid,set_value=strcompress(timeintm,/remove_all)
     endif

  ;  ##########################################################################
  ;  Register coming events

     junkid=widget_event(baseid,/nowait)

  ;  ##########################################################################
  ;  If Pause is requested stall the images and wait for Continue button
  ;  Give control to the mouse for image cursor trace

     widget_control,pauseid,get_value=bl
     if bl eq 'Continue' then begin
        widget_control,statusmsgid,set_value='Last Prof:'+num2str(k+kstep)+ $
        '-> Pause ...'

     ;  Initiate mouse events

        widget_control,scroll1id,/draw_button_events,event_pro='scrollw1_ev'
        if nimg eq 2 then $
        widget_control,scroll2id,/draw_button_events,event_pro='scrollw2_ev'

     ;  Establish plot parameters for cursor marks

        plot,[0,0],/nodata,/noerase,yrange=[0,(rangegates-1)>(minimgsz-1)], $
             xrange=[0,maximgsz-1],xstyle=5,ystyle=5,xmargin=[0,0],ymargin=[0,0]

     ;  Wait for a mouse event from draw widgets or Continue button

        while bl eq 'Continue' do begin
           junkid=widget_event(baseid,/nowait)
           widget_control,pauseid,get_value=bl
           wait,0.1  ; sleep 100ms minimizes a memory leak problem under Linux
                     ; widget_control,..,get_value is the source (reported to
                     ; RSI, Nov 2001

           if scrollw1e ne 0 and ivalue lt rangegates then begin
              if img1order eq 1 then $
		       junkid=rangegates-ivalue-1 else junkid=ivalue

              case image1 of
                'hh'  : ivalue1=hhz(itime,junkid)
                'vv'  : ivalue1=vvz(itime,junkid)
                'dv_h': ivalue1=dvh(itime,junkid)
                'dv_v': ivalue1=dvv(itime,junkid)
                'zdr' : ivalue1=zdr(itime,junkid)
                'ldr' : ivalue1=ldr(itime,junkid)
                'vh'  : ivalue1=vhz(itime,junkid)
                'sv'  : ivalue1=vhz(itime,junkid)
              endcase
              ivaluec='Img1:'+num2str(ivalue1,fr=1)

           ;  Determine the color for the cross (black: color table upper half;
           ;  white: color table lower half) and plot it

              widget_control,b1maxid,get_value=temp & bmaxc=(float(temp))(0)
              widget_control,b1minid,get_value=temp & bminc=(float(temp))(0)
              if ivalue1 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
              else xcol=0B
              wset, scrollw1
              oplot,[itime],[ivalue], psym=1,thick=2.,color=xcol
              ; It appears that print or empty are not needed for idl5.5 but 
              ; is needed for the second image (nimg eq 2)
              ; The following line was replaced with empty command      
              ; PRINT,format='($,"")' ; this line solves an idl problem that
                                      ; causes oplot activity to be finished
                                      ; after next I/O operation (for single 
                                      ; image mode)
              ; empty

              ; If image2 is present mark matching location and get the value

              if nimg eq 2 and ivalue lt rangegates then begin

                 case image2 of
                   'hh'  : ivalue2=hhz(itime,junkid)
                   'vv'  : ivalue2=vvz(itime,junkid)
                   'dv_h': ivalue2=dvh(itime,junkid)
                   'dv_v': ivalue2=dvv(itime,junkid)
                   'zdr' : ivalue2=zdr(itime,junkid)
                   'ldr' : ivalue2=ldr(itime,junkid)
                   'vh'  : ivalue2=vhz(itime,junkid)
                   'sv'  : ivalue2=vhz(itime,junkid)
                 endcase
                 ivaluec=ivaluec+' Img2:'+num2str(ivalue2,fr=1)
                 widget_control,b2maxid,get_value=temp & bmaxc=(float(temp))(0)
                 widget_control,b2minid,get_value=temp & bminc=(float(temp))(0)
                 if ivalue2 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
                 else xcol=0B
                 wset, scrollw2
                 if img2order ne img1order then $
		         junkid=rangegates-ivalue-1 else junkid=ivalue
                 oplot,[itime],[junkid],psym=5,thick=2,col=xcol
                 ; The following line was repalced with empty command      
                 ; PRINT,format='($,"")' ; this line solves an idl problem that
                                         ; causes oplot activity to be finished
                                         ; after next I/O operation
                 empty
             endif

           ;  Get time in hh:mm:ss
           ;  To reduce the effect of the rounding error in timeinterval
           ;  use dispitime for 1st half of the image and dispetime for 2nd half

              if (itime gt maximgsz/2) then $
                 itime=timestr2sec(dispetime)-$
                       (maximgsz-1-itime)*avestep*timeinterval $
              else itime=timestr2sec(dispitime)+itime*avestep*timeinterval

           ;  Take care if itime crosses midnight

              if itime lt 0. then itime=itime+86400. $
              else if itime gt 86400. then itime=itime-86400.
              itime=sec2timestr(itime)

           ;  Put the information on the status line

              widget_control,statusmsgid,set_value=ivaluec+'; T/R: '+itime+'/'+$
                 num2str((junkid*rangegatespacing+rangegatedelay)/1000.,fr=2)
              scrollw1e=0B

           endif else if scrollw2e and ivalue lt rangegates then begin
           ; mouse request from 2nd image
              if img2order eq 1 then $
		      junkid=rangegates-ivalue-1 else junkid=ivalue

              case image1 of
                'hh'  : ivalue1=hhz(itime,junkid)
                'vv'  : ivalue1=vvz(itime,junkid)
                'dv_h': ivalue1=dvh(itime,junkid)
                'dv_v': ivalue1=dvv(itime,junkid)
                'zdr' : ivalue1=zdr(itime,junkid)
                'ldr' : ivalue1=ldr(itime,junkid)
                'vh'  : ivalue1=vhz(itime,junkid)
                'sv'  : ivalue1=vhz(itime,junkid)
              endcase
              case image2 of
                'hh'  : ivalue2=hhz(itime,junkid)
                'vv'  : ivalue2=vvz(itime,junkid)
                'dv_h': ivalue2=dvh(itime,junkid)
                'dv_v': ivalue2=dvv(itime,junkid)
                'zdr' : ivalue2=zdr(itime,junkid)
                'ldr' : ivalue2=ldr(itime,junkid)
                'vh'  : ivalue2=vhz(itime,junkid)
                'sv'  : ivalue2=vhz(itime,junkid)
              endcase
              ivaluec='Img1:'+num2str(ivalue1,fr=1)+ $
                     ' Img2:'+num2str(ivalue2,fr=1)
              widget_control,b2maxid,get_value=temp & bmaxc=(float(temp))[0]
              widget_control,b2minid,get_value=temp & bminc=(float(temp))[0]
              if ivalue2 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
              else xcol=0B
              wset, scrollw2
              oplot,[itime],[ivalue],psym=1,thick=2.,color=xcol
              widget_control,b1maxid,get_value=temp & bmaxc=(float(temp))[0]
              widget_control,b1minid,get_value=temp & bminc=(float(temp))[0]
              if ivalue1 lt bminc+(bmaxc-bminc)/2. then xcol=255B $
              else xcol=0B
              wset, scrollw1
              if img2order ne img1order then $
		      junkid=rangegates-ivalue-1 else junkid=ivalue
              oplot,[itime],[junkid],psym=5,thick=2.,color=xcol
              PRINT,format='($,"")'

              if (itime gt maximgsz/2) then $
                 itime=timestr2sec(dispetime)- $
                               (maximgsz-1-itime)*avestep*timeinterval $
              else itime=timestr2sec(dispitime)+itime*avestep*timeinterval
              if itime lt 0. then itime=itime+86400. $
              else if itime gt 86400. then itime=itime-86400.
              itime=sec2timestr(itime)

              widget_control,statusmsgid,set_value=ivaluec+'; T/R: '+itime+'/'+$
                 num2str((junkid*rangegatespacing+rangegatedelay)/1000.,fr=2)
              scrollw2e=0B
           endif
        endwhile
     endif

  ;  ##########################################################################
  ;  Check for events from the main buttons

     CHECKEVENT1: check_ev,fn,data,fsize,strtp

     ; If cancel from pickfile just continue

     if destroym eq 2B then begin openr,lun1,fn & destroym=0B

     ; Process events from main widget

     endif else if destroyw eq 1B or quitw eq 1B then return $
     else if openwf eq 1B then goto, NEWFILE $
     else if catche eq 1B then goto, CATCH $
     else if starte eq 1B then goto, LOOP1 $

     ; If a display control parameter is changed restart the main loop

     else if dispc eq 1B then begin
        kstrt=k+kstep>0L & dispc=0B & goto, LOOP2

     ; If waiting for data give control to main widget

     endif else if destroym eq 1B then begin
        junkid=widget_event(baseid,/nowait) & goto,CHECKEVENT1 & end

  endfor  ; end of main LOOP
; .........................................................................

  ; Save data pointer for possible next portion of data. If CATCH is requested
  ; go to the end of the currently available data. Note that in IDL the loop
  ; index variable after the end of the loop is equal to kstrt+kstep

CATCH:  if catche eq 1B then kstrt=numrec else kstrt = k > 0L
        fsizeold=fsize
        destroyw=0B
        catche=0B
        widget_control,catchid,sensitive=0

  ; Check for new data and process events from appropriate main buttons

    close,lun1
    openr,lun1,fn
    fstatus = fstat(lun1)
    fsize   = fstatus.size

    if fsize le fsizeold+minfsize then begin

    ;  No new data, wait sleept secs

       widget_control,pauseid,sensitive=0
       close,lun1
       widget_control,statusmsgid,set_value='End of file.  Waiting for data ...'
       wait,sleept

    ;  Check for new data again

       NEWDATASTATUS:
         openr,lun1,fn
         fstatus = fstat(lun1)
         fsize   = fstatus.size
         close,lun1

         ; If no new data check for other events

         if fsize le fsizeold+minfsize then begin
         CHECKEVENT2: junkid=widget_event(baseid,/nowait)
            check_ev,fn,data,fsize,strtp
            if quitw eq 1B or destroyw eq 1B then return $
            else if openwf eq 1B then goto, NEWFILE $
            else if destroym eq 1B then goto, CHECKEVENT2 $
            else if destroym eq 2B then destroym=0B $
            else if starte eq 1B then begin openr,lun1,fn & goto, LOOP1 & end
            wait,sleept & goto,NEWDATASTATUS
         endif else openr,lun1,fn
    endif

    ; If new data exist go get them

    numrec=long(strtrim(fsize,2))/blocksize
    goto, LOOP2

end ; wcrdisp
