;$Id
;+
; NAME:
;       WCRPROC_SH
;
; PURPOSE:
;       wcrproc_s routine similar to wcrproc script for non-batch processing
;       (this routine is not intened for general use)
;
; CATEGORY:
;       WCR wcrtools
;
; CALLING SEQUENCE:
;       WCRPROC_SH,crmax,crmean,atten
;
; INPUTS:
;       Some input parameters have to be adjusted by editing the routine:
;       d - date, hdisk - data disk, expname - experiment title, etc.
;
; OUTPUTS:
;       depend on the processing modes (proccal,procimgz,proctstz,procimgv)
;-
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrproc_sh,crmax,crmean,atten

; IDL processing sequence

; If process the current date

  d=strlowcase(delchr(strmid(systime(),4,6),' '))
  if strlen(d) eq 4 then d=strmid(d,0,3)+'0'+strmid(d,3,1)

; Define d if you process a day different than the current

  d='/'         ; use for processing in the main expdir directory

  d='/jan09_calib_sideH/'

; WCR data disk (no end slash)

  hdisk='/sova4'

; Experiment name and main storage directory

  expname  ="ROLLS 2004"

  expdir   ='/rolls04'      ; ''

  datdir  = hdisk+expdir+d

  kadir   = '/net/bobcat/disk1/kingair_data/
  kancf   = '20001119.c25.nc'

  psdir   = datdir+'ps/'
  pdfdir  = datdir+'pdf'

;------------------------------------------------------------------------------
; Antenna(s) configuration
;------------------------------------------------------------------------------

  aconfig = 0 ;  0 - single side/up looking antenna, dual pol.; default
;                1 - two downlooking antennas (H-ch: down-fore, V-ch: down) 
;                2 - two sidelooking antennas (V-ch: side-fore, H-ch: side/up)
;                3 - down and side/up antennas (V-ch: down, H-ch: side/up) 
;                4 - two downlooking antennas (V-ch: down, H-ch: down-aft,
;                    this is for C130 and represents H/V swapped receivers after
;                    march 2002)

; For different antenna configs during one flight make aconfig an array with
; relevant values

; aconfig=[0,0,0,0,0, ...]   ; n_elements must be equal to WCR files    ;/W
                             ; and values has to match ant config for
                             ; each file in order determined by ls -l command

;------------------------------------------------------------------------------
; What to process
;------------------------------------------------------------------------------

  procimgz =0    ; 1 - reflectivity quick looks (no profile averaging, slow)
                 ; 2 - quick looks (average round(aspectratio) profs, fast) 
  proctstz =0    ; 1 - test data
  procimgv =0    ; 1 - corrected velocity quick looks 

  procpdf  =procimgz ; 1 - convert quick looks ps files to pdf files

;------------------------------------------------------------------------------
; Assign appropriate values to some input parameters
;------------------------------------------------------------------------------

  printer = '' ;  '' - skip printing

  ;calcoef = [48.7,48.3]  ; WYICE 00 (Oct-Nov)
  ;calcoef = [50.1,50.4]  ; DYCOMS01
  ;calcoef = [54.2,49.0]  ; HBDD01
  ;calcoef = [49.3,47.8]  ; CLEX01
  ;calcoef1 = [55.0,50.0] ; IHOP02, H: down-fore, V: down
  ;calcoef3 = [49.0,50.0] ; IHOP02, H: side/up, V: down
  ;calcoef1 = [55.0,50.0] ; HICU02, H: down-fore, V: down
  ;calcoef2 = [49.0,55.0] ; HICU02, H: side/up, V: side-slant
  ;calcoef3 = [49.0,50.0] ; HICU02, H: side/up, V: down
  ;calcoef1 = [53.0,48.0] ; HICU03, H: down-fore, V: down
  ;calcoef2 = [48.0,54.0] ; HICU03, H: side/up, V: side-slant
  ;calcoef3 = [48.0,48.0] ; HICU03, H: side/up, V: down

; Cal coefficients for h-beam and v-beam at 250ns; corrections for 100/500 ns
; (+2.5/-3.5) is applied in wcrzpsm

  calcoef0 = [49.0,49.0]
  calcoef3 = [49.0,52.0] ;  H: side, V: down, 250ns            ;/E

  ; For different antenna configs during one flight make calcoeff an array
  ; matching aconfig

  calcoef=fltarr(2,n_elements(aconfig))
  for k=0,n_elements(aconfig)-1 do   $
    case aconfig[k] of             & $
    0: calcoef[*,k]=calcoef0     & $
    3: calcoef[*,k]=calcoef3     & $
    else: message,'Invalid antenna configuration.'     & $  
  endcase

  ; Image Processing parameters

  fileid  = '200'
  filelen = 20

  procz   = 1              ; 0 -  quick looks for spp,ppm,fft,dpfft (HH or 
                           ;      VV if only V is transmitted)
                           ; 1 -  quick looks for ppmag HH and VV
                           ; 2 -  quick looks for ppmag or sppmag HH only
                           ; 3 -  quick looks for ppmag or sppmag VV only

  procv   = 0              ; 0 - velocity quick looks for H-ch (if absent use V)
                           ; 1 - velocity quick looks for V-ch (if absent use H)

  mingates= 70             ; minimum number of range gates to be plotted
  minlimz = -45.           ; plot min reflectivity in dBZ
  maxlimz = +35.           ; plot max reflectivity in dBZ
  hcor    = 0              ; 0 - no H-ch slope correction 
  vcor    = 0              ; 1 - no V-ch slope correction
  sm      = 0              ; 0 - no curve smoothing in the plots
  noiso   = 0              ; 0 - plot cross-channel data
  noleak  = 0              ; 0 - plot leakage data
  nonoise = 0              ; 0 - plot noise data 
  norg    = 0              ; 0 - plot rangegate data 
  rangeg  = [0,1]          ; range gates to be plotted (default 0,1,last)
  nosaveps= 0              ; 0 - save ps files
  ppmnoav = 1              ; 1 - do not average the pulses for ppmag
  setnoise= [10099,2]      ; set noise use for wcrzpsm  (sppmag or ppmag)
                           ; for fft setnoise=0 (nswin=99, nthresh=[-999,3])
                           ; is hardwired
  colortb = -99            ; color table for reflectivity images 

  minlimv = -8; -32.           ; plot min velocity, m/s
  maxlimv = +8; +32.           ; plot max velocity, m/s
  timehack=  0.            ; velocity time hack, sec
  timecor = 0.;-13.+timehack  ; time correction (usually GPS leap secs)+timehack
  threshv = 5.             ; power threshold for velocity images
                           ; positive, n, means n*stdev above minimum noise
                           ; negative means absolute threshold in dBm
  colortbv = -99           ; color table for velocity images

;------------------------------------------------------------------------------
; Process  CR data
;------------------------------------------------------------------------------

  ; Calibration parameters (edit proccal and others if necessary)

  proccal  = 2              ; 1 - corner reflector calibration, no attenuation
                            ; 2 - use multiple calibrated attenuatiors 
  dtcr     = 1.14       ; us

  dtcrr   = dtcr*150.  ; m
  plotps  = 2          ; 2  - skip isolation plots for calibration processing

  ;caldir   = '/data2/'+expdir+'/calib'+d
  caldir   ='/home/haimov/data/projects/cifex04/mar22_sideup_calib_CR1.14us/'
  pscaldir = caldir+'ps/'

  calplotd = '22 March 2004'
  calfile  = 'cal040322.side'

  antH     ='Side, H-pol'
  antV     ='Side, V-pol'
  calexp  = 'UW Hanger' 
  calplotth = calexp+','+antH
  calplottv = calexp+','+antV

  calfiles = 19;

  if proccal gt 0 then begin

    calfiles=(proccal eq 1)+calfiles*(proccal eq 2)
    proccal=1

    if calfiles gt 1 then begin

      clabel  = strarr(calfiles)
      atten = fltarr(2,calfiles)  ; [0,*] - H atten, [1,*] - V-atten.

      atten[*,0]  = [ 0.00,0]
      atten[*,1]  = [ 3.00,0]
      atten[*,2]  = [ 5.82,0]
      atten[*,3]  = [10.00,0]
      atten[*,4]  = [15.00,0]
      atten[*,5]  = [20.00,0]
      atten[*,6]  = [25.00,0]
      atten[*,7]  = [30.00,0]
      atten[*,8]  = [35.00,0]
      atten[*,9]  = [40.00,0]
      atten[*,10] = [45.00,0]
      atten[*,11] = [50.00,0]
      atten[*,12] = [55.00,0]
      atten[*,13] = [60.00,0]
      atten[*,14] = [65.00,0]
      atten[*,15] = [70.82,0]
      atten[*,16] = [71.75,0]
      atten[*,17] = [73.00,0]
      atten[*,18] = [ 0.00,0]
      ;atten[*,19] = [ 3.00,0]
      ;atten[*,20] = [10.00,0]


      attentr=num2str(reform(atten[0,*]),fr=1)+'/'+ $
               num2str(reform(atten[1,*]),fr=1)+$
              replicate(' dB H/V attenuator',calfiles)
	 	 
      for k=0,calfiles-1 do $
        clabel[k]=calexp+'!CCR: dt='+num2str(dtcr,fr=2)+ $
                  'us ('+num2str(dtcrr,fr=0)+$
                  'm),LIFD non-linear lookup, '+attentr[k]
    endif else begin
      attentr ='0./0. dB H/V attenuator'
      clabel  = calexp+', dt='+num2str(dtcr,fr=2)+ $
                'us ('+num2str(dtcrr,fr=0)+ $
                'm), non-linear lookup, '+attentr
    endelse

    print,'caldir=',caldir
    print,'pscaldir=',pscaldir

    fpos=strpos(caldir,fileID,/reverse_search)
    if fpos ge 0 and strpos(caldir,'/',/reverse_search) ne strlen(caldir)-1 then $
    begin
     flst=strmid(caldir,fpos-1,fileLen)
     dir=strsub(caldir,0,fpos-2)
    endif else begin
     if strmid(caldir,strlen(caldir)-1,1) ne '/' then dir=caldir+'/' else dir=caldir
     spawn, 'ls '+dir+'?'+fileID+'*',flst
     flst=strmid(flst,strpos(flst(0),fileID,/reverse_search)-1,fileLen)
    endelse

    plotwcrcalm,caldir,pri=printer,fil=fileid,ind='all',tlabel=['cr',clabel],$
      psdir=pscaldir,plotps=plotps,nosaveps=nosaveps,crmax=crmax,crmean=crmean

    Hmaxc=(Hmnc=(Vmaxc=(Vmnc=fltarr(calfiles)-99.)))

;   For SPPMAG files (single antenna calib) swap crmax, etc. (comment next 2 lines)
;   crmax=reverse(crmax,1)  
;   crmean=reverse(crmean,1)

    indH=where(atten[0,*] ne 99.)
    indV=where(atten[1,*] ne 99.)

    wcrcalcoeff,replicate(dtcrr,calfiles),reform(crmax[0,*]+atten[0,*]),HmaxC 
;    wcrcalcoeff,replicate(dtcrr,calfiles),reform(crmax[1,*]+atten[1,*]),VmaxC

    wcrcalcoeff,replicate(dtcrr,calfiles),reform(crmean[0,*]+atten[0,*]),HmnC 
;    wcrcalcoeff,replicate(dtcrr,calfiles),reform(crmean[1,*]+atten[1,*]),VmnC

    if n_elements(indH) lt calfiles then begin
      HmaxC[where(atten[0,*] eq 99.)]=-99.
      HmnC[where(atten[0,*] eq 99.)]=-99.
    endif
    if n_elements(indV) lt calfiles then begin
      VmaxC[where(atten[1,*] eq 99.)]=-99.
      VmnC[where(atten[1,*] eq 99.)]=-99.
    endif

    setxpswin,99,2,psfile=calfile+'.ps'

    plot,atten[0,indH],HmaxC[indH],/ysty,/xsty,psym=2,xtit='Attenuator [dB]', $
      ytit='H-ch Calibration constant [dB]',$
       yrange=[min(HmaxC[indH])-1,max(HmaxC[indH])+1],$
      tit=calplotth+', H-Cal(max), '+calplotd,xrange=[-5,80]
    grid
    plot,atten[0,indH ],HmnC[indH],/ysty,/xsty,psym=5,xtit='Attenuator [dB]', $
      ytit='H-ch Calibration constant [dB]',$
      yrange=[min(HmnC[indH])-1,max(HmnC[indH])+1], $
      tit=calplotth+', H-Cal(mean), '+calplotd,xrange=[-5,80]
    grid
    datestamp
;    plot,atten[1,indV],VmaxC[indV],/ysty,/xsty,psym=4,xtit='Attenuator [dB]', $
;      ytit='V-ch Calibration constant [dB]',$
;      yrange=[min(VmaxC[indV])-1,max(VmaxC[indV])+1],$
;      tit=calplottv+', V-Cal(max), '+calplotd,xrange=[-5,80]
;    grid
;    plot,atten[1,indV],VmnC[indV],/ysty,/xsty,psym=6,xtit='Attenuator [dB]', $
;      ytit='V-ch Calibration constant [dB]',$
;      yrange=[min(VmnC[indV])-1,max(VmnC[indV])+1],$
;      tit=calplottv+', V-Cal(mean), '+calplotd,xrange=[-5,80]
;    grid
;    datestamp

    resetps2x
 
    i=sort(atten[0,*])
    Hpr        = strarr(6,calfiles+3)
    Hpr[*,0]   = ['      WCR File      ','    Atten. ','H-CRmax','H-CRmax','H-CRmean','H-CRmean']
    Hpr[*,1]   = ['                    ','      dB    ','  dBm  ','  dBZ  ','  dBm   ','  dBZ   ']
    Hpr[*,2]   = replicate(' ',6)
    Hpr[*,3:*] = [reform(flst[i],1,calfiles),num2str([atten[0,i],crmax[0,i], $
                  reform((HmaxC)[i],1,calfiles),crmean[0,i], $
                  reform((HmnC)[i],1,calfiles)],fr=1,len=8)]

    i=sort(atten[1,*])
    Vpr        = strarr(6,calfiles+3)
    Vpr[*,0]   = ['      WCR File      ','    Atten. ','V-CRmax','V-CRmax','V-CRmean','V-CRmean']
    Vpr[*,1]   = ['                    ','      dB    ','  dBm  ','  dBZ  ','  dBm   ','  dBZ   ']
    Vpr[*,2]   = replicate(' ',6)
    Vpr[*,3:*] = [reform(flst[i],1,calfiles),num2str([atten[1,i],crmax[1,i], $
                  reform((VmaxC)[i],1,calfiles),crmean[1,i], $
                  reform((VmnC)[i],1,calfiles)],fr=1,len=8)]

    get_lun, lun1
    openu,lun1,calfile+'.txt', /append

    printf,lun1,''
    printf,lun1,''
    printf,lun1,calexp+'('+antH+','+antV+', dt='+num2str(dtcr,fr=2)+'us/'+ $
                num2str(150.*dtcr,fr=0)+'m:'+' Calibration Data, '+calplotd
    printf,lun1,''
    printf,lun1,Hpr
    printf,lun1,''
    printf,lun1,Vpr
    close, lun1
    free_lun, lun1

  endif
;###############################################################################

;------------------------------------------------------------------------------
; Process data
;------------------------------------------------------------------------------

  ; Create quick look plots

  if procimgz gt 0 or procimgv gt 0 or proctstz gt 0 then begin
    print,'datdir=',datdir
    print,'psdir=',psdir
  endif

  if procimgz eq 1 then $
    wcrzpsm,datdir,printer=printer,tlabel=expname,timecor=timecor, $ 
      calcoef=calcoef,proc=procz,fileid=fileid,psdir=psdir,minlim=minlimz, $
      maxlim=maxlimz,hcor=hcor,vcor=vcor,setnoise=setnoise,mingates=mingates, $
      aconfig=aconfig,colortb=colortb $
  else if procimgz eq 2 then $ 
    wcrzpsm1,datdir,printer=printer,tlabel=expname,timecor=timecor, $ 
      calcoef=calcoef,proc=procz,fileid=fileid,psdir=psdir,minlim=minlimz, $
      maxlim=maxlimz,hcor=hcor,vcor=vcor,setnoise=setnoise,mingates=mingates, $
      aconfig=aconfig,colortb=colortb

  ; Flight test data

  if proctstz then $
    plotwcrtestm,datdir,printer=printer,tlabel=expname,plotps=plotps, $
      nosaveps=nosaveps,indjump=ijump,fileid=fileid,timecor=timecor, $
      noiso=noiso,noleak=noleak,nonoise=nonoise,rangeg=rangeg,norg=norg, $
      hcor=hcor,vcor=vcor,smooth=sm,ppmnoav=ppmnoav,psdir=psdir

  ; Corrected Doppler velocity quick looks

  if procimgv then begin  ; timehack is always 0 (it is added to timecor)
    print,kadir+kancf 
    wcrvpsm,datdir,printer=printer,tlabel=expname,timecor=timecor, $ 
      timehack=0,proc=procv,fileid=fileid,psdir=psdir,minlim=minlimv, $
      maxlim=maxlimv,mingates=mingates,thresh=threshv,aconfig=aconfig, $
      acfile=kadir+kancf,colortb=colortbv
  endif

  ; Convert quick looks ps files to pdf files

  if procpdf then begin
    spawn,'mkdir '+pdfdir+' >& /dev/null'
    spawn,'wcrps2pdf '+psdir+' '+pdfdir
  endif

  print,'End WCRPROC_S'
stop
end
