;$Id:
;+
; NAME:
;       WCRTFPPJFIX
;
; PURPOSE:
;       Fixes positive time jumps in the time stamps and rewrites the raw file
;       starting with 'T'. The corrupted raw file must be open and read 
;       before running this routine.  The problem is intermittent and appears
;       in some of the data collected during CIFEX in 2004.                
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRTFPPJFIX,wcrtime,wcrtnew,dt, writewcr=writewcr,help=help
;
; INPUTS:
;       Wcrtime:  dblarr(n), corrupted wcr time in UNIX seconds for n profs 
;
; KEYWORD PARAMETERS:
;       WRITEWCR:      when set, the program rewrites wcrfile with first
;                      char. of the name replaced with 'T'
;       HELP:          show this text
;
; OUTPUTS:
;       Wcrtnew:  dblarr(n), corrected wcr time in UNIX seconds
;       Dt:       dblarr(n-1), time interval between profiles from Wcrtnew
;
; COMMON BLOCKS:
;       wcrparams, wcrdata
;
; SIDE EFFECTS:
;
; NOTES:
;       
;-      
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Apr 2004

pro wcrtfppjfix,wcrtime,wcrtnew,dt, writewcr=writewcr,help=help

@commonwcrinclude

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrtfppjfix'
  return
endif

if n_params() lt 1 then begin
   message,'Usage: wcrtfppjfix,wcrtime,wcrtnew,dt,writewcr=writewcr',/info
   return
endif 

; Initilize

  leapsec=-13
  minjump= 5.
 
  wcrt=long(wcrtime)
  wcrtnew=wcrtime

; Check if 1st time stamps has messed up date

  if strmid(wcrfile,3,2) lt '90' then k='20'+strmid(wcrfile,3,2) $
  else k='19'+strmid(wcrfile,3,2)
  k=long(k+strmid(wcrfile,6,2)+strmid(wcrfile,9,2))

  if  wcrdate ne k then begin  ; fix wcrdate and find the first good time stamp
    wcrdate=k
    k1=1
    ; look in the next 4 sec for a correct time stamp; assume no additional
    ; jump happens during that period
    while abs(wcrt[k1]-wcrt[0]) lt minjump do k1=k1+1
    message,'Fixing date and 1st time stamp ...',/info
    message,'First correct time stamp is for profile:'+string(k1),/info
  endif else k1=0

; Correct unix time stamps, assuming no error in the fractional seconds

  ; Fix first time stamp
 
  if k1 gt 0 then begin
    junk1=wcrtime[0]  mod 1d0
    junk2=wcrtime[k1] mod 1d0 
    wcrtnew[0]=wcrtime[k1]-junk2+junk1+(junk2 lt junk1)
    wcrt[0]=long(wcrtnew[0])
  endif

  ; and now assuming that 1st stamp is correct fix jumps greater than 5 secs

  for k1=0L,numrec-2 do begin
    k=k1+1
    if wcrt[k]-wcrt[k1] gt long(minjump) then begin
      junk1=wcrtime[k1] mod 1d0 ;1440d0
      junk2=wcrtime[k]  mod 1d0 ;1440d0 
      wcrtnew[k]=wcrtnew[k1]-junk1+junk2+(junk2 lt junk1)
      wcrt[k]=long(wcrtnew[k])
    endif
  endfor

; Rewrite the raw file using the corrected time if requested.

  if n_elements(writewcr) gt 0 then begin

    message,'Reading data file '+wcrfile+'...',/info
    readwcrraw,filen=wcrfile,recraw,fileh=fileh

    recraw.header[1]=long(wcrtnew)-leapsec
    recraw.header[2]=long((wcrtnew mod 1d0)*1d6)
  
    fnout='T'+strmid(wcrfile,1)
    message,'Re-writing data file '+fnout+'...',/info
    writewcrraw,recraw,fileh,fnout=fnout
  endif
  
end
