;$Id: wcrthresh.pro,v 1.4 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRTHRESH
;
; PURPOSE:
;       Return range adjusted reflectivity threshold,fltarr(rangegates,profile),
;       given radar range gates in meters and reflectivity noise standard
;       deviation in equivalent dBZ at 1km
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       result=wcrthresh(snoise,radrng,sfactor)
;
; INPUTS:
;       Snoise:  fltarr(profiles), standard deviation of the reflectivity 
;                noise in equivalent dBZ at 1 km
;       Radrng:  fltarr(rangegates) radar range gates in meters 
;       Sfactor: factor applied to snoise to form the threshold (default 1)
;                (e.g., if sfactor=2, threshold is snoise+10*alog10(sfactor))
;
; KEYWORD PARAMETERS:
;       HELP:     show this text (use print,wcrthresh(/help) )
;
; OUTPUTS:
;
; SIDE EFFECTS:
;
; NOTES:
;
; EXAMPLES:
;
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, March 2002
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

function wcrthresh,snoise,radrng,sfactor,help=help

  on_error,2

  if keyword_set(help) then begin
    doc_library,'wcrthresh'
    return,''
  endif

  if n_params() lt 2 then begin
    message,"Usage: res=wcrthresh(snoise,radrng,sfactor)",/info
    return,''
  endif else if n_params() eq 2 then sfactor=1.

  ; Form range correction in dB/km

  rangecor=20.*alog10(radrng)-60.

  ; Return the threshold

  return, replicarr(snoise+10.*alog10(sfactor),n_elements(radrng),dim=1)+ $
          replicarr(rangecor,n_elements(snoise))

end
