;$Id: wcrtools.pro,v 1.11 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       WCRTOOLS
;
; PURPOSE:
;       Show WCR IDL routines with one line description
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       wcrtools
;
; KEYWORD PARAMETERS:
;       HELP:  Show this text
;-
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, Oct 1996
;       S.H., Nov 99 -- modified help info
;       S.H., Jan 00 -- added help keyword
;       S.H., Dec 00 -- added calcdopvel function to the list
;       S.H., Mar 01 -- changed routine name from showvxiprg.pro to wcrtools.pro
;       S.H., Mar 02 -- adding new entries
;       S.H., Feb 03 -- added new entries
;       S.H., Apr 03 -- reflect changes in some routine names
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrtools,help=help

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrtools'
  return
endif

print,"------------------ WCRTOOLS IDL routines ------------------------------"
print,""
print,"wcrtools   - this help; one line description of wcr IDL routines"
print,""
print,"**********************************************************************"
print,"**************** Routines for handling WCR raw data ******************" 
print,"**********************************************************************"
print,""
print,"clearwcr         - Initilize wcr common blocks: wcrparams and wcrdata"
print,"closewcrf        - Close WCR file."
print,"commonwcrinclude - Set wcr common blocks as include file in other"
print,"                   wcrtools routines"
print,"commonwcrset     - Set wcr common blocks in the current work space"
print,""
print,"openwcrf   - Open 'SPPMAG' , 'PPMAG' or 'FFT' WCR file"
print,"openwcrm   - Main program that set wcr common blocks and open a WCR file"
print,""
print,"plotwcrcalm  - Plot test data or corner reflector calibration data for"
print,"               selected files"
print,"plotwcrcalm1 - Make isolation plots (plotwcrcocr) only"
print,"plotwcrcocr  - Plot received power ratios: hh/vv, vh/hh, hv/vv"
print,"plotwcrcr    - Plot CR hh, vv, hv, vh at the range gate of max reflec." 
print,"plotwcrcrm   - Plot test data or corner reflector calibration data" 
print,"plotwcrfft   - Plot fftps at given range, ffttx and fftnoise."
print,"plotwcrfftm  - Plot fft data for test purposes "
print,"plotwcrnoise - Plot noise and leakage vars and calc. corr. coeffs." 
print,"plotwcrtest  - Plot noise, leakage, 0, 1 and last range gate data"
print,"plotwcrtestm - Plot noise, leakage, 0, 1 and last range gate data" 
print,"               for multiple files"
print,""
print,"printwcrprms - Print radar parameters for a raw wcr opened data file"
print,"printwcrstat - Print min, max, mean, std (in dBm) and varc=std/mean (%)"
print,"               for WCR noise, leak, and isolation of a given WCR file"
print,"printwcrstamp- Prints WCR file parameters and statistics at the bottom "
print,"               of a page. Prints datestamp at the upper left corner"
print,""
print,"readwcrall      - Load into the memory all radar data from a wcr file"
print,"                  (this routine is obsolete for data after 1996, use readwcrdata)" 
print,"readwcrdata     - Load into the memory all radar data from a wcr file"
print,"                  (faster than readwcrall, use more memmory)" 
print,"readwcrfftspec  - Read wcr fft data"
print,"                  (this routine is obsolete for data after 1996, use readwcrdata)"
print,"readwcrnoise    - Read wcr noise and cal. data"
print,"                  (this routine is obsolete for data after 1996, use readwcrdata)"
print,"readwcrpowerpp  - Read wcr radar power and pulse pair data"
print,"                  (this routine is obsolete for data after 1996, use readwcrdata)"
print,"readwcrraw      - Load array of structures with the raw WCR data"
print,"readwcrtb       - Read wcr time and profile recordings"
print,"                  (this routine is obsolete for data after 1996, use readwcrdata)"
print,""
print,"showwcrprms  - Show wcr parameters and variables"
print,""
print,"wcrcalcoeff  - Calculate calib. coeff. from corner reflector measurements."
print,"wcrcalcoref  - Calibrate co-reflectivity,hh,vv,abs(pp)."
print,"               Allow additional pre-calibration profile/range averaging."
print,"wcrcalcrref  - Calibrate cross-reflectivity,vh,hv."
print,"               Allow additional pre-calibration profile/range averaging."
print,"wcrcalinfo   - Show calibration constants for different experiments"
print,"               or returns cal. constants for a particular experiment."
print,""
print,"wcrdisp    - Widget controled WCR real-time display; this program is"
print,"             obsolete and replaced by wcrdisplay"
print,"wcrdisplay - Widget controled WCR data display. Similar to wcrdisp"
print,"             plus option to correct Doppler velocity for the"
print,"             platform motion. It could be used as WCR data browser."
print,"wcrdopvel  - Calclulate (uncorrected) Mean Doppler velocity"
print,"wcrdvsnr   - Calculate mean and standard deviation of the error in the"
print,"             measured WCR Doppler velocity as a function of SNR "
print,"wcrhcor    - Linear correction of H receiver data"
print,"             (WCR internal lookup table adjustment for data before 1999"
print,""
print,"wcrproc    - simple idl batch program for test/quick look processing"
print,"wcrproc_s  - subroutine version of wcrproc"
print,""
print,"wcrscope   - Create PS file with WCR transmitted power samples"
print,""
print,"wcrtfp1scor- Corrects a specific problem with the WCR Time (IRIG) card"
print,"             The problem appears as one or more 1 second negative jumps" 
print,"             at the top of a second followed by a single recovery "
print,"             positive jump. The problem is intermittent and appears in"
print,"             some of the WCR data collected in year 2002."
print,""
print,"wcrvcor    - Linear correction of V receiver data"
print,"             (WCR internal lookup table adjustment for data before 1999"
print,"wcrvps     - Corr. Doppler velocity PS file (3 plots/page max.)"
print,"wcrvpsm    - Corr. Doppler Velocity PS files from one day flight data"
print,""
print,"wcrzps     - Create a reflectivity PS file from a WCR file"
print,"wcrzpsm    - Create reflectivity PS files from one day flight wcr data"
print,"wcrzpsm1   - Create reflectivity PS files from one day flight wcr data"
print,"             (similar to wcrzpsm but first does incoherent profile"
print,"              averaging for SPPmag and PPMag modes only)"
print,""
print,"writewcrraw  - Write raw WCR wcr data to a new file" 
print,"wcrzoomprof  - Determine indexes of the wcr file profiles or time"
print,"wcrzoomrange - Return indices of the requested range gates"
print,""
print,"**********************************************************************"
print,"****** Routines for creating WCR NetCDF files from WCR raw data ******" 
print,"**********************************************************************"
print,""
print,"wcrcdl2nc  - Generate WCR netcdf file from a prototype cdl file (ncgen utility)"
print,"wcrcdv2nc  - Write corrected for platform motion, Doppler velocity and"  
print,"             related variables to the corresponding WCR netcdf file"             
print,"wcrcreatenc- Create a netcdf file from a given cdl file, WCR raw data"
print,"             file, and optional WCR platform motion (aircraft)"
print,"             navigational data file"
print,"wcrwrite1nc- Write WCR time and dimension related variables, calculated" 
print,"             reflectivity, uncorrected Doppler velocity and associated"
print,"             attributes to an existing netcdf file matching loaded WCR"
print,"             raw data"
print,"wcrwrite2nc- Calculate and write corrected for platform motion, Doppler" 
print,"             velocity and related variables to the corresponding WCR"
print,"             netcdf file"
print,"wcrupdatenc- Update reflectivity or Doppler velocity variables of WCR"
print,"             netcdf file(s) as well as some other associated variables"
print,"             and attributes"
print,""
print,"**********************************************************************"
print,"******** Reading variables from AC, WCR or WCR/AC NetCDF files *******" 
print,"**********************************************************************"
print,""
print,"acloadnc      - Returns AC data variable from an AC netCDF data file."
print,"acloadins     - Returns UW KingAir or NCAR C130 selected INS data "
print,"                variables from a netCDF file."
print,""
print,"kawcrmirror   - Returns time array for the KA radar wing mirror position" 
print,"                (associated with side/up pointing antenna; 0-side, 1-up)."
print,"kawcrmirrorprint - Create an ascii file containing times of radar mirror" 
print,"                position changes for a given flight."
print,"kawcrreform   - Returns a reformed WCR variable from a KA/WCR netcdf " 
print,"                file by megring 'profiles' and 'time' dimensions.
print,""
print,"wcrloadnc     - Load variables from a WCR netcdf file. This version of the" 
print,"                WCR netcdf file uses 'profile' dimension instead of 'time'" 
print,"                dimension. There is no time dimension for these type of WCR" 
print,"                data files and profile dimension represents non-regularly" 
print,"                sampled radar data in time. The time stamps matching the" 
print,"                profiles can be composed from the netcdf file variables "
print,"                base_time and time_offset."
print,""
print,"**********************************************************************"
print,"**** Routines associated with WCR Doppler correction for AC motion ***" 
print,"**********************************************************************"
print,""
print,"ac2wcr     - Interpolates an aircraft variable with the time dimension" 
print,"             to match WCR time dimension (profiles) or vice versa."   
print,"ac2gt      - Returns single precision AC (aircraft) to EARTH or"
print,"             EARTH to AC coordinate transformation matrix."
print,"wcracvel   - Returns AC velocity contribution w.r.t. ground into the WCR"
print,"             beam. Positive contribution means is away from the radar."
print,"wcracvelm  - Returns AC velocity contribution w.r.t. ground into the WCR" 
print,"             beam. This routine uses acloadins to load the appropriate"
print,"             data and then wcracvel to calculate the AC velocity "
print,"             contribution. Positive contribution means away from the" 
print,"             radar."
print,"wcracwind  - Returns aircraft (AC) measured wind contribution (fltarr(n)" 
print,"             in m/s) into the WCR beam. Positive contribution means away" 
print,"             from the radar."
print,"wcrbeam    - Returns WCR antenna beam coordinates in Earth or aircraft"
print,"             coordinate system."
print,"wcrbeamfind- Finds unit vector (pointing angles in aircraft,AC,coordinate"
print,"             system) of a wcr antenna beam from wcr surface returns"
print,"wcrdopcor  - Returns WCR radial Doppler velocity corrected for AC motion."
print,"             Positive velocity is toward the radar."
print,"wcrdopcorm - Returns WCR radial Doppler velocity corrected for AC motion." 
print,"             Positive velocity is toward the radar. This routine uses" 
print,"             wcrdopvel to calculate uncorrected Doppler velocity"
print,"             wcracvelm to calculate AC velocity contribution to the beam."  
print,"             The corrected Doppler velocity is calculated using" 
print,"             wcrdopcor routine."
print,"wcrtimeoff - Calculates time offset between WCR time stamps and N2UW "
print,"             KingAir data system time stamps.  It also returns the"
print,"             corresponding correlation coefficient."
print,"wcrwunfold - Returns shifted/unfolded WCR Doppler corrected for AC motion"
print,"             velocity around given wind value component into the WCR beam"
print,""
print,"**********************************************************************"
print,"******** Routines associated with WCR Dual-Doppler Processing ********" 
print,"**********************************************************************"
print,""
print,"wcrdd     - Performs dual-Doppler analysis using airborne WCR"
print,"            dual-beam data and aircraft(AC) supplemental data"
print,"wcrdddata - Prepare input for WCR dual-doppler analysis from a WCR raw" 
print,"            data file and an aircraft (AC) data file (netcdf, N2UW" 
print,"            KingAir or NCAR C130)."
print,"wcrddinit - Reads/creates dual-Doppler input parameters file and outputs"
print,"            data structures necessary to perform dual-Doppler analysis"
print,"wcrddecho - Prints the values of the parameters used for dual-Doppler"
print,"            analysis in a file. Statistics of selected aircraft(AC)"
print,"            attitude parameters and processing results are also printed."
print,""
print,"**********************************************************************"
print,"******** Miscelaneous routines handling WCR and AC NetCDF data ********" 
print,"**********************************************************************"
print,""
print,"wcrthresh - Return range adjusted reflectivity threshold,"
print,"            fltarr(rangegates,profile), given radar range gates in meters"
print,"            and reflectivity noise standard deviation in equivalent dBZ@1km"
print,""
print," ----------------------------------------------------------------------- "
end     
