;$Id: wcrzoomprof.pro,v 1.3 2003/12/23 22:15:43 haimov Exp $
;+
; NAME:
;       ZOOMPROF
;
; PURPOSE:
;       Determine indexes of the wcr file profiles or time
;
; CATEGORY:
;       wcrtools
;
; CALLING SEQUENCE:
;       WCRZOOMPROF, Zoom, Ind, Btime, Etime
;
; INPUTS:
;       Zoom:      [ readmode, str, stp ]
;                  readmode: 0  - str/stp, start/stop record numbers
;                            1  - str/stp, start/stop profile numbers
;                            2  - str/stp, start/stop UTC in hhmmss
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       Ind:       long array, record indices according to zoom
;       Btime:     string, begining UTC for start record/profile
;       Etime:     string, end UTC for stop record/profile
;       Help:      Show this text
;
; COMMON BLOCKS:
;       wcrdata 
;
; NOTES:
;       1) See READWCRTB.PRO for somewhat similar algorithm.  In READWCRTB
;          zoom is implemented by reading the appropriate segment from the
;          wcr file, while zoomind uses data in the memory 
;       2) WCRZOOMPROF requires that READWCRTB with timeMode = 2 or 3 
;          or READWCRDATA with timeMode = 2 or 4 run first
;-          
; MODIFICATION HISTORY:
;       Written by:  Samuel Haimov, November 1995
;       S.H., Aug 99 -- modified help info and other minor changes
;       S.H., Jan 00 -- uses timemode from the common block
;                       added help keyword
;       S.H., Feb 01 -- changed routine name from zoomprof.pro to wcrzoomprof
;
; Copyright (C) 1996-2003, Samuel Haimov, Dept. of Atmos. Sci., University of 
; Wyoming.  This software may be used, copied, or redistributed as long as it 
; is not sold and this copyright notice is reproduced on each copy made. This
; routine is provided as is without any expressed or implied warranties
; whatsoever.  Other limitations apply as described in the Readme file.
;

pro wcrzoomprof, zoom, ind, btime, etime, help=help

common wcrdata

on_error,2

if keyword_set(help) then begin
  doc_library,'wcrzoomprof'
  return
endif

nprof = n_elements(prof)-1

; Check the timemode; 1 - not allowed, 2 - time in hhmmss, 
;                     3 - time in seconds from 0, 4 - time in secs from midnight

if (timemode eq 3) then tstrs=timestr2sec(begtime) $
else if (timemode eq 4) then tstrs=time[0]

; Check the input

if (n_elements(zoom) ne 3) or (zoom(0) ne 0 and zoom(0) ne 1 and zoom(0) ne 2) $
then message, 'Wrong input !' 

if (zoom(0) eq 0) then $
   if (zoom(2)-zoom(1)+1 le 1 or zoom(1) lt 0 or zoom(2) gt nprof) then $
      message,'Wrong start/stop record number(s) or zoom(2)-zoom(1)<=1 !' $
else $

if (zoom(0) eq 1) then $ 
   if (zoom(1) lt prof(0) or zoom(2) gt prof(nprof)) then $ 
      message,'Incorrect start/stop profile number(s) !' $
else $

if (zoom(0) eq 2) then begin
   k=endtime
   k=long(strmid(k,0,2))*10000+long(strmid(k,3,2))*100+long(strmid(k,6,2))
   if (zoom(1) lt tstrs or zoom(2) gt k ) then $
      message,'Incorrect start/stop UTC !'
endif

; Process indices

if (zoom(0) eq 0) then $
   ind = lindgen(zoom(2)-zoom(1)+1)+zoom(1) $
else if (zoom(0) eq 1) then $
   ind = where(prof ge zoom(1) and prof le zoom(2)) $
else if (zoom(0) eq 2) then $
   if (timemode eq 2) then $
      ind = where(long(time) ge zoom(1) and long(time) le zoom(2)) $
   else if (timemode eq 3) then begin
      str = time2sec(zoom(1))
      stp = time2sec(zoom(2))
      ind = where(long(time) ge str-tstrs and long(time) le stp-tstrs)
   endif else if (timemode eq 4) then begin
      str = time2sec(zoom(1))
      stp = time2sec(zoom(2))
      ind = where(long(time) ge str and long(time) le stp)
   endif

; Process bTime, eTime

if (ind(0) eq 0) then $
   bTime = begTime $
else begin
   if (timemode eq 2) then $
      btime = time2timestr(floor(time(ind(0)))) $ 
   else if (timemode eq 3) then $
      btime = sec2timestr(tstrs+floor(time(ind(0)))) $
   else if (timemode eq 4) then btime=sec2timestr(floor(time(ind(0))))
endelse

nind = n_elements(ind)-1
if (nind eq nprof) then $
   eTime = endtime $
else begin
   if (timemode eq 2) then $
      etime = time2timestr(long(time(ind(nind)))) $ 
   else if (timemode eq 3) then $
      etime = sec2timestr(tstrs+long(time(ind(nind)))) $
   else if (timemode eq 4) then etime=sec2timestr(floor(time(ind(nind))))
endelse

end
